@@ -23,14 +23,23 @@ elif [ "${TOP_SRCDIR}" != "" ]; then
 	PREFIX=${TOP_SRCDIR}/
 fi
 
-if [ -z "$CONCURRENCY_LEVEL" ]; then
+if [ "${TOP_BUILDDIR}" != "" ]; then
+	BUILT_COCCI_FILES=$(ls ${TOP_BUILDDIR}/qa/coccinelle/*.cocci)
+else
+	BUILT_COCCI_FILES=""
+fi
+
+if ! command -v parallel > /dev/null; then
+	echo "Concurrency disabled, command 'parallel' not available"
+	CONCURRENCY_LEVEL=1
+elif [ -z "$CONCURRENCY_LEVEL" ]; then
+        echo "No concurrency"
 	CONCURRENCY_LEVEL=1
-	echo "No concurrency"
 else
 	echo "Using concurrency level $CONCURRENCY_LEVEL"
 fi
 
-for SMPL in ${PREFIX}qa/coccinelle/*.cocci; do
+for SMPL in ${PREFIX}qa/coccinelle/*.cocci ${BUILT_COCCI_FILES}; do
 	echo "Testing cocci file: $SMPL"
 	if command -v parallel >/dev/null; then
 		echo -n $LIST | parallel -d ' ' -j $CONCURRENCY_LEVEL spatch --very-quiet -sp_file $SMPL --undefined UNITTESTS $PREFIX{} || if [ -z "$NOT_TERMINAL" ]; then exit 1; fi
