@@ -21,8 +21,8 @@
  * \author Victor Julien <victor@inliniac.net>
  */
 
-#ifndef __ACTION_GLOBALS_H__
-#define __ACTION_GLOBALS_H__
+#ifndef SURICATA_ACTION_GLOBALS_H
+#define SURICATA_ACTION_GLOBALS_H
 
 /* Changing them as flags, so later we can have alerts
  * and drop simultaneously */
@@ -32,5 +32,19 @@
 #define ACTION_REJECT_DST   0x08
 #define ACTION_REJECT_BOTH  0x10
 #define ACTION_PASS         0x20
+#define ACTION_CONFIG       0x40
+#define ACTION_ACCEPT       0x80 /**< firewall 'accept' rule */
 
-#endif /* __ACTION_GLOBALS_H__ */
+#define ACTION_REJECT_ANY   (ACTION_REJECT|ACTION_REJECT_DST|ACTION_REJECT_BOTH)
+
+#define ACTION_DROP_REJECT (ACTION_REJECT_ANY | ACTION_DROP)
+
+enum ActionScope {
+    ACTION_SCOPE_AUTO = 0,
+    ACTION_SCOPE_PACKET, /**< apply action to packet */
+    ACTION_SCOPE_FLOW,   /**< apply drop/pass/accept action to flow */
+    ACTION_SCOPE_HOOK,   /**< apply action to current hook */
+    ACTION_SCOPE_TX,     /**< apply action to current tx */
+};
+
+#endif /* SURICATA_ACTION_GLOBALS_H */
