@@ -20,14 +20,13 @@
  *
  * \author Gurvinder Singh <gurvindersinghdahiya@gmail.com>
  *
- * alert syslog modeule header file
+ * alert syslog module header file
  *
  */
 
-#ifndef __ALERT_SYSLOG_H__
-#define __ALERT_SYSLOG_H__
+#ifndef SURICATA_ALERT_SYSLOG_H
+#define SURICATA_ALERT_SYSLOG_H
 
-void TmModuleAlertSyslogRegister(void);
-
-#endif /* __ALERT_SYSLOG_H__ */
+void AlertSyslogRegister(void);
 
+#endif /* SURICATA_ALERT_SYSLOG_H */
