@@ -15,14 +15,17 @@
  * 02110-1301, USA.
  */
 
-#include "stream-tcp-reassemble.h"
+/**
+ * \file
+ *
+ * Memory management functions for HTP in the application layer
+ */
 
 void HTPParseMemcap(void);
 void *HTPMalloc(size_t size);
 void *HTPCalloc(size_t n, size_t size);
 void *HTPRealloc(void *ptr, size_t orig_size, size_t size);
 void HTPFree(void *ptr, size_t size);
-void HTPDestroyMemcap(void);
 
 int HTPSetMemcap(uint64_t size);
 uint64_t HTPGetMemcap(void);
