@@ -16,12 +16,16 @@
  */
 
 /**
+ * \file
+ *
  * \author Victor Julien <victor@inliniac.net>
  * \author Anoop Saldanha <anoopsaldanha@gmail.com>
+ *
+ *  Application layer handling and protocols implementation
  */
 
-#ifndef __APP_LAYER_H__
-#define __APP_LAYER_H__
+#ifndef SURICATA_APP_LAYER_H
+#define SURICATA_APP_LAYER_H
 
 #include "threadvars.h"
 #include "decode.h"
@@ -29,9 +33,9 @@
 
 #include "stream-tcp-private.h"
 #include "stream-tcp-reassemble.h"
-#include "stream.h"
 
-#include "util-profiling.h"
+
+#include "rust.h"
 
 #define APP_LAYER_DATA_ALREADY_SENT_TO_APP_LAYER \
     (~STREAM_TOSERVER & ~STREAM_TOCLIENT)
@@ -41,11 +45,9 @@
 /**
  * \brief Handles reassembled tcp stream.
  */
-int AppLayerHandleTCPData(ThreadVars *tv, TcpReassemblyThreadCtx *ra_ctx,
-                          Packet *p, Flow *f,
-                          TcpSession *ssn, TcpStream *stream,
-                          uint8_t *data, uint32_t data_len,
-                          uint8_t flags);
+int AppLayerHandleTCPData(ThreadVars *tv, TcpReassemblyThreadCtx *ra_ctx, Packet *p, Flow *f,
+        TcpSession *ssn, TcpStream **stream, uint8_t *data, uint32_t data_len, uint8_t flags,
+        enum StreamUpdateDir dir);
 
 /**
  * \brief Handles an udp chunk.
@@ -61,7 +63,7 @@ int AppLayerHandleUdp(ThreadVars *tv, AppLayerThreadCtx *app_tctx,
  *
  * \param The internal protocol id.
  */
-AppProto AppLayerGetProtoByName(char *alproto_name);
+AppProto AppLayerGetProtoByName(const char *alproto_name);
 
 /**
  * \brief Given the internal protocol id, returns a string representation
@@ -100,10 +102,10 @@ int AppLayerDeSetup(void);
  * \retval Pointer to the newly create thread context, on success;
  *         NULL, on failure.
  */
-AppLayerThreadCtx *AppLayerGetCtxThread(ThreadVars *tv);
+AppLayerThreadCtx *AppLayerGetCtxThread(void);
 
 /**
- * \brief Destroys the context created by AppLayeGetCtxThread().
+ * \brief Destroys the context created by AppLayerGetCtxThread().
  *
  * \param tctx Pointer to the thread context to destroy.
  */
@@ -119,21 +121,15 @@ void AppLayerRegisterThreadCounters(ThreadVars *tv);
 
 void AppLayerProfilingResetInternal(AppLayerThreadCtx *app_tctx);
 
-static inline void AppLayerProfilingReset(AppLayerThreadCtx *app_tctx)
-{
-#ifdef PROFILING
-    AppLayerProfilingResetInternal(app_tctx);
-#endif
-}
-
 void AppLayerProfilingStoreInternal(AppLayerThreadCtx *app_tctx, Packet *p);
 
-static inline void AppLayerProfilingStore(AppLayerThreadCtx *app_tctx, Packet *p)
-{
 #ifdef PROFILING
-    AppLayerProfilingStoreInternal(app_tctx, p);
+#define AppLayerProfilingReset(app_tctx)    AppLayerProfilingResetInternal(app_tctx)
+#define AppLayerProfilingStore(app_tctx, p) AppLayerProfilingStoreInternal(app_tctx, p)
+#else
+#define AppLayerProfilingReset(app_tctx)
+#define AppLayerProfilingStore(app_tctx, p)
 #endif
-}
 
 void AppLayerRegisterGlobalCounters(void);
 
@@ -144,5 +140,19 @@ void AppLayerUnittestsRegister(void);
 #endif
 
 void AppLayerIncTxCounter(ThreadVars *tv, Flow *f, uint64_t step);
+void AppLayerIncGapErrorCounter(ThreadVars *tv, Flow *f);
+void AppLayerIncAllocErrorCounter(ThreadVars *tv, Flow *f);
+void AppLayerIncParserErrorCounter(ThreadVars *tv, Flow *f);
+void AppLayerIncInternalErrorCounter(ThreadVars *tv, Flow *f);
+
+static inline const uint8_t *StreamSliceGetData(const StreamSlice *stream_slice)
+{
+    return stream_slice->input;
+}
+
+static inline uint32_t StreamSliceGetDataLen(const StreamSlice *stream_slice)
+{
+    return stream_slice->input_len;
+}
 
 #endif
