@@ -22,11 +22,8 @@
  *
  */
 
-#ifndef __DECODE_ERSPAN_H__
-#define __DECODE_ERSPAN_H__
-
-#include "decode.h"
-#include "threadvars.h"
+#ifndef SURICATA_DECODE_ERSPAN_H
+#define SURICATA_DECODE_ERSPAN_H
 
 typedef struct ErspanHdr_ {
     uint16_t ver_vlan;
@@ -34,4 +31,5 @@ typedef struct ErspanHdr_ {
     uint32_t padding;
 } __attribute__((__packed__)) ErspanHdr;
 
-#endif /* __DECODE_ERSPAN_H__ */
+void DecodeERSPANConfig(void);
+#endif /* SURICATA_DECODE_ERSPAN_H */
