@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2013 Open Information Security Foundation
+/* Copyright (C) 2007-2021 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -36,6 +36,7 @@
 #include "decode-events.h"
 #include "decode-gre.h"
 
+#include "util-validate.h"
 #include "util-unittest.h"
 #include "util-debug.h"
 
@@ -43,10 +44,13 @@
  * \brief Function to decode GRE packets
  */
 
-int DecodeGRE(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p, uint8_t *pkt, uint32_t len, PacketQueue *pq)
+int DecodeGRE(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p, const uint8_t *pkt, uint32_t len)
 {
+    DEBUG_VALIDATE_BUG_ON(pkt == NULL);
+
     uint32_t header_len = GRE_HDR_LEN;
     GRESreHdr *gsre = NULL;
+    GREPPtPHd *gre_pptp_h = NULL;
 
     StatsIncr(tv, dtv->counter_gre);
 
@@ -54,19 +58,19 @@ int DecodeGRE(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p, uint8_t *pkt, ui
         ENGINE_SET_INVALID_EVENT(p, GRE_PKT_TOO_SMALL);
         return TM_ECODE_FAILED;
     }
-
-    p->greh = (GREHdr *)pkt;
-    if(p->greh == NULL)
+    if (!PacketIncreaseCheckLayers(p)) {
         return TM_ECODE_FAILED;
+    }
 
-    SCLogDebug("p %p pkt %p GRE protocol %04x Len: %d GRE version %x",
-        p, pkt, GRE_GET_PROTO(p->greh), len,GRE_GET_VERSION(p->greh));
+    GREHdr *greh = PacketSetGRE(p, pkt);
 
-    switch (GRE_GET_VERSION(p->greh))
-    {
+    SCLogDebug("p %p pkt %p GRE protocol %04x Len: %d GRE version %x", p, pkt, GRE_GET_PROTO(greh),
+            len, GRE_GET_VERSION(greh));
+
+    switch (GRE_GET_VERSION(greh)) {
         case GRE_VERSION_0:
 
-            /* GRE version 0 doenst support the fields below RFC 1701 */
+            /* GRE version 0 doesn't support the fields below RFC 1701 */
 
             /**
              * \todo We need to make sure this does not allow bypassing
@@ -75,25 +79,25 @@ int DecodeGRE(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p, uint8_t *pkt, ui
              *       further into it.
              */
 
-            if (GRE_FLAG_ISSET_RECUR(p->greh)) {
+            if (GRE_FLAG_ISSET_RECUR(greh)) {
                 ENGINE_SET_INVALID_EVENT(p, GRE_VERSION0_RECUR);
                 return TM_ECODE_OK;
             }
 
-            if (GREV1_FLAG_ISSET_FLAGS(p->greh))   {
+            if (GREV1_FLAG_ISSET_FLAGS(greh)) {
                 ENGINE_SET_INVALID_EVENT(p, GRE_VERSION0_FLAGS);
                 return TM_ECODE_OK;
             }
 
             /* Adjust header length based on content */
 
-            if (GRE_FLAG_ISSET_KY(p->greh))
+            if (GRE_FLAG_ISSET_KY(greh))
                 header_len += GRE_KEY_LEN;
 
-            if (GRE_FLAG_ISSET_SQ(p->greh))
+            if (GRE_FLAG_ISSET_SQ(greh))
                 header_len += GRE_SEQ_LEN;
 
-            if (GRE_FLAG_ISSET_CHKSUM(p->greh) || GRE_FLAG_ISSET_ROUTE(p->greh))
+            if (GRE_FLAG_ISSET_CHKSUM(greh) || GRE_FLAG_ISSET_ROUTE(greh))
                 header_len += GRE_CHKSUM_LEN + GRE_OFFSET_LEN;
 
             if (header_len > len)   {
@@ -101,8 +105,7 @@ int DecodeGRE(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p, uint8_t *pkt, ui
                 return TM_ECODE_OK;
             }
 
-            if (GRE_FLAG_ISSET_ROUTE(p->greh))
-            {
+            if (GRE_FLAG_ISSET_ROUTE(greh)) {
                 while (1)
                 {
                     if ((header_len + GRE_SRE_HDR_LEN) > len) {
@@ -130,7 +133,7 @@ int DecodeGRE(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p, uint8_t *pkt, ui
 
         case GRE_VERSION_1:
 
-            /* GRE version 1 doenst support the fields below RFC 1701 */
+            /* GRE version 1 doesn't support the fields below RFC 1701 */
 
             /**
              * \todo We need to make sure this does not allow bypassing
@@ -139,49 +142,51 @@ int DecodeGRE(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p, uint8_t *pkt, ui
              *       further into it.
              */
 
-            if (GRE_FLAG_ISSET_CHKSUM(p->greh))    {
+            if (GRE_FLAG_ISSET_CHKSUM(greh)) {
                 ENGINE_SET_INVALID_EVENT(p,GRE_VERSION1_CHKSUM);
                 return TM_ECODE_OK;
             }
 
-            if (GRE_FLAG_ISSET_ROUTE(p->greh)) {
+            if (GRE_FLAG_ISSET_ROUTE(greh)) {
                 ENGINE_SET_INVALID_EVENT(p,GRE_VERSION1_ROUTE);
                 return TM_ECODE_OK;
             }
 
-            if (GRE_FLAG_ISSET_SSR(p->greh))   {
+            if (GRE_FLAG_ISSET_SSR(greh)) {
                 ENGINE_SET_INVALID_EVENT(p,GRE_VERSION1_SSR);
                 return TM_ECODE_OK;
             }
 
-            if (GRE_FLAG_ISSET_RECUR(p->greh)) {
+            if (GRE_FLAG_ISSET_RECUR(greh)) {
                 ENGINE_SET_INVALID_EVENT(p,GRE_VERSION1_RECUR);
                 return TM_ECODE_OK;
             }
 
-            if (GREV1_FLAG_ISSET_FLAGS(p->greh))   {
+            if (GREV1_FLAG_ISSET_FLAGS(greh)) {
                 ENGINE_SET_INVALID_EVENT(p,GRE_VERSION1_FLAGS);
                 return TM_ECODE_OK;
             }
 
-            if (GRE_GET_PROTO(p->greh) != GRE_PROTO_PPP)  {
+            if (GRE_GET_PROTO(greh) != GRE_PROTO_PPP) {
                 ENGINE_SET_INVALID_EVENT(p,GRE_VERSION1_WRONG_PROTOCOL);
                 return TM_ECODE_OK;
             }
 
-            if (!(GRE_FLAG_ISSET_KY(p->greh))) {
+            if (!(GRE_FLAG_ISSET_KY(greh))) {
                 ENGINE_SET_INVALID_EVENT(p,GRE_VERSION1_NO_KEY);
                 return TM_ECODE_OK;
             }
 
             header_len += GRE_KEY_LEN;
+            /* key is set and proto == PPP */
+            gre_pptp_h = (GREPPtPHd *)pkt;
 
             /* Adjust header length based on content */
 
-            if (GRE_FLAG_ISSET_SQ(p->greh))
+            if (GRE_FLAG_ISSET_SQ(greh))
                 header_len += GRE_SEQ_LEN;
 
-            if (GREV1_FLAG_ISSET_ACK(p->greh))
+            if (GREV1_FLAG_ISSET_ACK(greh))
                 header_len += GREV1_ACK_LEN;
 
             if (header_len > len)   {
@@ -195,85 +200,89 @@ int DecodeGRE(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p, uint8_t *pkt, ui
             return TM_ECODE_OK;
     }
 
-    switch (GRE_GET_PROTO(p->greh))
-    {
+    switch (GRE_GET_PROTO(greh)) {
         case ETHERNET_TYPE_IP:
-            {
-                if (pq != NULL) {
-                    Packet *tp = PacketTunnelPktSetup(tv, dtv, p, pkt + header_len,
-                            len - header_len, DECODE_TUNNEL_IPV4, pq);
-                    if (tp != NULL) {
-                        PKT_SET_SRC(tp, PKT_SRC_DECODER_GRE);
-                        PacketEnqueue(pq,tp);
-                    }
-                }
-                break;
+        {
+            Packet *tp = PacketTunnelPktSetup(tv, dtv, p, pkt + header_len,
+                    len - header_len, DECODE_TUNNEL_IPV4);
+            if (tp != NULL) {
+                PKT_SET_SRC(tp, PKT_SRC_DECODER_GRE);
+                PacketEnqueueNoLock(&tv->decode_pq,tp);
             }
+            break;
+        }
 
         case GRE_PROTO_PPP:
-            {
-                if (pq != NULL) {
-                    Packet *tp = PacketTunnelPktSetup(tv, dtv, p, pkt + header_len,
-                            len - header_len, DECODE_TUNNEL_PPP, pq);
-                    if (tp != NULL) {
-                        PKT_SET_SRC(tp, PKT_SRC_DECODER_GRE);
-                        PacketEnqueue(pq,tp);
-                    }
-                }
-                break;
+        {
+            if (gre_pptp_h && !gre_pptp_h->payload_length)
+                return TM_ECODE_OK;
+
+            Packet *tp = PacketTunnelPktSetup(tv, dtv, p, pkt + header_len,
+                    len - header_len, DECODE_TUNNEL_PPP);
+            if (tp != NULL) {
+                PKT_SET_SRC(tp, PKT_SRC_DECODER_GRE);
+                PacketEnqueueNoLock(&tv->decode_pq,tp);
             }
+            break;
+        }
 
         case ETHERNET_TYPE_IPV6:
-            {
-                if (pq != NULL) {
-                    Packet *tp = PacketTunnelPktSetup(tv, dtv, p, pkt + header_len,
-                            len - header_len, DECODE_TUNNEL_IPV6, pq);
-                    if (tp != NULL) {
-                        PKT_SET_SRC(tp, PKT_SRC_DECODER_GRE);
-                        PacketEnqueue(pq,tp);
-                    }
-                }
-                break;
+        {
+            Packet *tp = PacketTunnelPktSetup(tv, dtv, p, pkt + header_len,
+                    len - header_len, DECODE_TUNNEL_IPV6);
+            if (tp != NULL) {
+                PKT_SET_SRC(tp, PKT_SRC_DECODER_GRE);
+                PacketEnqueueNoLock(&tv->decode_pq,tp);
             }
+            break;
+        }
 
         case ETHERNET_TYPE_VLAN:
-            {
-                if (pq != NULL) {
-                    Packet *tp = PacketTunnelPktSetup(tv, dtv, p, pkt + header_len,
-                            len - header_len, DECODE_TUNNEL_VLAN, pq);
-                    if (tp != NULL) {
-                        PKT_SET_SRC(tp, PKT_SRC_DECODER_GRE);
-                        PacketEnqueue(pq,tp);
-                    }
-                }
-                break;
+        {
+            Packet *tp = PacketTunnelPktSetup(tv, dtv, p, pkt + header_len,
+                    len - header_len, DECODE_TUNNEL_VLAN);
+            if (tp != NULL) {
+                PKT_SET_SRC(tp, PKT_SRC_DECODER_GRE);
+                PacketEnqueueNoLock(&tv->decode_pq,tp);
             }
+            break;
+        }
 
         case ETHERNET_TYPE_ERSPAN:
         {
-            if (pq != NULL) {
-                Packet *tp = PacketTunnelPktSetup(tv, dtv, p, pkt + header_len,
-                        len - header_len, DECODE_TUNNEL_ERSPAN, pq);
-                if (tp != NULL) {
-                    PKT_SET_SRC(tp, PKT_SRC_DECODER_GRE);
-                    PacketEnqueue(pq,tp);
-                }
+            // Determine if it's Type I or Type II based on the flags in the GRE header.
+            // Type I:  0|0|0|0|0|00000|000000000|00000
+            // Type II: 0|0|0|1|0|00000|000000000|00000
+            //                Seq
+            Packet *tp = PacketTunnelPktSetup(tv, dtv, p, pkt + header_len, len - header_len,
+                    GRE_FLAG_ISSET_SQ(greh) == 0 ? DECODE_TUNNEL_ERSPANI : DECODE_TUNNEL_ERSPANII);
+            if (tp != NULL) {
+                PKT_SET_SRC(tp, PKT_SRC_DECODER_GRE);
+                PacketEnqueueNoLock(&tv->decode_pq,tp);
             }
             break;
         }
 
         case ETHERNET_TYPE_BRIDGE:
-            {
-                if (pq != NULL) {
-                    Packet *tp = PacketTunnelPktSetup(tv, dtv, p, pkt + header_len,
-                            len - header_len, DECODE_TUNNEL_ETHERNET, pq);
-                    if (tp != NULL) {
-                        PKT_SET_SRC(tp, PKT_SRC_DECODER_GRE);
-                        PacketEnqueue(pq,tp);
-                    }
-                }
-                break;
+        {
+            Packet *tp = PacketTunnelPktSetup(tv, dtv, p, pkt + header_len,
+                    len - header_len, DECODE_TUNNEL_ETHERNET);
+            if (tp != NULL) {
+                PKT_SET_SRC(tp, PKT_SRC_DECODER_GRE);
+                PacketEnqueueNoLock(&tv->decode_pq,tp);
             }
+            break;
+        }
+
+        case ETHERNET_TYPE_ARP: {
+            Packet *tp = PacketTunnelPktSetup(
+                    tv, dtv, p, pkt + header_len, len - header_len, DECODE_TUNNEL_ARP);
+            if (tp != NULL) {
+                PKT_SET_SRC(tp, PKT_SRC_DECODER_GRE);
+                PacketEnqueueNoLock(&tv->decode_pq, tp);
+            }
+            break;
+        }
 
         default:
             return TM_ECODE_OK;
@@ -289,26 +298,20 @@ int DecodeGRE(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p, uint8_t *pkt, ui
 
 static int DecodeGREtest01 (void)
 {
-
     uint8_t raw_gre[] = { 0x00 ,0x6e ,0x62 };
     Packet *p = PacketGetFromAlloc();
-    if (unlikely(p == NULL))
-        return 0;
+    FAIL_IF_NULL(p);
     ThreadVars tv;
     DecodeThreadVars dtv;
 
     memset(&tv, 0, sizeof(ThreadVars));
     memset(&dtv, 0, sizeof(DecodeThreadVars));
 
-    DecodeGRE(&tv, &dtv, p, raw_gre, sizeof(raw_gre), NULL);
-
-    if(ENGINE_ISSET_EVENT(p,GRE_PKT_TOO_SMALL))  {
-        SCFree(p);
-        return 1;
-    }
+    DecodeGRE(&tv, &dtv, p, raw_gre, sizeof(raw_gre));
+    FAIL_IF_NOT(ENGINE_ISSET_EVENT(p, GRE_PKT_TOO_SMALL));
 
     SCFree(p);
-    return 0;
+    PASS;
 }
 
 /**
@@ -333,23 +336,18 @@ static int DecodeGREtest02 (void)
         0x00, 0x00, 0x29, 0x10, 0x00, 0x00, 0x00, 0x00,
         0x00, 0x00, 0x00 };
     Packet *p = PacketGetFromAlloc();
-    if (unlikely(p == NULL))
-        return 0;
+    FAIL_IF_NULL(p);
     ThreadVars tv;
     DecodeThreadVars dtv;
 
     memset(&tv, 0, sizeof(ThreadVars));
     memset(&dtv, 0, sizeof(DecodeThreadVars));
 
-    DecodeGRE(&tv, &dtv, p, raw_gre, sizeof(raw_gre), NULL);
-
-    if(ENGINE_ISSET_EVENT(p,GRE_WRONG_VERSION))  {
-        SCFree(p);
-        return 1;
-    }
+    DecodeGRE(&tv, &dtv, p, raw_gre, sizeof(raw_gre));
+    FAIL_IF_NOT(ENGINE_ISSET_EVENT(p, GRE_WRONG_VERSION));
 
     SCFree(p);
-    return 0;
+    PASS;
 }
 
 
@@ -375,24 +373,18 @@ static int DecodeGREtest03 (void)
         0x01, 0x00, 0x00, 0x29, 0x10, 0x00, 0x00, 0x00,
         0x00, 0x00, 0x00, 0x00 };
     Packet *p = PacketGetFromAlloc();
-    if (unlikely(p == NULL))
-        return 0;
+    FAIL_IF_NULL(p);
     ThreadVars tv;
     DecodeThreadVars dtv;
 
     memset(&tv, 0, sizeof(ThreadVars));
     memset(&dtv, 0, sizeof(DecodeThreadVars));
 
-    DecodeGRE(&tv, &dtv, p, raw_gre, sizeof(raw_gre), NULL);
-
-    if(p->greh == NULL) {
-        SCFree(p);
-        return 0;
-    }
-
+    DecodeGRE(&tv, &dtv, p, raw_gre, sizeof(raw_gre));
+    FAIL_IF_NOT(PacketIsGRE(p));
 
     SCFree(p);
-    return 1;
+    PASS;
 }
 #endif /* UNITTESTS */
 
