@@ -21,19 +21,12 @@
  * \author Eric Leblond <eric@regit.org>
  */
 
-#ifndef __DECODE_SCTP_H__
-#define __DECODE_SCTP_H__
+#ifndef SURICATA_DECODE_SCTP_H
+#define SURICATA_DECODE_SCTP_H
 
 /** size of the packet header without any chunk headers */
 #define SCTP_HEADER_LEN                       12
 
-/* XXX RAW* needs to be really 'raw', so no ntohs there */
-#define SCTP_GET_RAW_SRC_PORT(sctph)          ntohs((sctph)->sh_sport)
-#define SCTP_GET_RAW_DST_PORT(sctph)          ntohs((sctph)->sh_dport)
-
-#define SCTP_GET_SRC_PORT(p)                  SCTP_GET_RAW_SRC_PORT(p->sctph)
-#define SCTP_GET_DST_PORT(p)                  SCTP_GET_RAW_DST_PORT(p->sctph)
-
 typedef struct SCTPHdr_
 {
     uint16_t sh_sport;     /* source port */
@@ -42,10 +35,6 @@ typedef struct SCTPHdr_
     uint32_t sh_sum;       /* checksum, computed via crc32 */
 } __attribute__((__packed__)) SCTPHdr;
 
-#define CLEAR_SCTP_PACKET(p) { \
-    (p)->sctph = NULL; \
-} while (0)
-
 void DecodeSCTPRegisterTests(void);
 
-#endif /* __DECODE_SCTP_H__ */
+#endif /* SURICATA_DECODE_SCTP_H */
