@@ -21,8 +21,8 @@
  * \author Victor Julien <victor@inliniac.net>
  */
 
-#ifndef __DECODE_SLL_H__
-#define __DECODE_SLL_H__
+#ifndef SURICATA_DECODE_SLL_H
+#define SURICATA_DECODE_SLL_H
 
 #define SLL_HEADER_LEN                16
 
@@ -32,7 +32,6 @@ typedef struct SllHdr_ {
     uint16_t sll_halen;        /* link-layer address length */
     uint8_t sll_addr[8];       /* link-layer address */
     uint16_t sll_protocol;     /* protocol */
-} SllHdr;
-
-#endif /* __DECODE_SLL_H__ */
+} __attribute__((__packed__)) SllHdr;
 
+#endif /* SURICATA_DECODE_SLL_H */
