@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2013 Open Information Security Foundation
+/* Copyright (C) 2007-2022 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -23,65 +23,69 @@
  */
 
 #include "suricata-common.h"
-#include "queue.h"
-#include "suricata.h"
-#include "conf.h"
-#include "util-debug.h"
-#include "util-misc.h"
 #include "defrag-config.h"
-
-static SCRadixTree *defrag_tree = NULL;
-
-static int default_timeout = 0;
+#include "util-misc.h"
+#include "conf.h"
+#include "util-radix4-tree.h"
+#include "util-radix6-tree.h"
 
 static void DefragPolicyFreeUserData(void *data)
 {
     if (data != NULL)
         SCFree(data);
-
-    return;
 }
 
-static void DefragPolicyAddHostInfo(char *host_ip_range, uint64_t timeout)
+static SCRadix4Tree defrag4_tree = SC_RADIX4_TREE_INITIALIZER;
+static SCRadix6Tree defrag6_tree = SC_RADIX6_TREE_INITIALIZER;
+static SCRadix4Config defrag4_config = { DefragPolicyFreeUserData, NULL };
+static SCRadix6Config defrag6_config = { DefragPolicyFreeUserData, NULL };
+
+static int default_timeout = 0;
+
+static void DefragPolicyAddHostInfo(const char *host_ip_range, uint64_t timeout)
 {
     uint64_t *user_data = NULL;
 
     if ( (user_data = SCMalloc(sizeof(uint64_t))) == NULL) {
-        SCLogError(SC_ERR_FATAL, "Error allocating memory. Exiting");
-        exit(EXIT_FAILURE);
+        FatalError("Error allocating memory. Exiting");
     }
 
     *user_data = timeout;
 
     if (strchr(host_ip_range, ':') != NULL) {
         SCLogDebug("adding ipv6 host %s", host_ip_range);
-        if (SCRadixAddKeyIPV6String(host_ip_range, defrag_tree, (void *)user_data) == NULL) {
-            SCLogWarning(SC_ERR_INVALID_VALUE,
-                        "failed to add ipv6 host %s", host_ip_range);
+        if (!SCRadix6AddKeyIPV6String(
+                    &defrag6_tree, &defrag6_config, host_ip_range, (void *)user_data)) {
+            SCFree(user_data);
+            if (sc_errno != SC_EEXIST) {
+                SCLogWarning("failed to add ipv6 host %s", host_ip_range);
+            }
         }
     } else {
         SCLogDebug("adding ipv4 host %s", host_ip_range);
-        if (SCRadixAddKeyIPV4String(host_ip_range, defrag_tree, (void *)user_data) == NULL) {
-            SCLogWarning(SC_ERR_INVALID_VALUE,
-                        "failed to add ipv4 host %s", host_ip_range);
+        if (!SCRadix4AddKeyIPV4String(
+                    &defrag4_tree, &defrag4_config, host_ip_range, (void *)user_data)) {
+            if (sc_errno != SC_EEXIST) {
+                SCLogWarning("failed to add ipv4 host %s", host_ip_range);
+            }
         }
     }
 }
 
-static int DefragPolicyGetIPv4HostTimeout(uint8_t *ipv4_addr)
+static int DefragPolicyGetIPv4HostTimeout(const uint8_t *ipv4_addr)
 {
     void *user_data = NULL;
-    (void)SCRadixFindKeyIPV4BestMatch(ipv4_addr, defrag_tree, &user_data);
+    (void)SCRadix4TreeFindBestMatch(&defrag4_tree, ipv4_addr, &user_data);
     if (user_data == NULL)
         return -1;
 
     return *((int *)user_data);
 }
 
-static int DefragPolicyGetIPv6HostTimeout(uint8_t *ipv6_addr)
+static int DefragPolicyGetIPv6HostTimeout(const uint8_t *ipv6_addr)
 {
     void *user_data = NULL;
-    (void)SCRadixFindKeyIPV6BestMatch(ipv6_addr, defrag_tree, &user_data);
+    (void)SCRadix6TreeFindBestMatch(&defrag6_tree, ipv6_addr, &user_data);
     if (user_data == NULL)
         return -1;
 
@@ -92,10 +96,10 @@ int DefragPolicyGetHostTimeout(Packet *p)
 {
     int timeout = 0;
 
-    if (PKT_IS_IPV4(p))
-        timeout = DefragPolicyGetIPv4HostTimeout((uint8_t *)GET_IPV4_DST_ADDR_PTR(p));
-    else if (PKT_IS_IPV6(p))
-        timeout = DefragPolicyGetIPv6HostTimeout((uint8_t *)GET_IPV6_DST_ADDR(p));
+    if (PacketIsIPv4(p))
+        timeout = DefragPolicyGetIPv4HostTimeout((const uint8_t *)GET_IPV4_DST_ADDR_PTR(p));
+    else if (PacketIsIPv6(p))
+        timeout = DefragPolicyGetIPv6HostTimeout((const uint8_t *)GET_IPV6_DST_ADDR(p));
 
     if (timeout <= 0)
         timeout = default_timeout;
@@ -103,21 +107,21 @@ int DefragPolicyGetHostTimeout(Packet *p)
     return timeout;
 }
 
-static void DefragParseParameters(ConfNode *n)
+static void DefragParseParameters(SCConfNode *n)
 {
-    ConfNode *si;
+    SCConfNode *si;
     uint64_t timeout = 0;
 
     TAILQ_FOREACH(si, &n->head, next) {
         if (strcasecmp("timeout", si->name) == 0) {
             SCLogDebug("timeout value  %s", si->val);
             if (ParseSizeStringU64(si->val, &timeout) < 0) {
-                SCLogError(SC_ERR_SIZE_PARSE, "Error parsing timeout "
-                        "from conf file");
+                SCLogError("Error parsing timeout "
+                           "from conf file");
             }
         }
         if (strcasecmp("address", si->name) == 0) {
-            ConfNode *pval;
+            SCConfNode *pval;
             TAILQ_FOREACH(pval, &si->head, next) {
                 DefragPolicyAddHostInfo(pval->val, timeout);
             }
@@ -125,7 +129,7 @@ static void DefragParseParameters(ConfNode *n)
     }
 }
 
-void DefragSetDefaultTimeout(intmax_t timeout)
+void DefragSetDefaultTimeout(int timeout)
 {
     default_timeout = timeout;
     SCLogDebug("default timeout %d", default_timeout);
@@ -135,24 +139,17 @@ void DefragPolicyLoadFromConfig(void)
 {
     SCEnter();
 
-    defrag_tree = SCRadixCreateRadixTree(DefragPolicyFreeUserData, NULL);
-    if (defrag_tree == NULL) {
-        SCLogError(SC_ERR_MEM_ALLOC,
-            "Can't alloc memory for the defrag config tree.");
-        exit(EXIT_FAILURE);
-    }
-
-    ConfNode *server_config = ConfGetNode("defrag.host-config");
+    SCConfNode *server_config = SCConfGetNode("defrag.host-config");
     if (server_config == NULL) {
         SCLogDebug("failed to read host config");
         SCReturn;
     }
 
     SCLogDebug("configuring host config %p", server_config);
-    ConfNode *sc;
+    SCConfNode *sc;
 
     TAILQ_FOREACH(sc, &server_config->head, next) {
-        ConfNode *p = NULL;
+        SCConfNode *p = NULL;
 
         TAILQ_FOREACH(p, &sc->head, next) {
             SCLogDebug("parsing configuration for %s", p->name);
@@ -163,8 +160,6 @@ void DefragPolicyLoadFromConfig(void)
 
 void DefragTreeDestroy(void)
 {
-    if (defrag_tree != NULL) {
-        SCRadixReleaseRadixTree(defrag_tree);
-    }
-    defrag_tree = NULL;
+    SCRadix4TreeRelease(&defrag4_tree, &defrag4_config);
+    SCRadix6TreeRelease(&defrag6_tree, &defrag6_config);
 }
