@@ -1,4 +1,4 @@
-/* Copyright (C) 2015 Open Information Security Foundation
+/* Copyright (C) 2015-2022 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -15,19 +15,11 @@
  * 02110-1301, USA.
  */
 
-#ifndef __DETECT_BASE64_DECODE_H__
-#define __DETECT_BASE64_DECODE_H__
-
-#include "app-layer-template.h"
-
-typedef struct DetectBase64Decode_ {
-    uint32_t bytes;
-    uint32_t offset;
-    uint8_t relative;
-} DetectBase64Decode;
+#ifndef SURICATA_DETECT_BASE64_DECODE_H
+#define SURICATA_DETECT_BASE64_DECODE_H
 
 void DetectBase64DecodeRegister(void);
 int DetectBase64DecodeDoMatch(DetectEngineThreadCtx *, const Signature *,
-    const SigMatchData *, uint8_t *, uint32_t);
+    const SigMatchData *, const uint8_t *, uint32_t);
 
-#endif /* __DETECT_BASE64_DECODE_H__ */
+#endif /* SURICATA_DETECT_BASE64_DECODE_H */
