@@ -15,34 +15,9 @@
  * 02110-1301, USA.
  */
 
-/**
- * \file
- *
- * \author Anoop Saldanha <anoopsaldanha@gmail.com>
- */
-
-#ifndef __DETECT_DCE_IFACE_H__
-#define __DETECT_DCE_IFACE_H__
-
-#include "app-layer-dcerpc.h"
-
-typedef enum DetectDceIfaceOperators_ {
-    DETECT_DCE_IFACE_OP_NONE = 0,
-    DETECT_DCE_IFACE_OP_LT,
-    DETECT_DCE_IFACE_OP_GT,
-    DETECT_DCE_IFACE_OP_EQ,
-    DETECT_DCE_IFACE_OP_NE,
-} DetectDceIfaceOperators;
-
-typedef struct DetectDceIfaceData_ {
-    uint8_t uuid[16];
-    uint8_t op;
-    uint16_t version;
-    uint8_t any_frag;
-} DetectDceIfaceData;
+#ifndef SURICATA_DETECT_DCE_IFACE_H
+#define SURICATA_DETECT_DCE_IFACE_H
 
 void DetectDceIfaceRegister(void);
 
-DCERPCState *DetectDceGetState(AppProto alproto, void *alstate);
-
-#endif /* __DETECT_DCE_IFACE_H__ */
+#endif /* SURICATA_DETECT_DCE_IFACE_H */
