@@ -21,19 +21,16 @@
  * \author Victor Julien <victor@inliniac.net>
  */
 
-#ifndef __DETECT_ENGINE_ADDRESS_IPV4_H__
-#define __DETECT_ENGINE_ADDRESS_IPV4_H__
+#ifndef SURICATA_DETECT_ENGINE_ADDRESS_IPV4_H
+#define SURICATA_DETECT_ENGINE_ADDRESS_IPV4_H
 
 int DetectAddressCutNotIPv4(DetectAddress *, DetectAddress **);
 int DetectAddressCmpIPv4(DetectAddress *a, DetectAddress *b);
 
 int DetectAddressCutIPv4(DetectEngineCtx *, DetectAddress *,
                          DetectAddress *, DetectAddress **);
-int DetectAddressJoinIPv4(DetectEngineCtx *, DetectAddress *target,
-                          DetectAddress *source);
 int DetectAddressIsCompleteIPSpaceIPv4(DetectAddress *);
 
 void DetectAddressIPv4Tests(void);
 
-#endif /* __DETECT_ENGINE_ADDRESS_IPV4_H__ */
-
+#endif /* SURICATA_DETECT_ENGINE_ADDRESS_IPV4_H */
