@@ -49,7 +49,7 @@
  * \retval 1 If a < b.
  * \retval 0 Otherwise, i.e. a >= b.
  */
-int AddressIPv6Lt(Address *a, Address *b)
+int AddressIPv6Lt(const Address *a, const Address *b)
 {
     int i = 0;
 
@@ -87,7 +87,7 @@ int AddressIPv6LtU32(uint32_t *a, uint32_t *b)
  * \retval 1 If a > b.
  * \retval 0 Otherwise, i.e. a <= b.
  */
-int AddressIPv6Gt(Address *a, Address *b)
+int AddressIPv6Gt(const Address *a, const Address *b)
 {
     int i = 0;
 
@@ -125,7 +125,7 @@ int AddressIPv6GtU32(uint32_t *a, uint32_t *b)
  * \retval 1 If a == b.
  * \retval 0 Otherwise.
  */
-int AddressIPv6Eq(Address *a, Address *b)
+int AddressIPv6Eq(const Address *a, const Address *b)
 {
     int i = 0;
 
@@ -159,7 +159,7 @@ int AddressIPv6EqU32(uint32_t *a, uint32_t *b)
  * \retval 1 If a <= b.
  * \retval 0 Otherwise, i.e. a > b.
  */
-int AddressIPv6Le(Address *a, Address *b)
+int AddressIPv6Le(const Address *a, const Address *b)
 {
 
     if (AddressIPv6Eq(a, b) == 1)
@@ -191,7 +191,7 @@ int AddressIPv6LeU32(uint32_t *a, uint32_t *b)
  * \retval 1 If a >= b.
  * \retval 0 Otherwise, i.e. a < b.
  */
-int AddressIPv6Ge(Address *a, Address *b)
+int AddressIPv6Ge(const Address *a, const Address *b)
 {
 
     if (AddressIPv6Eq(a, b) == 1)
@@ -296,8 +296,6 @@ static void AddressCutIPv6CopySubOne(uint32_t *a, uint32_t *b)
     b[1] = htonl(b[1]);
     b[2] = htonl(b[2]);
     b[3] = htonl(b[3]);
-
-    return;
 }
 
 /**
@@ -334,8 +332,6 @@ static void AddressCutIPv6CopyAddOne(uint32_t *a, uint32_t *b)
     b[1] = htonl(b[1]);
     b[2] = htonl(b[2]);
     b[3] = htonl(b[3]);
-
-    return;
 }
 
 /**
@@ -352,8 +348,6 @@ static void AddressCutIPv6Copy(uint32_t *a, uint32_t *b)
     b[1] = htonl(a[1]);
     b[2] = htonl(a[2]);
     b[3] = htonl(a[3]);
-
-    return;
 }
 
 int DetectAddressCutIPv6(DetectEngineCtx *de_ctx, DetectAddress *a,
@@ -368,8 +362,6 @@ int DetectAddressCutIPv6(DetectEngineCtx *de_ctx, DetectAddress *a,
     uint32_t b_ip2[4] = { SCNtohl(b->ip2.addr_data32[0]), SCNtohl(b->ip2.addr_data32[1]),
                           SCNtohl(b->ip2.addr_data32[2]), SCNtohl(b->ip2.addr_data32[3]) };
 
-    DetectAddress *tmp = NULL;
-
     /* default to NULL */
     *c = NULL;
 
@@ -378,12 +370,6 @@ int DetectAddressCutIPv6(DetectEngineCtx *de_ctx, DetectAddress *a,
         goto error;
     }
 
-    /* get a place to temporary put sigs lists */
-    tmp = DetectAddressInit();
-    if (tmp == NULL)
-        goto error;
-    memset(tmp,0,sizeof(DetectAddress));
-
     /* we have 3 parts: [aaa[abab]bbb]
      * part a: a_ip1 <-> b_ip1 - 1
      * part b: b_ip1 <-> a_ip2
@@ -522,14 +508,9 @@ int DetectAddressCutIPv6(DetectEngineCtx *de_ctx, DetectAddress *a,
         }
     }
 
-    if (tmp != NULL)
-        DetectAddressFree(tmp);
-
     return 0;
 
 error:
-    if (tmp != NULL)
-        DetectAddressFree(tmp);
     return -1;
 }
 
@@ -769,44 +750,6 @@ error:
     return -1;
 }
 
-/**
- * \brief Extends a target address range if the the source address range is
- *        wider than the target address range on either sides.
- *
- *        Every address is a range, i.e. address->ip1....address->ip2.  For
- *        example 2000::-2010::
- *        if source->ip1 is smaller than target->ip1, it indicates that the
- *        source's left address limit is greater(range wise) than the target's
- *        left address limit, and hence we reassign the target's left address
- *        limit to source's left address limit.
- *        Similary if source->ip2 is greater than target->ip2, it indicates that
- *        the source's right address limit is greater(range wise) than the
- *        target's right address limit, and hence we reassign the target's right
- *        address limit to source's right address limit.
- *
- * \param de_ctx Pointer to the detection engine context.
- * \param target Pointer to the target DetectAddress instance that has to be
- *               updated.
- * \param source Pointer to the source DetectAddress instance that is used
- *               to decided whether we extend the target's address range.
- *
- * \retval  0 On success.
- * \retval -1 On failure.
- */
-int DetectAddressJoinIPv6(DetectEngineCtx *de_ctx, DetectAddress *target,
-                          DetectAddress *source)
-{
-    if (AddressIPv6Lt(&source->ip, &target->ip)) {
-        COPY_ADDRESS(&source->ip, &target->ip);
-    }
-
-    if (AddressIPv6Gt(&source->ip, &target->ip)) {
-        COPY_ADDRESS(&source->ip2, &target->ip2);
-    }
-
-    return 0;
-}
-
 
 /***************************************Unittests******************************/
 
@@ -1926,156 +1869,6 @@ static int AddressTestIPv6CutNot05(void)
     return 0;
 }
 
-static int AddressTestIPv6Join01(void)
-{
-    DetectAddress *source = DetectAddressInit();
-    DetectAddress *target = DetectAddressInit();
-    DetectAddress *temp = DetectAddressInit();
-    struct in6_addr in6;
-    int result = 1;
-
-    if (source == NULL || target == NULL || temp == NULL)
-        goto error;
-
-    /* case 1 */
-    if (inet_pton(AF_INET6, "2000::10", &in6) != 1)
-        goto error;
-    memcpy(&target->ip.address, in6.s6_addr, sizeof(in6.s6_addr));
-    if (inet_pton(AF_INET6, "2000::20", &in6) != 1)
-        goto error;
-    memcpy(&target->ip2.address, in6.s6_addr, sizeof(in6.s6_addr));
-
-    if (inet_pton(AF_INET6, "2000::1", &in6) != 1)
-        goto error;
-    memcpy(&source->ip.address, in6.s6_addr, sizeof(in6.s6_addr));
-    if (inet_pton(AF_INET6, "2000::20", &in6) != 1)
-        goto error;
-    memcpy(&source->ip2.address, in6.s6_addr, sizeof(in6.s6_addr));
-
-    result &= (DetectAddressJoinIPv6(NULL, target, source) == 0);
-    if (inet_pton(AF_INET6, "2000::1", &in6) != 1)
-        goto error;
-    memcpy(&temp->ip.address, in6.s6_addr, sizeof(in6.s6_addr));
-    if (inet_pton(AF_INET6, "2000::20", &in6) != 1)
-        goto error;
-    memcpy(&temp->ip2.address, in6.s6_addr, sizeof(in6.s6_addr));
-    result = (DetectAddressCmpIPv6(target, temp) == ADDRESS_EQ);
-
-    /* case 2 */
-    if (inet_pton(AF_INET6, "2000::1", &in6) != 1)
-        goto error;
-    memcpy(&target->ip.address, in6.s6_addr, sizeof(in6.s6_addr));
-    if (inet_pton(AF_INET6, "2000::20", &in6) != 1)
-        goto error;
-    memcpy(&target->ip2.address, in6.s6_addr, sizeof(in6.s6_addr));
-
-    if (inet_pton(AF_INET6, "2000::2", &in6) != 1)
-        goto error;
-    memcpy(&source->ip.address, in6.s6_addr, sizeof(in6.s6_addr));
-    if (inet_pton(AF_INET6, "2000::19", &in6) != 1)
-        goto error;
-    memcpy(&source->ip2.address, in6.s6_addr, sizeof(in6.s6_addr));
-
-    result &= (DetectAddressJoinIPv6(NULL, target, source) == 0);
-    if (inet_pton(AF_INET6, "2000::1", &in6) != 1)
-        goto error;
-    memcpy(&temp->ip2.address, in6.s6_addr, sizeof(in6.s6_addr));
-    if (inet_pton(AF_INET6, "2000::20", &in6) != 1)
-        goto error;
-    memcpy(&temp->ip2.address, in6.s6_addr, sizeof(in6.s6_addr));
-    result = (DetectAddressCmpIPv6(target, temp) == ADDRESS_EQ);
-
-    /* case 3 */
-    if (inet_pton(AF_INET6, "2000::1", &in6) != 1)
-        goto error;
-    memcpy(&target->ip.address, in6.s6_addr, sizeof(in6.s6_addr));
-    if (inet_pton(AF_INET6, "2000::15", &in6) != 1)
-        goto error;
-    memcpy(&target->ip2.address, in6.s6_addr, sizeof(in6.s6_addr));
-
-    if (inet_pton(AF_INET6, "2000::10", &in6) != 1)
-        goto error;
-    memcpy(&source->ip.address, in6.s6_addr, sizeof(in6.s6_addr));
-    if (inet_pton(AF_INET6, "2000::20", &in6) != 1)
-        goto error;
-    memcpy(&source->ip2.address, in6.s6_addr, sizeof(in6.s6_addr));
-
-    result &= (DetectAddressJoinIPv6(NULL, target, source) == 0);
-    if (inet_pton(AF_INET6, "2000::1", &in6) != 1)
-        goto error;
-    memcpy(&temp->ip.address, in6.s6_addr, sizeof(in6.s6_addr));
-    if (inet_pton(AF_INET6, "2000::20", &in6) != 1)
-        goto error;
-    memcpy(&temp->ip2.address, in6.s6_addr, sizeof(in6.s6_addr));
-    result = (DetectAddressCmpIPv6(target, temp) == ADDRESS_EQ);
-
-    /* case 4 */
-    if (inet_pton(AF_INET6, "2000::10", &in6) != 1)
-        goto error;
-    memcpy(&target->ip.address, in6.s6_addr, sizeof(in6.s6_addr));
-    if (inet_pton(AF_INET6, "2000::20", &in6) != 1)
-        goto error;
-    memcpy(&target->ip2.address, in6.s6_addr, sizeof(in6.s6_addr));
-
-    if (inet_pton(AF_INET6, "2000::1", &in6) != 1)
-        goto error;
-    memcpy(&source->ip.address, in6.s6_addr, sizeof(in6.s6_addr));
-    if (inet_pton(AF_INET6, "2000::20", &in6) != 1)
-        goto error;
-    memcpy(&source->ip2.address, in6.s6_addr, sizeof(in6.s6_addr));
-
-    result &= (DetectAddressJoinIPv6(NULL, target, source) == 0);
-    if (inet_pton(AF_INET6, "2000::1", &in6) != 1)
-        goto error;
-    memcpy(&temp->ip.address, in6.s6_addr, sizeof(in6.s6_addr));
-    if (inet_pton(AF_INET6, "2000::20", &in6) != 1)
-        goto error;
-    memcpy(&temp->ip2.address, in6.s6_addr, sizeof(in6.s6_addr));
-    result = (DetectAddressCmpIPv6(target, temp) == ADDRESS_EQ);
-
-    /* case 5 */
-    if (inet_pton(AF_INET6, "2000::1", &in6) != 1)
-        goto error;
-    memcpy(&target->ip.address, in6.s6_addr, sizeof(in6.s6_addr));
-    if (inet_pton(AF_INET6, "2000::20", &in6) != 1)
-        goto error;
-    memcpy(&target->ip2.address, in6.s6_addr, sizeof(in6.s6_addr));
-
-    if (inet_pton(AF_INET6, "2000::1", &in6) != 1)
-        goto error;
-    memcpy(&source->ip.address, in6.s6_addr, sizeof(in6.s6_addr));
-    if (inet_pton(AF_INET6, "2000::20", &in6) != 1)
-        goto error;
-    memcpy(&source->ip2.address, in6.s6_addr, sizeof(in6.s6_addr));
-
-    result &= (DetectAddressJoinIPv6(NULL, target, source) == 0);
-    if (inet_pton(AF_INET6, "2000::1", &in6) != 1)
-        goto error;
-    memcpy(&temp->ip.address, in6.s6_addr, sizeof(in6.s6_addr));
-    if (inet_pton(AF_INET6, "2000::20", &in6) != 1)
-        goto error;
-    memcpy(&temp->ip2.address, in6.s6_addr, sizeof(in6.s6_addr));
-    result = (DetectAddressCmpIPv6(target, temp) == ADDRESS_EQ);
-
-    if (source != NULL)
-        DetectAddressFree(source);
-    if (target != NULL)
-        DetectAddressFree(target);
-    if (temp != NULL)
-        DetectAddressFree(temp);
-    return result;
-
- error:
-    if (source != NULL)
-        DetectAddressFree(source);
-    if (target != NULL)
-        DetectAddressFree(target);
-    if (temp != NULL)
-        DetectAddressFree(temp);
-
-    return 0;
-}
-
 #endif /* UNITTESTS */
 
 void DetectAddressIPv6Tests(void)
@@ -2122,9 +1915,5 @@ void DetectAddressIPv6Tests(void)
     UtRegisterTest("AddressTestIPv6CutNot03", AddressTestIPv6CutNot03);
     UtRegisterTest("AddressTestIPv6CutNot04", AddressTestIPv6CutNot04);
     UtRegisterTest("AddressTestIPv6CutNot05", AddressTestIPv6CutNot05);
-
-    UtRegisterTest("AddressTestIPv6Join01", AddressTestIPv6Join01);
 #endif /* UNITTESTS */
-
-    return;
 }
