@@ -21,12 +21,12 @@
  * \author Victor Julien <victor@inliniac.net>
  */
 
-#ifndef __DETECT_ENGINE_DCEPAYLOAD_H__
-#define __DETECT_ENGINE_DCEPAYLOAD_H__
+#ifndef SURICATA_DETECT_ENGINE_DCEPAYLOAD_H
+#define SURICATA_DETECT_ENGINE_DCEPAYLOAD_H
 
-int DetectEngineInspectDcePayload(DetectEngineCtx *, DetectEngineThreadCtx *,
-        Signature *, Flow *, uint8_t, void *);
+// uint8_t DetectEngineInspectDcePayload(DetectEngineCtx *, DetectEngineThreadCtx *,
+//        const Signature *, Flow *, uint8_t, void *);
 
 void DcePayloadRegisterTests(void);
 
-#endif /* __DETECT_ENGINE_DCEPAYLOAD_H__ */
+#endif /* SURICATA_DETECT_ENGINE_DCEPAYLOAD_H */
