@@ -21,28 +21,41 @@
  * \author Victor Julien <victor@inliniac.net>
  */
 
-#ifndef __DETECT_PORT_H__
-#define __DETECT_PORT_H__
+#ifndef SURICATA_DETECT_ENGINE_PORT_H
+#define SURICATA_DETECT_ENGINE_PORT_H
+
+#include "interval-tree.h"
+#include "detect.h"
+
+typedef struct SCPortIntervalNode {
+    uint16_t port;  /* low port of a port range */
+    uint16_t port2; /* high port of a port range */
+    uint16_t max;   /* max value of the high port in the subtree rooted at this node */
+
+    struct SigGroupHead_ *sh; /* SGHs corresponding to this port */
+
+    IRB_ENTRY(SCPortIntervalNode) irb; /* parent entry of the interval tree */
+} SCPortIntervalNode;
+
+IRB_HEAD(PI, SCPortIntervalNode); /* head of the interval tree */
+IRB_PROTOTYPE(PI, SCPortIntervalNode, irb,
+        SCPortIntervalCompare); /* prototype definition of the interval tree */
 
 /* prototypes */
 int DetectPortParse(const DetectEngineCtx *, DetectPort **head, const char *str);
 
-DetectPort *DetectPortCopy(DetectEngineCtx *, DetectPort *);
 DetectPort *DetectPortCopySingle(DetectEngineCtx *, DetectPort *);
-int DetectPortInsertCopy(DetectEngineCtx *,DetectPort **, DetectPort *);
 int DetectPortInsert(DetectEngineCtx *,DetectPort **, DetectPort *);
 void DetectPortCleanupList (const DetectEngineCtx *de_ctx, DetectPort *head);
 
 DetectPort *DetectPortLookupGroup(DetectPort *dp, uint16_t port);
-DetectPort *DetectPortLookupInList(DetectPort *head, DetectPort *gr);
-
-int DetectPortJoin(DetectEngineCtx *,DetectPort *target, DetectPort *source);
 
 bool DetectPortListsAreEqual(DetectPort *list1, DetectPort *list2);
 
 void DetectPortPrint(DetectPort *);
 void DetectPortPrintList(DetectPort *head);
 int DetectPortCmp(DetectPort *, DetectPort *);
+DetectPort *DetectPortInit(void);
 void DetectPortFree(const DetectEngineCtx *de_ctx, DetectPort *);
 
 int DetectPortTestConfVars(void);
@@ -52,7 +65,8 @@ void DetectPortHashFree(DetectEngineCtx *de_ctx);
 int DetectPortHashAdd(DetectEngineCtx *de_ctx, DetectPort *dp);
 int DetectPortHashInit(DetectEngineCtx *de_ctx);
 
+#ifdef UNITTESTS
 void DetectPortTests(void);
+#endif
 
-#endif /* __DETECT_PORT_H__ */
-
+#endif /* SURICATA_DETECT_ENGINE_PORT_H */
