@@ -21,8 +21,8 @@
  * \author Pablo Rincon <pablo.rincon.crespo@gmail.com>
  */
 
-#ifndef __DETECT_FLOWINT_H__
-#define __DETECT_FLOWINT_H__
+#ifndef SURICATA_DETECT_FLOWINT_H
+#define SURICATA_DETECT_FLOWINT_H
 
 /** Flowint operations allowed */
 enum {
@@ -40,17 +40,16 @@ enum {
     FLOWINT_MODIFIER_GT,
     /** Checking if a var is set (keyword isset/notset)*/
     FLOWINT_MODIFIER_ISSET,
-    FLOWINT_MODIFIER_NOTSET,
+    FLOWINT_MODIFIER_ISNOTSET,
 
     FLOWINT_MODIFIER_UNKNOWN
 };
 
-/** The target can be a value, or another variable arleady declared */
+/** The target can be a value, or another variable already declared */
 enum {
     FLOWINT_TARGET_VAL,
     FLOWINT_TARGET_VAR,
     FLOWINT_TARGET_SELF,
-    FLOWINT_TARGET_UNKNOWN
 };
 
 /** If the target is another var, get the name and the idx */
@@ -64,7 +63,7 @@ typedef struct DetectFlowintData_ {
     * against the target */
     char *name;
     /* Internal id of the var */
-    uint16_t idx;
+    uint32_t idx;
 
     /* The modifier/operation/condition we are
     * going to execute */
@@ -82,5 +81,4 @@ typedef struct DetectFlowintData_ {
 /* prototypes */
 void DetectFlowintRegister (void);
 
-#endif /* __DETECT_FLOWINT_H__ */
-
+#endif /* SURICATA_DETECT_FLOWINT_H */
