@@ -21,12 +21,8 @@
  * \author Breno Silva <breno.silva@gmail.com>
  */
 
-#ifndef __DETECT_FRAGBITS_H__
-#define __DETECT_FRAGBITS_H__
-
-#include "decode-events.h"
-#include "decode-ipv4.h"
-#include "decode-tcp.h"
+#ifndef SURICATA_DETECT_FRAGBITS_H
+#define SURICATA_DETECT_FRAGBITS_H
 
 /**
  * \struct DetectFragBitsData_
@@ -49,10 +45,4 @@ typedef struct DetectFragBitsData_ {
 
 void DetectFragBitsRegister (void);
 
-/**
- * This function registers unit tests for FragBits
- */
-
-void FragBitsRegisterTests(void);
-
-#endif /*__DETECT_FRAGBITS_H__ */
+#endif /*SURICATA_DETECT_FRAGBITS_H */
