@@ -18,12 +18,12 @@
 /**
  * \file
  *
- * \author Anoop Saldanha <poonaatsoc@gmail.com>
+ * \author Anoop Saldanha <anoopsaldanha@gmail.com>
  */
 
-#ifndef __DETECT_HTTP_CLIENT_BODY_H__
-#define __DETECT_HTTP_CLIENT_BODY_H__
+#ifndef SURICATA_DETECT_HTTP_CLIENT_BODY_H
+#define SURICATA_DETECT_HTTP_CLIENT_BODY_H
 
 void DetectHttpClientBodyRegister(void);
 
-#endif /* __DETECT_HTTP_CLIENT_BODY_H__ */
+#endif /* SURICATA_DETECT_HTTP_CLIENT_BODY_H */
