@@ -21,10 +21,11 @@
  * \author Pablo Rincon <pablo.rincon.crespo@gmail.com>
  */
 
-#ifndef __DETECT_HTTP_HEADER_H__
-#define __DETECT_HTTP_HEADER_H__
+#ifndef SURICATA_DETECT_HTTP_HEADER_H
+#define SURICATA_DETECT_HTTP_HEADER_H
 
 void DetectHttpHeaderRegister(void);
-void DetectHttpRawHeaderRegister(void);
+void DetectHttpRequestHeaderRegister(void);
+void DetectHttpResponseHeaderRegister(void);
 
-#endif /* __DETECT_HTTP_HEADER_H__ */
+#endif /* SURICATA_DETECT_HTTP_HEADER_H */
