@@ -21,14 +21,10 @@
  * \author Brian Rectanus <brectanu@gmail.com>
  */
 
-#ifndef __DETECT_HTTP_METHOD_H__
-#define __DETECT_HTTP_METHOD_H__
+#ifndef SURICATA_DETECT_HTTP_METHOD_H
+#define SURICATA_DETECT_HTTP_METHOD_H
 
 /* prototypes */
 void DetectHttpMethodRegister(void);
-int DetectHttpMethodDoMatch(DetectEngineThreadCtx *, Signature *, SigMatch *,
-                            Flow *, uint8_t, void *);
-int DetectHttpMethodValidateRule(const Signature *s);
-
-#endif /* __DETECT_HTTP_METHOD_H__ */
 
+#endif /* SURICATA_DETECT_HTTP_METHOD_H */
