@@ -21,22 +21,10 @@
  * \author Gerardo Iglesias <iglesiasg@gmail.com>
  */
 
-#ifndef __DETECT_ITYPE_H__
-#define __DETECT_ITYPE_H__
-
-#define DETECT_ITYPE_EQ   0   /**< "equal" operator */
-#define DETECT_ITYPE_LT   1   /**< "less than" operator */
-#define DETECT_ITYPE_GT   2   /**< "greater than" operator */
-#define DETECT_ITYPE_RN   3   /**< "range" operator */
-
-typedef struct DetectITypeData_ {
-    uint8_t type1;
-    uint8_t type2;
-
-    uint8_t mode;
-} DetectITypeData;
+#ifndef SURICATA_DETECT_ITYPE_H
+#define SURICATA_DETECT_ITYPE_H
 
 /* prototypes */
 void DetectITypeRegister(void);
 
-#endif /* __DETECT_ITYPE_H__ */
+#endif /* SURICATA_DETECT_ITYPE_H */
