@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2013 Open Information Security Foundation
+/* Copyright (C) 2007-2022 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -21,45 +21,50 @@
  * \author Victor Julien <victor@inliniac.net>
  */
 
-#ifndef __DETECT_LUAJIT_H__
-#define __DETECT_LUAJIT_H__
+#ifndef SURICATA_DETECT_LUA_H
+#define SURICATA_DETECT_LUA_H
 
-#ifdef HAVE_LUA
+#include "util-lua.h"
+#include "util-lua-sandbox.h"
 
 typedef struct DetectLuaThreadData {
     lua_State *luastate;
     uint32_t flags;
-    int alproto;
 } DetectLuaThreadData;
 
-#define DETECT_LUAJIT_MAX_FLOWVARS  15
-#define DETECT_LUAJIT_MAX_FLOWINTS  15
+#define DETECT_LUA_MAX_FLOWVARS 15
+#define DETECT_LUA_MAX_FLOWINTS 15
+#define DETECT_LUA_MAX_BYTEVARS 15
+
+typedef struct DetectLuaDataBytevarEntry_ {
+    char *name;
+    uint32_t id;
+} DetectLuaDataBytevarEntry;
 
 typedef struct DetectLuaData {
     int thread_ctx_id;
     int negated;
     char *filename;
     uint32_t flags;
-    AppProto alproto;
     char *buffername; /* buffer name in case of a single buffer */
-    uint32_t flowint[DETECT_LUAJIT_MAX_FLOWINTS];
+    uint32_t flowint[DETECT_LUA_MAX_FLOWINTS];
     uint16_t flowints;
     uint16_t flowvars;
-    uint32_t flowvar[DETECT_LUAJIT_MAX_FLOWVARS];
-    uint32_t sid;
-    uint32_t rev;
-    uint32_t gid;
+    uint32_t flowvar[DETECT_LUA_MAX_FLOWVARS];
+    uint16_t bytevars;
+    DetectLuaDataBytevarEntry bytevar[DETECT_LUA_MAX_BYTEVARS];
+    uint64_t alloc_limit;
+    uint64_t instruction_limit;
+    int allow_restricted_functions;
 } DetectLuaData;
 
-#endif /* HAVE_LUA */
-
 /* prototypes */
 void DetectLuaRegister (void);
 int DetectLuaMatchBuffer(DetectEngineThreadCtx *det_ctx,
         const Signature *s, const SigMatchData *smd,
-        uint8_t *buffer, uint32_t buffer_len, uint32_t offset,
+        const uint8_t *buffer, uint32_t buffer_len, uint32_t offset,
         Flow *f);
 
-void DetectLuaPostSetup(Signature *s);
+void LuaDumpStack(lua_State *state, const char *prefix);
 
-#endif /* __DETECT_FILELUAJIT_H__ */
+#endif /* SURICATA_DETECT_LUA_H */
