@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2010 Open Information Security Foundation
+/* Copyright (C) 2007-2019 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -32,11 +32,13 @@
 #include "detect-parse.h"
 #include "detect-content.h"
 #include "detect-uricontent.h"
+#include "detect-byte.h"
 #include "detect-byte-extract.h"
 #include "detect-offset.h"
 
 #include "flow-var.h"
 
+#include "util-byte.h"
 #include "util-debug.h"
 
 static int DetectOffsetSetup(DetectEngineCtx *, Signature *, const char *);
@@ -45,63 +47,65 @@ void DetectOffsetRegister (void)
 {
     sigmatch_table[DETECT_OFFSET].name = "offset";
     sigmatch_table[DETECT_OFFSET].desc = "designate from which byte in the payload will be checked to find a match";
-    sigmatch_table[DETECT_OFFSET].url = DOC_URL DOC_VERSION "/rules/payload-keywords.html#offset";
-    sigmatch_table[DETECT_OFFSET].Match = NULL;
+    sigmatch_table[DETECT_OFFSET].url = "/rules/payload-keywords.html#offset";
     sigmatch_table[DETECT_OFFSET].Setup = DetectOffsetSetup;
-    sigmatch_table[DETECT_OFFSET].Free  = NULL;
-    sigmatch_table[DETECT_OFFSET].RegisterTests = NULL;
 }
 
 int DetectOffsetSetup (DetectEngineCtx *de_ctx, Signature *s, const char *offsetstr)
 {
     const char *str = offsetstr;
-    SigMatch *pm = NULL;
-    int ret = -1;
 
-    /* retrive the sm to apply the offset against */
-    pm = DetectGetLastSMFromLists(s, DETECT_CONTENT, -1);
+    /* retrieve the sm to apply the offset against */
+    SigMatch *pm = DetectGetLastSMFromLists(s, DETECT_CONTENT, -1);
     if (pm == NULL) {
-        SCLogError(SC_ERR_OFFSET_MISSING_CONTENT, "offset needs "
-                   "preceding content option");
-        goto end;
+        SCLogError("offset needs preceding content option.");
+        return -1;
     }
 
     /* verify other conditions */
     DetectContentData *cd = (DetectContentData *)pm->ctx;
 
-    if (cd->flags & DETECT_CONTENT_OFFSET) {
-        SCLogError(SC_ERR_INVALID_SIGNATURE, "can't use multiple offsets for the same content. ");
-        goto end;
+    if (cd->flags & DETECT_CONTENT_STARTS_WITH) {
+        SCLogError("can't use offset with startswith.");
+        return -1;
     }
-    if ((cd->flags & DETECT_CONTENT_WITHIN) || (cd->flags & DETECT_CONTENT_DISTANCE)) {
-        SCLogError(SC_ERR_INVALID_SIGNATURE, "can't use a relative "
+    if (cd->flags & DETECT_CONTENT_OFFSET) {
+        SCLogError("can't use multiple offsets for the same content.");
+        return -1;
+    }
+    if (cd->flags & (DETECT_CONTENT_WITHIN | DETECT_CONTENT_DISTANCE)) {
+        SCLogError("can't use a relative "
                    "keyword like within/distance with a absolute "
                    "relative keyword like depth/offset for the same "
-                   "content." );
-        goto end;
+                   "content.");
+        return -1;
     }
     if (cd->flags & DETECT_CONTENT_NEGATED && cd->flags & DETECT_CONTENT_FAST_PATTERN) {
-        SCLogError(SC_ERR_INVALID_SIGNATURE, "can't have a relative "
-                   "negated keyword set along with a fast_pattern");
-        goto end;
+        SCLogError("can't have a relative "
+                   "negated keyword set along with 'fast_pattern'.");
+        return -1;
     }
     if (cd->flags & DETECT_CONTENT_FAST_PATTERN_ONLY) {
-        SCLogError(SC_ERR_INVALID_SIGNATURE, "can't have a relative "
-                   "keyword set along with a fast_pattern:only;");
-        goto end;
+        SCLogError("can't have a relative "
+                   "keyword set along with 'fast_pattern:only;'.");
+        return -1;
     }
     if (str[0] != '-' && isalpha((unsigned char)str[0])) {
-        SigMatch *bed_sm =
-            DetectByteExtractRetrieveSMVar(str, s);
-        if (bed_sm == NULL) {
-            SCLogError(SC_ERR_INVALID_SIGNATURE, "unknown byte_extract var "
-                       "seen in offset - %s\n", str);
-            goto end;
+        DetectByteIndexType index;
+        if (!DetectByteRetrieveSMVar(str, s, -1, &index)) {
+            SCLogError("unknown byte_ keyword var "
+                       "seen in offset - %s.",
+                    str);
+            return -1;
         }
-        cd->offset = ((DetectByteExtractData *)bed_sm->ctx)->local_id;
-        cd->flags |= DETECT_CONTENT_OFFSET_BE;
+        cd->offset = index;
+        cd->flags |= DETECT_CONTENT_OFFSET_VAR;
     } else {
-        cd->offset = (uint32_t)atoi(str);
+        if (StringParseUint16(&cd->offset, 0, 0, str) < 0)
+        {
+            SCLogError("invalid value for offset: %s.", str);
+            return -1;
+        }
         if (cd->depth != 0) {
             if (cd->depth < cd->content_len) {
                 SCLogDebug("depth increased to %"PRIu32" to match pattern len",
@@ -113,9 +117,6 @@ int DetectOffsetSetup (DetectEngineCtx *de_ctx, Signature *s, const char *offset
         }
     }
     cd->flags |= DETECT_CONTENT_OFFSET;
-
-    ret = 0;
- end:
-    return ret;
+    return 0;
 }
 
