@@ -21,18 +21,17 @@
  * \author Victor Julien <victor@inliniac.net>
  */
 
-#ifndef __DETECT_PKTVAR_H__
-#define __DETECT_PKTVAR_H__
+#ifndef SURICATA_DETECT_PKTVAR_H
+#define SURICATA_DETECT_PKTVAR_H
 
 typedef struct DetectPktvarData_ {
-    char *name;
-    uint8_t *content;
-    uint8_t content_len;
+    uint32_t id;
+    uint16_t content_len;
     uint8_t flags;
+    uint8_t *content;
 } DetectPktvarData;
 
 /* prototypes */
 void DetectPktvarRegister (void);
 
-#endif /* __DETECT_PKTVAR_H__ */
-
+#endif /* SURICATA_DETECT_PKTVAR_H */
