@@ -22,12 +22,10 @@
  * \author Victor Julien <victor@inliniac.net>
  */
 
-#ifndef __DETECT_TAG_H__
-#define __DETECT_TAG_H__
+#ifndef SURICATA_DETECT_TAG_H
+#define SURICATA_DETECT_TAG_H
 
 #include "suricata-common.h"
-#include "suricata.h"
-#include "util-time.h"
 
 /* Limit the number of times a session can be tagged by the
  * same rule without finishing older tags */
@@ -51,14 +49,12 @@ enum {
 enum {
     DETECT_TAG_DIR_SRC,
     DETECT_TAG_DIR_DST,
-    DETECT_TAG_DIR_MAX
 };
 
 enum {
     DETECT_TAG_METRIC_PACKET,
     DETECT_TAG_METRIC_SECONDS,
     DETECT_TAG_METRIC_BYTES,
-    DETECT_TAG_METRIC_MAX
 };
 
 /** This will be the rule options/parameters */
@@ -66,7 +62,7 @@ typedef struct DetectTagData_ {
     uint8_t type;          /**< tag type */
     uint8_t direction;     /**< host direction */
     uint32_t count;        /**< count */
-    uint32_t metric;       /**< metric */
+    uint8_t metric;        /**< metric */
 } DetectTagData;
 
 /** This is the installed data at the session/global or host table */
@@ -83,11 +79,8 @@ typedef struct DetectTagDataEntry_ {
         uint32_t packets;               /**< number of packets (metric packets) */
         uint32_t bytes;                 /**< number of bytes (metric bytes) */
     };
-    uint32_t first_ts;                  /**< First time seen (for metric = seconds) */
-    uint32_t last_ts;                   /**< Last time seen (to prune old sessions) */
-#if __WORDSIZE == 64
-    uint32_t pad1;
-#endif
+    SCTime_t first_ts;                  /**< First time seen (for metric = seconds) */
+    SCTime_t last_ts;                   /**< Last time seen (to prune old sessions) */
     struct DetectTagDataEntry_ *next;   /**< Pointer to the next tag of this
                                          *   session/src_host/dst_host (if any from other rule) */
 } DetectTagDataEntry;
@@ -97,9 +90,9 @@ typedef struct DetectTagDataEntry_ {
 #define TAG_ENTRY_FLAG_SKIPPED_FIRST    0x04
 
 /* prototypes */
+struct DetectEngineCtx_ ;
 void DetectTagRegister(void);
-void DetectTagDataFree(void *ptr);
+void DetectTagDataFree(struct DetectEngineCtx_ *, void *ptr);
 void DetectTagDataListFree(void *ptr);
 
-#endif /* __DETECT_TAG_H__ */
-
+#endif /* SURICATA_DETECT_TAG_H */
