@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2013 Open Information Security Foundation
+/* Copyright (C) 2007-2020 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -21,32 +21,20 @@
  * \author Anoop Saldanha <anoopsaldanha@gmail.com>
  */
 
-#ifndef __FLOW_MANAGER_H__
-#define __FLOW_MANAGER_H__
+#ifndef SURICATA_FLOW_MANAGER_H
+#define SURICATA_FLOW_MANAGER_H
+
+void FlowWakeupFlowManagerThread(void);
+void FlowWakeupFlowRecyclerThread(void);
 
 #define FlowTimeoutsReset() FlowTimeoutsInit()
 void FlowTimeoutsInit(void);
 void FlowTimeoutsEmergency(void);
-
-/** flow manager scheduling condition */
-SCCtrlCondT flow_manager_ctrl_cond;
-SCCtrlMutex flow_manager_ctrl_mutex;
-#define FlowWakeupFlowManagerThread() SCCtrlCondSignal(&flow_manager_ctrl_cond)
-
 void FlowManagerThreadSpawn(void);
 void FlowDisableFlowManagerThread(void);
-void FlowMgrRegisterTests (void);
-
-/** flow recycler scheduling condition */
-SCCtrlCondT flow_recycler_ctrl_cond;
-SCCtrlMutex flow_recycler_ctrl_mutex;
-#define FlowWakeupFlowRecyclerThread() \
-    SCCtrlCondSignal(&flow_recycler_ctrl_cond)
-
 void FlowRecyclerThreadSpawn(void);
 void FlowDisableFlowRecyclerThread(void);
-
 void TmModuleFlowManagerRegister (void);
 void TmModuleFlowRecyclerRegister (void);
 
-#endif /* __FLOW_MANAGER_H__ */
+#endif /* SURICATA_FLOW_MANAGER_H */
