@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2012 Open Information Security Foundation
+/* Copyright (C) 2007-2024 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -21,12 +21,14 @@
  * \author Anoop Saldanha <anoopsaldanha@gmail.com>
  */
 
-#ifndef __FLOW_TIMEOUT_H__
-#define __FLOW_TIMEOUT_H__
+#ifndef SURICATA_FLOW_TIMEOUT_H
+#define SURICATA_FLOW_TIMEOUT_H
 
-int FlowForceReassemblyForFlow(Flow *f, int server, int client);
-int FlowForceReassemblyNeedReassembly(Flow *f, int *server, int *client);
-void FlowForceReassembly(void);
-void FlowForceReassemblySetup(int detect_disabled);
+#include "stream-tcp-private.h"
 
-#endif /* __FLOW_TIMEOUT_H__ */
+void FlowSendToLocalThread(Flow *f);
+bool FlowNeedsReassembly(Flow *f);
+void FlowWorkToDoCleanup(void);
+Packet *FlowPseudoPacketGet(int direction, Flow *f, const TcpSession *ssn);
+
+#endif /* SURICATA_FLOW_TIMEOUT_H */
