@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2013 Open Information Security Foundation
+/* Copyright (C) 2007-2021 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -23,23 +23,26 @@
  * Host wrapper around storage api
  */
 
-#ifndef __HOST_STORAGE_H__
-#define __HOST_STORAGE_H__
+#ifndef SURICATA_HOST_STORAGE_H
+#define SURICATA_HOST_STORAGE_H
 
-#include "util-storage.h"
 #include "host.h"
 
+typedef struct HostStorageId_ {
+    int id;
+} HostStorageId;
+
 unsigned int HostStorageSize(void);
 
-void *HostGetStorageById(Host *h, int id);
-int HostSetStorageById(Host *h, int id, void *ptr);
-void *HostAllocStorageById(Host *h, int id);
+void *HostGetStorageById(Host *h, HostStorageId id);
+int HostSetStorageById(Host *h, HostStorageId id, void *ptr);
+void *HostAllocStorageById(Host *h, HostStorageId id);
 
-void HostFreeStorageById(Host *h, int id);
 void HostFreeStorage(Host *h);
 
 void RegisterHostStorageTests(void);
 
-int HostStorageRegister(const char *name, const unsigned int size, void *(*Alloc)(unsigned int), void (*Free)(void *));
+HostStorageId HostStorageRegister(const char *name, const unsigned int size,
+        void *(*Alloc)(unsigned int), void (*Free)(void *));
 
-#endif /* __HOST_STORAGE_H__ */
+#endif /* SURICATA_HOST_STORAGE_H */
