@@ -23,36 +23,24 @@
  * \author Victor Julien <victor@inliniac.net>
  * \author Paulo Pacheco <fooinha@gmail.com>
  *
- * Implements TLS logging portion of the engine. The TLS logger is
- * implemented as a packet logger, as the TLS parser is not transaction
- * aware.
+ * Implements TLS logging portion of the engine.
  */
 
 #include "suricata-common.h"
-#include "debug.h"
-#include "detect.h"
-#include "pkt-var.h"
 #include "conf.h"
 
-#include "threads.h"
 #include "threadvars.h"
-#include "tm-threads.h"
 
 #include "util-print.h"
-#include "util-unittest.h"
-
 #include "util-debug.h"
 
 #include "output.h"
 #include "log-tlslog.h"
 #include "app-layer-ssl.h"
-#include "app-layer.h"
 #include "app-layer-parser.h"
-#include "util-privs.h"
 #include "util-buffer.h"
 
 #include "util-logopenfile.h"
-#include "util-crypt.h"
 #include "util-time.h"
 #include "log-cf-common.h"
 
@@ -60,101 +48,52 @@
 
 #define MODULE_NAME "LogTlsLog"
 
+#define PRINT_BUF_LEN 46
+
 #define OUTPUT_BUFFER_SIZE 65535
-#define CERT_ENC_BUFFER_SIZE 2048
+
+#define LOG_TLS_DEFAULT  0
+#define LOG_TLS_EXTENDED 1
+#define LOG_TLS_CUSTOM   2
 
-#define LOG_TLS_DEFAULT            0
-#define LOG_TLS_EXTENDED           1
-#define LOG_TLS_CUSTOM             2
 #define LOG_TLS_SESSION_RESUMPTION 4
 
-#define LOG_TLS_CF_VERSION 'v'
+#define LOG_TLS_CF_VERSION         'v'
 #define LOG_TLS_CF_DATE_NOT_BEFORE 'd'
-#define LOG_TLS_CF_DATE_NOT_AFTER 'D'
-#define LOG_TLS_CF_SHA1 'f'
-#define LOG_TLS_CF_SNI 'n'
-#define LOG_TLS_CF_SUBJECT 's'
-#define LOG_TLS_CF_ISSUER 'i'
-#define LOG_TLS_CF_EXTENDED 'E'
+#define LOG_TLS_CF_DATE_NOT_AFTER  'D'
+#define LOG_TLS_CF_SHA1            'f'
+#define LOG_TLS_CF_SNI             'n'
+#define LOG_TLS_CF_SUBJECT         's'
+#define LOG_TLS_CF_ISSUER          'i'
+#define LOG_TLS_CF_EXTENDED        'E'
 
 typedef struct LogTlsFileCtx_ {
     LogFileCtx *file_ctx;
-    uint32_t flags; /** Store mode */
+    uint32_t flags;  /** Store mode */
     LogCustomFormat *cf;
 } LogTlsFileCtx;
 
 typedef struct LogTlsLogThread_ {
     LogTlsFileCtx *tlslog_ctx;
-
-    /** LogTlsFileCtx has the pointer to the file and a mutex to allow multithreading */
-    uint32_t tls_cnt;
-
     MemBuffer *buffer;
 } LogTlsLogThread;
 
-static void LogTlsLogVersion(MemBuffer *buffer, uint16_t version)
-{
-    char ssl_version[SSL_VERSION_MAX_STRLEN];
-    SSLVersionToString(version, ssl_version);
-    MemBufferWriteString(buffer, "VERSION='%s'", ssl_version);
-}
-
-static void LogTlsLogDate(MemBuffer *buffer, const char *title, time_t *date)
-{
-    char timebuf[64] = {0};
-    struct timeval tv;
-    tv.tv_sec = *date;
-    tv.tv_usec = 0;
-    CreateUtcIsoTimeString(&tv, timebuf, sizeof(timebuf));
-    MemBufferWriteString(buffer, "%s='%s'", title, timebuf);
-}
-
-static void LogTlsLogString(MemBuffer *buffer, const char *title, const char *value)
-{
-    MemBufferWriteString(buffer, "%s='%s'", title, value);
-}
-
-static void LogTlsLogExtended(LogTlsLogThread *aft, SSLState * state)
-{
-    if (state->server_connp.cert0_fingerprint != NULL) {
-        LOG_CF_WRITE_SPACE_SEPARATOR(aft->buffer);
-        LogTlsLogString(aft->buffer, "SHA1", state->server_connp.cert0_fingerprint);
-    }
-    if (state->client_connp.sni != NULL) {
-        LOG_CF_WRITE_SPACE_SEPARATOR(aft->buffer);
-        LogTlsLogString(aft->buffer, "SNI", state->client_connp.sni);
-    }
-    if (state->server_connp.cert0_serial != NULL) {
-        LOG_CF_WRITE_SPACE_SEPARATOR(aft->buffer);
-        LogTlsLogString(aft->buffer, "SERIAL", state->server_connp.cert0_serial);
-    }
-
-    LOG_CF_WRITE_SPACE_SEPARATOR(aft->buffer);
-    LogTlsLogVersion(aft->buffer, state->server_connp.version);
-
-    if (state->server_connp.cert0_not_before != 0) {
-        LOG_CF_WRITE_SPACE_SEPARATOR(aft->buffer);
-        LogTlsLogDate(aft->buffer, "NOTBEFORE", &state->server_connp.cert0_not_before);
-    }
-    if (state->server_connp.cert0_not_after != 0) {
-        LOG_CF_WRITE_SPACE_SEPARATOR(aft->buffer);
-        LogTlsLogDate(aft->buffer, "NOTAFTER", &state->server_connp.cert0_not_after);
-    }
-}
-
-int TLSGetIPInformations(const Packet *p, char* srcip, size_t srcip_len,
-                             Port* sp, char* dstip, size_t dstip_len,
-                             Port* dp, int ipproto)
+int TLSGetIPInformations(const Packet *p, char *srcip, socklen_t srcip_len, Port *sp, char *dstip,
+        socklen_t dstip_len, Port *dp, int ipproto)
 {
     if ((PKT_IS_TOSERVER(p))) {
         switch (ipproto) {
             case AF_INET:
-                PrintInet(AF_INET, (const void *) GET_IPV4_SRC_ADDR_PTR(p), srcip, srcip_len);
-                PrintInet(AF_INET, (const void *) GET_IPV4_DST_ADDR_PTR(p), dstip, dstip_len);
+                PrintInet(AF_INET, (const void *) GET_IPV4_SRC_ADDR_PTR(p),
+                          srcip, srcip_len);
+                PrintInet(AF_INET, (const void *) GET_IPV4_DST_ADDR_PTR(p),
+                          dstip, dstip_len);
                 break;
             case AF_INET6:
-                PrintInet(AF_INET6, (const void *) GET_IPV6_SRC_ADDR(p), srcip, srcip_len);
-                PrintInet(AF_INET6, (const void *) GET_IPV6_DST_ADDR(p), dstip, dstip_len);
+                PrintInet(AF_INET6, (const void *) GET_IPV6_SRC_ADDR(p), srcip,
+                          srcip_len);
+                PrintInet(AF_INET6, (const void *) GET_IPV6_DST_ADDR(p), dstip,
+                          dstip_len);
                 break;
             default:
                 return 0;
@@ -164,12 +103,16 @@ int TLSGetIPInformations(const Packet *p, char* srcip, size_t srcip_len,
     } else {
         switch (ipproto) {
             case AF_INET:
-                PrintInet(AF_INET, (const void *) GET_IPV4_DST_ADDR_PTR(p), srcip, srcip_len);
-                PrintInet(AF_INET, (const void *) GET_IPV4_SRC_ADDR_PTR(p), dstip, dstip_len);
+                PrintInet(AF_INET, (const void *) GET_IPV4_DST_ADDR_PTR(p),
+                          srcip, srcip_len);
+                PrintInet(AF_INET, (const void *) GET_IPV4_SRC_ADDR_PTR(p),
+                          dstip, dstip_len);
                 break;
             case AF_INET6:
-                PrintInet(AF_INET6, (const void *) GET_IPV6_DST_ADDR(p), srcip, srcip_len);
-                PrintInet(AF_INET6, (const void *) GET_IPV6_SRC_ADDR(p), dstip, dstip_len);
+                PrintInet(AF_INET6, (const void *) GET_IPV6_DST_ADDR(p), srcip,
+                          srcip_len);
+                PrintInet(AF_INET6, (const void *) GET_IPV6_SRC_ADDR(p), dstip,
+                          dstip_len);
                 break;
             default:
                 return 0;
@@ -180,15 +123,15 @@ int TLSGetIPInformations(const Packet *p, char* srcip, size_t srcip_len,
     return 1;
 }
 
-static TmEcode LogTlsLogThreadInit(ThreadVars *t, const void *initdata, void **data)
+static TmEcode LogTlsLogThreadInit(ThreadVars *t, const void *initdata,
+                                   void **data)
 {
-    LogTlsLogThread *aft = SCMalloc(sizeof(LogTlsLogThread));
+    LogTlsLogThread *aft = SCCalloc(1, sizeof(LogTlsLogThread));
     if (unlikely(aft == NULL))
         return TM_ECODE_FAILED;
-    memset(aft, 0, sizeof(LogTlsLogThread));
 
     if (initdata == NULL) {
-        SCLogDebug( "Error getting context for TLSLog.  \"initdata\" argument NULL");
+        SCLogDebug("Error getting context for TLSLog. \"initdata\" argument NULL");
         SCFree(aft);
         return TM_ECODE_FAILED;
     }
@@ -199,22 +142,21 @@ static TmEcode LogTlsLogThreadInit(ThreadVars *t, const void *initdata, void **d
         return TM_ECODE_FAILED;
     }
 
-    /* Use the Ouptut Context (file pointer and mutex) */
+    /* Use the Output Context (file pointer and mutex) */
     aft->tlslog_ctx = ((OutputCtx *) initdata)->data;
 
-    *data = (void *) aft;
+    *data = (void *)aft;
     return TM_ECODE_OK;
 }
 
 static TmEcode LogTlsLogThreadDeinit(ThreadVars *t, void *data)
 {
-    LogTlsLogThread *aft = (LogTlsLogThread *) data;
+    LogTlsLogThread *aft = (LogTlsLogThread *)data;
     if (aft == NULL) {
         return TM_ECODE_OK;
     }
 
     MemBufferFree(aft->buffer);
-    /* clear memory */
     memset(aft, 0, sizeof(LogTlsLogThread));
 
     SCFree(aft);
@@ -230,28 +172,20 @@ static void LogTlsLogDeInitCtx(OutputCtx *output_ctx)
     SCFree(output_ctx);
 }
 
-static void LogTlsLogExitPrintStats(ThreadVars *tv, void *data)
-{
-    LogTlsLogThread *aft = (LogTlsLogThread *) data;
-    if (aft == NULL) {
-        return;
-    }
-
-    SCLogInfo("TLS logger logged %" PRIu32 " requests", aft->tls_cnt);
-}
-
 /** \brief Create a new tls log LogFileCtx.
  *  \param conf Pointer to ConfNode containing this loggers configuration.
  *  \return NULL if failure, LogFileCtx* to the file_ctx if succesful
  * */
-static OutputInitResult LogTlsLogInitCtx(ConfNode *conf)
+static OutputInitResult LogTlsLogInitCtx(SCConfNode *conf)
 {
+    SCLogWarning("The tls-log output has been deprecated and will be removed in Suricata 9.0.");
+
     OutputInitResult result = { NULL, false };
     LogFileCtx* file_ctx = LogFileNewCtx();
 
     if (file_ctx == NULL) {
-        SCLogError(SC_ERR_TLS_LOG_GENERIC, "LogTlsLogInitCtx: Couldn't "
-        "create new file_ctx");
+        SCLogError("LogTlsLogInitCtx: Couldn't "
+                   "create new file_ctx");
         return result;
     }
 
@@ -260,57 +194,61 @@ static OutputInitResult LogTlsLogInitCtx(ConfNode *conf)
     }
 
     LogTlsFileCtx *tlslog_ctx = SCCalloc(1, sizeof(LogTlsFileCtx));
-    if (unlikely(tlslog_ctx == NULL))
+    if (unlikely(tlslog_ctx == NULL)) {
         goto filectx_error;
+    }
     tlslog_ctx->file_ctx = file_ctx;
 
-    const char *extended = ConfNodeLookupChildValue(conf, "extended");
-    const char *custom = ConfNodeLookupChildValue(conf, "custom");
-    const char *customformat = ConfNodeLookupChildValue(conf, "customformat");
+    const char *extended = SCConfNodeLookupChildValue(conf, "extended");
+    const char *custom = SCConfNodeLookupChildValue(conf, "custom");
+    const char *customformat = SCConfNodeLookupChildValue(conf, "customformat");
 
     /* If custom logging format is selected, lets parse it */
-    if (custom != NULL && customformat != NULL && ConfValIsTrue(custom)) {
+    if (custom != NULL && customformat != NULL && SCConfValIsTrue(custom)) {
         tlslog_ctx->cf = LogCustomFormatAlloc();
         if (!tlslog_ctx->cf) {
             goto tlslog_error;
         }
 
         tlslog_ctx->flags |= LOG_TLS_CUSTOM;
-        /* Parsing */
-        if ( ! LogCustomFormatParse(tlslog_ctx->cf, customformat)) {
+
+        if (!LogCustomFormatParse(tlslog_ctx->cf, customformat)) {
             goto parser_error;
         }
     } else {
         if (extended == NULL) {
             tlslog_ctx->flags |= LOG_TLS_DEFAULT;
         } else {
-            if (ConfValIsTrue(extended)) {
+            if (SCConfValIsTrue(extended)) {
                 tlslog_ctx->flags |= LOG_TLS_EXTENDED;
             }
         }
     }
 
-    const char *session_resumption = ConfNodeLookupChildValue(conf, "session-resumption");
-    if (session_resumption == NULL || ConfValIsTrue(session_resumption)) {
+    const char *resumption = SCConfNodeLookupChildValue(conf, "session-resumption");
+    if (resumption == NULL || SCConfValIsTrue(resumption)) {
         tlslog_ctx->flags |= LOG_TLS_SESSION_RESUMPTION;
     }
 
     OutputCtx *output_ctx = SCCalloc(1, sizeof(OutputCtx));
-    if (unlikely(output_ctx == NULL))
+    if (unlikely(output_ctx == NULL)) {
         goto tlslog_error;
+    }
     output_ctx->data = tlslog_ctx;
     output_ctx->DeInit = LogTlsLogDeInitCtx;
 
     SCLogDebug("TLS log output initialized");
 
-    /* enable the logger for the app layer */
-    AppLayerParserRegisterLogger(IPPROTO_TCP, ALPROTO_TLS);
+    /* Enable the logger for the app layer */
+    SCAppLayerParserRegisterLogger(IPPROTO_TCP, ALPROTO_TLS);
 
     result.ctx = output_ctx;
     result.ok = true;
     return result;
+
 parser_error:
-    SCLogError(SC_ERR_INVALID_ARGUMENT,"Syntax error in custom tls log format string.");
+    SCLogError("Syntax error in custom tls log "
+               "format string.");
 tlslog_error:
     LogCustomFormatFree(tlslog_ctx->cf);
     SCFree(tlslog_ctx);
@@ -319,21 +257,106 @@ filectx_error:
     return result;
 }
 
+static void LogTlsLogVersion(MemBuffer *buffer, uint16_t version)
+{
+    char ssl_version[SSL_VERSION_MAX_STRLEN];
+    SSLVersionToString(version, ssl_version);
+    MemBufferWriteString(buffer, "VERSION='%s'", ssl_version);
+}
+
+static void LogTlsLogDate(MemBuffer *buffer, const char *title, int64_t *date)
+{
+    char timebuf[64] = {0};
+    if (sc_x509_format_timestamp(*date, timebuf, sizeof(timebuf))) {
+        MemBufferWriteString(buffer, "%s='%s'", title, timebuf);
+    }
+}
+
+static void LogTlsLogString(MemBuffer *buffer, const char *title,
+                            const char *value)
+{
+    MemBufferWriteString(buffer, "%s='%s'", title, value);
+}
+
+static void LogTlsLogBasic(LogTlsLogThread *aft, SSLState *ssl_state, const SCTime_t ts,
+        char *srcip, Port sp, char *dstip, Port dp)
+{
+    char timebuf[64];
+    CreateTimeString(ts, timebuf, sizeof(timebuf));
+    MemBufferWriteString(aft->buffer,
+                         "%s %s:%d -> %s:%d  TLS:",
+                         timebuf, srcip, sp, dstip, dp);
+
+    if (ssl_state->server_connp.cert0_subject != NULL) {
+        MemBufferWriteString(aft->buffer, " Subject='%s'",
+        ssl_state->server_connp.cert0_subject);
+    }
+
+    if (ssl_state->server_connp.cert0_issuerdn != NULL) {
+        MemBufferWriteString(aft->buffer, " Issuerdn='%s'",
+                             ssl_state->server_connp.cert0_issuerdn);
+    }
+
+    if (ssl_state->flags & SSL_AL_FLAG_SESSION_RESUMED) {
+        /* Only log a session as 'resumed' if a certificate has not
+           been seen. */
+        if ((ssl_state->server_connp.cert0_issuerdn == NULL) &&
+                (ssl_state->server_connp.cert0_subject == NULL) &&
+                (ssl_state->flags & SSL_AL_FLAG_STATE_SERVER_HELLO) &&
+                ((ssl_state->flags & SSL_AL_FLAG_LOG_WITHOUT_CERT) == 0)) {
+            MemBufferWriteString(aft->buffer, " Session='resumed'");
+        }
+    }
+}
+
+static void LogTlsLogExtended(LogTlsLogThread *aft, SSLState *ssl_state, const SCTime_t ts,
+        char *srcip, Port sp, char *dstip, Port dp)
+{
+    if (ssl_state->server_connp.cert0_fingerprint != NULL) {
+        LOG_CF_WRITE_SPACE_SEPARATOR(aft->buffer);
+        LogTlsLogString(aft->buffer, "SHA1",
+                        ssl_state->server_connp.cert0_fingerprint);
+    }
+    if (ssl_state->client_connp.sni != NULL) {
+        LOG_CF_WRITE_SPACE_SEPARATOR(aft->buffer);
+        LogTlsLogString(aft->buffer, "SNI", ssl_state->client_connp.sni);
+    }
+    if (ssl_state->server_connp.cert0_serial != NULL) {
+        LOG_CF_WRITE_SPACE_SEPARATOR(aft->buffer);
+        LogTlsLogString(aft->buffer, "SERIAL",
+                        ssl_state->server_connp.cert0_serial);
+    }
+
+    LOG_CF_WRITE_SPACE_SEPARATOR(aft->buffer);
+    LogTlsLogVersion(aft->buffer, ssl_state->server_connp.version);
+
+    if (ssl_state->server_connp.cert0_not_before != 0) {
+        LOG_CF_WRITE_SPACE_SEPARATOR(aft->buffer);
+        LogTlsLogDate(aft->buffer, "NOTBEFORE",
+                      &ssl_state->server_connp.cert0_not_before);
+    }
+    if (ssl_state->server_connp.cert0_not_after != 0) {
+        LOG_CF_WRITE_SPACE_SEPARATOR(aft->buffer);
+        LogTlsLogDate(aft->buffer, "NOTAFTER",
+                      &ssl_state->server_connp.cert0_not_after);
+    }
+}
+
 /* Custom format logging */
-static void LogTlsLogCustom(LogTlsLogThread *aft, SSLState *ssl_state, const struct timeval *ts,
-                            char *srcip, Port sp, char *dstip, Port dp)
+static void LogTlsLogCustom(LogTlsLogThread *aft, SSLState *ssl_state, const SCTime_t ts,
+        char *srcip, Port sp, char *dstip, Port dp)
 {
     LogTlsFileCtx *tlslog_ctx = aft->tlslog_ctx;
     uint32_t i;
     char buf[64];
 
-    for (i = 0; i < tlslog_ctx->cf->cf_n; i++) {
-
-        LogCustomFormatNode * node = tlslog_ctx->cf->cf_nodes[i];
-        if (! node) /* Should never happen */
+    for (i = 0; i < tlslog_ctx->cf->cf_n; i++)
+    {
+        LogCustomFormatNode *node = tlslog_ctx->cf->cf_nodes[i];
+        if (!node) /* Should never happen */
             continue;
 
-        switch (node->type){
+        switch (node->type) {
             case LOG_CF_LITERAL:
             /* LITERAL */
                 MemBufferWriteString(aft->buffer, "%s", node->data);
@@ -344,19 +367,21 @@ static void LogTlsLogCustom(LogTlsLogThread *aft, SSLState *ssl_state, const str
                 break;
             case LOG_CF_TIMESTAMP_U:
             /* TIMESTAMP USECONDS */
-                snprintf(buf, sizeof(buf), "%06u", (unsigned int) ts->tv_usec);
-                PrintRawUriBuf((char *)aft->buffer->buffer, &aft->buffer->offset,
-                            aft->buffer->size, (uint8_t *)buf, MIN(strlen(buf),6));
-                break;
+            snprintf(buf, sizeof(buf), "%06u", (unsigned int)SCTIME_USECS(ts));
+            PrintRawUriBuf((char *)aft->buffer->buffer, &aft->buffer->offset, aft->buffer->size,
+                    (uint8_t *)buf, MIN(strlen(buf), 6));
+            break;
             case LOG_CF_CLIENT_IP:
             /* CLIENT IP ADDRESS */
-                PrintRawUriBuf((char *)aft->buffer->buffer, &aft->buffer->offset,
-                            aft->buffer->size, (uint8_t *)srcip,strlen(srcip));
+                PrintRawUriBuf((char *)aft->buffer->buffer,
+                               &aft->buffer->offset, aft->buffer->size,
+                               (uint8_t *)srcip,strlen(srcip));
                 break;
             case LOG_CF_SERVER_IP:
             /* SERVER IP ADDRESS */
-                PrintRawUriBuf((char *)aft->buffer->buffer, &aft->buffer->offset,
-                            aft->buffer->size, (uint8_t *)dstip,strlen(dstip));
+                PrintRawUriBuf((char *)aft->buffer->buffer,
+                               &aft->buffer->offset, aft->buffer->size,
+                               (uint8_t *)dstip, strlen(dstip));
                 break;
             case LOG_CF_CLIENT_PORT:
             /* CLIENT PORT */
@@ -370,15 +395,17 @@ static void LogTlsLogCustom(LogTlsLogThread *aft, SSLState *ssl_state, const str
                 LogTlsLogVersion(aft->buffer, ssl_state->server_connp.version);
                 break;
             case LOG_TLS_CF_DATE_NOT_BEFORE:
-                LogTlsLogDate(aft->buffer, "NOTBEFORE", &ssl_state->server_connp.cert0_not_before);
+                LogTlsLogDate(aft->buffer, "NOTBEFORE",
+                              &ssl_state->server_connp.cert0_not_before);
                 break;
             case LOG_TLS_CF_DATE_NOT_AFTER:
-                LogTlsLogDate(aft->buffer, "NOTAFTER", &ssl_state->server_connp.cert0_not_after);
+                LogTlsLogDate(aft->buffer, "NOTAFTER",
+                              &ssl_state->server_connp.cert0_not_after);
                 break;
             case LOG_TLS_CF_SHA1:
                 if (ssl_state->server_connp.cert0_fingerprint != NULL) {
                     MemBufferWriteString(aft->buffer, "%s",
-                                         ssl_state->server_connp.cert0_fingerprint);
+                            ssl_state->server_connp.cert0_fingerprint);
                 } else {
                     LOG_CF_WRITE_UNKNOWN_VALUE(aft->buffer);
                 }
@@ -394,7 +421,7 @@ static void LogTlsLogCustom(LogTlsLogThread *aft, SSLState *ssl_state, const str
             case LOG_TLS_CF_SUBJECT:
                 if (ssl_state->server_connp.cert0_subject != NULL) {
                     MemBufferWriteString(aft->buffer, "%s",
-                                         ssl_state->server_connp.cert0_subject);
+                            ssl_state->server_connp.cert0_subject);
                 } else {
                     LOG_CF_WRITE_UNKNOWN_VALUE(aft->buffer);
                 }
@@ -402,23 +429,23 @@ static void LogTlsLogCustom(LogTlsLogThread *aft, SSLState *ssl_state, const str
             case LOG_TLS_CF_ISSUER:
                 if (ssl_state->server_connp.cert0_issuerdn != NULL) {
                     MemBufferWriteString(aft->buffer, "%s",
-                                         ssl_state->server_connp.cert0_issuerdn);
+                            ssl_state->server_connp.cert0_issuerdn);
                 } else {
                     LOG_CF_WRITE_UNKNOWN_VALUE(aft->buffer);
                 }
                 break;
             case LOG_TLS_CF_EXTENDED:
             /* Extended format  */
-                LogTlsLogExtended(aft, ssl_state);
+                LogTlsLogExtended(aft, ssl_state, ts, srcip, sp, dstip, dp);
                 break;
             default:
             /* NO MATCH */
                 MemBufferWriteString(aft->buffer, LOG_CF_NONE);
-                SCLogDebug("No matching parameter %%%c for custom tls log.", node->type);
+                SCLogDebug("No matching parameter %%%c for custom tls log.",
+                           node->type);
                 break;
         }
     }
-    MemBufferWriteString(aft->buffer, "\n");
 }
 
 
@@ -427,8 +454,7 @@ static int LogTlsLogger(ThreadVars *tv, void *thread_data, const Packet *p,
 {
     LogTlsLogThread *aft = (LogTlsLogThread *)thread_data;
     LogTlsFileCtx *hlog = aft->tlslog_ctx;
-    char timebuf[64];
-    int ipproto = (PKT_IS_IPV4(p)) ? AF_INET : AF_INET6;
+    int ipproto = (PacketIsIPv4(p)) ? AF_INET : AF_INET6;
 
     SSLState *ssl_state = (SSLState *)state;
     if (unlikely(ssl_state == NULL)) {
@@ -443,53 +469,26 @@ static int LogTlsLogger(ThreadVars *tv, void *thread_data, const Packet *p,
         return 0;
     }
 
-#define PRINT_BUF_LEN 46
     char srcip[PRINT_BUF_LEN], dstip[PRINT_BUF_LEN];
+
     Port sp, dp;
     if (!TLSGetIPInformations(p, srcip, PRINT_BUF_LEN, &sp, dstip,
                               PRINT_BUF_LEN, &dp, ipproto)) {
         return 0;
     }
 
-    /* Custom format */
+    MemBufferReset(aft->buffer);
+
     if (hlog->flags & LOG_TLS_CUSTOM) {
-        LogTlsLogCustom(aft, ssl_state, &p->ts, srcip, sp, dstip, dp);
+        LogTlsLogCustom(aft, ssl_state, p->ts, srcip, sp, dstip, dp);
+    } else if (hlog->flags & LOG_TLS_EXTENDED) {
+        LogTlsLogBasic(aft, ssl_state, p->ts, srcip, sp, dstip, dp);
+        LogTlsLogExtended(aft, ssl_state, p->ts, srcip, sp, dstip, dp);
     } else {
-
-        MemBufferReset(aft->buffer);
-        CreateTimeString(&p->ts, timebuf, sizeof(timebuf));
-        MemBufferWriteString(aft->buffer,
-                             "%s %s:%d -> %s:%d  TLS:",
-                             timebuf, srcip, sp, dstip, dp);
-
-        if (ssl_state->server_connp.cert0_subject != NULL) {
-            MemBufferWriteString(aft->buffer, " Subject='%s'",
-                                 ssl_state->server_connp.cert0_subject);
-        }
-        if (ssl_state->server_connp.cert0_issuerdn != NULL) {
-            MemBufferWriteString(aft->buffer, " Issuerdn='%s'",
-                                 ssl_state->server_connp.cert0_issuerdn);
-        }
-        if (ssl_state->flags & SSL_AL_FLAG_SESSION_RESUMED) {
-            /* Only log a session as 'resumed' if a certificate has not
-               been seen. */
-            if ((ssl_state->server_connp.cert0_issuerdn == NULL) &&
-                    (ssl_state->server_connp.cert0_subject == NULL) &&
-                    (ssl_state->flags & SSL_AL_FLAG_STATE_SERVER_HELLO) &&
-                    ((ssl_state->flags & SSL_AL_FLAG_LOG_WITHOUT_CERT) == 0)) {
-                MemBufferWriteString(aft->buffer, " Session='resumed'");
-            }
-        }
-
-        if (hlog->flags & LOG_TLS_EXTENDED) {
-            LogTlsLogExtended(aft, ssl_state);
-            MemBufferWriteString(aft->buffer, "\n");
-        } else {
-            MemBufferWriteString(aft->buffer, "\n");
-        }
+        LogTlsLogBasic(aft, ssl_state, p->ts, srcip, sp, dstip, dp);
     }
 
-    aft->tls_cnt++;
+    MemBufferWriteString(aft->buffer, "\n");
 
     hlog->file_ctx->Write((const char *)MEMBUFFER_BUFFER(aft->buffer),
         MEMBUFFER_OFFSET(aft->buffer), hlog->file_ctx);
@@ -499,8 +498,7 @@ static int LogTlsLogger(ThreadVars *tv, void *thread_data, const Packet *p,
 
 void LogTlsLogRegister(void)
 {
-    OutputRegisterTxModuleWithProgress(LOGGER_TLS, MODULE_NAME, "tls-log",
-        LogTlsLogInitCtx, ALPROTO_TLS, LogTlsLogger, TLS_HANDSHAKE_DONE,
-        TLS_HANDSHAKE_DONE, LogTlsLogThreadInit, LogTlsLogThreadDeinit,
-        LogTlsLogExitPrintStats);
+    OutputRegisterTxModuleWithProgress(LOGGER_TLS, MODULE_NAME, "tls-log", LogTlsLogInitCtx,
+            ALPROTO_TLS, LogTlsLogger, TLS_STATE_CLIENT_HANDSHAKE_DONE,
+            TLS_STATE_SERVER_HANDSHAKE_DONE, LogTlsLogThreadInit, LogTlsLogThreadDeinit);
 }
