@@ -22,14 +22,12 @@
  * \author Eric Leblond <eric@regit.org>
  */
 
-#ifndef __LOG_TLSLOG_H__
-#define __LOG_TLSLOG_H__
+#ifndef SURICATA_LOG_TLSLOG_H
+#define SURICATA_LOG_TLSLOG_H
 
-void TmModuleLogTlsLogRegister (void);
+void LogTlsLogRegister(void);
 
-int TLSGetIPInformations(const Packet *p, char* srcip, size_t srcip_len,
-                             Port* sp, char* dstip, size_t dstip_len,
-                             Port* dp, int ipproto);
-
-#endif /* __LOG_TLSLOG_H__ */
+int TLSGetIPInformations(const Packet *p, char *srcip, socklen_t srcip_len, Port *sp, char *dstip,
+        socklen_t dstip_len, Port *dp, int ipproto);
 
+#endif /* SURICATA_LOG_TLSLOG_H */
