@@ -1,4 +1,4 @@
-/* Copyright (C) 2018 Open Information Security Foundation
+/* Copyright (C) 2018-2022 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -16,26 +16,25 @@
  */
 
 #include "suricata-common.h"
+#include "output-filestore.h"
 
-#include "app-layer-parser.h"
-#include "app-layer-htp.h"
-#include "app-layer-htp-xff.h"
-#include "app-layer-smtp.h"
+#include "stream-tcp.h"
+
+#include "feature.h"
 
 #include "output.h"
-#include "output-filestore.h"
 #include "output-json-file.h"
 
-#include "util-print.h"
+#include "util-conf.h"
 #include "util-misc.h"
-
-#ifdef HAVE_NSS
+#include "util-path.h"
+#include "util-print.h"
 
 #define MODULE_NAME "OutputFilestore"
 
 /* Create a filestore specific PATH_MAX that is less than the system
  * PATH_MAX to prevent newer gcc truncation warnings with snprint. */
-#define SHA256_STRING_LEN (SHA256_LENGTH * 2)
+#define SHA256_STRING_LEN    (SC_SHA256_LEN * 2)
 #define LEAF_DIR_MAX_LEN 4
 #define FILESTORE_PREFIX_MAX (PATH_MAX - SHA256_STRING_LEN - LEAF_DIR_MAX_LEN)
 
@@ -59,15 +58,26 @@ typedef struct OutputFilestoreLogThread_ {
     uint16_t fs_error_counter;
 } OutputFilestoreLogThread;
 
+enum WarnOnceTypes {
+    WOT_OPEN,
+    WOT_WRITE,
+    WOT_UNLINK,
+    WOT_RENAME,
+    WOT_SNPRINTF,
+
+    WOT_MAX,
+};
+
 /* For WARN_ONCE, a record of warnings that have already been
  * issued. */
-static __thread bool once_errs[SC_ERR_MAX];
+static thread_local bool once_errs[WOT_MAX];
 
-#define WARN_ONCE(err_code, ...)  do {                   \
-        if (!once_errs[err_code]) {                      \
-            once_errs[err_code] = true;                  \
-            SCLogWarning(err_code, __VA_ARGS__);         \
-        }                                                \
+#define WARN_ONCE(wot_type, ...)                                                                   \
+    do {                                                                                           \
+        if (!once_errs[wot_type]) {                                                                \
+            once_errs[wot_type] = true;                                                            \
+            SCLogWarning(__VA_ARGS__);                                                             \
+        }                                                                                          \
     } while (0)
 
 static uint64_t OutputFilestoreOpenFilesCounter(void)
@@ -87,15 +97,6 @@ static uint32_t FileGetMaxOpenFiles(void)
     return g_file_store_max_open_files;
 }
 
-static void PrintHexString(char *str, size_t size, uint8_t *buf, size_t buf_len)
-{
-    int i = 0;
-    size_t x = 0;
-    for (i = 0, x = 0; x < buf_len; x++) {
-        i += snprintf(&str[i], size - i, "%02x", buf[x]);
-    }
-}
-
 /**
  * \brief Update the timestamps on a file to match those of another
  *     file.
@@ -121,12 +122,13 @@ static void OutputFilestoreUpdateFileTime(const char *src_filename,
     }
 }
 
-static void OutputFilestoreFinalizeFiles(ThreadVars *tv,
-        const OutputFilestoreLogThread *oft, const OutputFilestoreCtx *ctx,
-        const Packet *p, File *ff, uint8_t dir) {
+static void OutputFilestoreFinalizeFiles(ThreadVars *tv, const OutputFilestoreLogThread *oft,
+        const OutputFilestoreCtx *ctx, const Packet *p, File *ff, void *tx, const uint64_t tx_id,
+        uint8_t dir)
+{
     /* Stringify the SHA256 which will be used in the final
      * filename. */
-    char sha256string[(SHA256_LENGTH * 2) + 1];
+    char sha256string[(SC_SHA256_LEN * 2) + 1];
     PrintHexString(sha256string, sizeof(sha256string), ff->sha256,
             sizeof(ff->sha256));
 
@@ -142,14 +144,13 @@ static void OutputFilestoreFinalizeFiles(ThreadVars *tv,
         OutputFilestoreUpdateFileTime(tmp_filename, final_filename);
         if (unlink(tmp_filename) != 0) {
             StatsIncr(tv, oft->fs_error_counter);
-            WARN_ONCE(SC_WARN_REMOVE_FILE,
-                    "Failed to remove temporary file %s: %s", tmp_filename,
+            WARN_ONCE(WOT_UNLINK, "Failed to remove temporary file %s: %s", tmp_filename,
                     strerror(errno));
         }
     } else if (rename(tmp_filename, final_filename) != 0) {
         StatsIncr(tv, oft->fs_error_counter);
-        WARN_ONCE(SC_WARN_RENAMING_FILE, "Failed to rename %s to %s: %s",
-                tmp_filename, final_filename, strerror(errno));
+        WARN_ONCE(WOT_RENAME, "Failed to rename %s to %s: %s", tmp_filename, final_filename,
+                strerror(errno));
         if (unlink(tmp_filename) != 0) {
             /* Just increment, don't log as has_fs_errors would
              * already be set above. */
@@ -158,30 +159,32 @@ static void OutputFilestoreFinalizeFiles(ThreadVars *tv,
         return;
     }
 
-#ifdef HAVE_LIBJANSSON
     if (ctx->fileinfo) {
         char js_metadata_filename[PATH_MAX];
-        if (snprintf(js_metadata_filename, sizeof(js_metadata_filename),
-                        "%s.%"PRIuMAX".%u.json", final_filename,
-                        (uintmax_t)p->ts.tv_sec, ff->file_store_id)
-                == (int)sizeof(js_metadata_filename)) {
-            WARN_ONCE(SC_ERR_SPRINTF,
-                "Failed to write file info record. Output filename truncated.");
+        if (snprintf(js_metadata_filename, sizeof(js_metadata_filename), "%s.%" PRIuMAX ".%u.json",
+                    final_filename, (uintmax_t)SCTIME_SECS(p->ts),
+                    ff->file_store_id) == (int)sizeof(js_metadata_filename)) {
+            WARN_ONCE(WOT_SNPRINTF, "Failed to write file info record. Output filename truncated.");
         } else {
-            json_t *js_fileinfo = JsonBuildFileInfoRecord(p, ff, true, dir,
-                    ctx->xff_cfg);
+            SCJsonBuilder *js_fileinfo =
+                    JsonBuildFileInfoRecord(p, ff, tx, tx_id, true, dir, ctx->xff_cfg, NULL);
             if (likely(js_fileinfo != NULL)) {
-                json_dump_file(js_fileinfo, js_metadata_filename, 0);
-                json_decref(js_fileinfo);
+                SCJbClose(js_fileinfo);
+                FILE *out = fopen(js_metadata_filename, "w");
+                if (out != NULL) {
+                    size_t js_len = SCJbLen(js_fileinfo);
+                    fwrite(SCJbPtr(js_fileinfo), js_len, 1, out);
+                    fclose(out);
+                }
+                SCJbFree(js_fileinfo);
             }
         }
     }
-#endif
 }
 
-static int OutputFilestoreLogger(ThreadVars *tv, void *thread_data,
-        const Packet *p, File *ff, const uint8_t *data, uint32_t data_len,
-        uint8_t flags, uint8_t dir)
+static int OutputFilestoreLogger(ThreadVars *tv, void *thread_data, const Packet *p, File *ff,
+        void *tx, const uint64_t tx_id, const uint8_t *data, uint32_t data_len, uint8_t flags,
+        uint8_t dir)
 {
     SCEnter();
     OutputFilestoreLogThread *aft = (OutputFilestoreLogThread *)thread_data;
@@ -189,30 +192,15 @@ static int OutputFilestoreLogger(ThreadVars *tv, void *thread_data,
     char filename[PATH_MAX] = "";
     int file_fd = -1;
 
-    /* no flow, no files */
-    if (p->flow == NULL) {
-        SCReturnInt(TM_ECODE_OK);
-    }
-
-    if (!(PKT_IS_IPV4(p) || PKT_IS_IPV6(p))) {
-        return 0;
-    }
-
     SCLogDebug("ff %p, data %p, data_len %u", ff, data, data_len);
 
-    char base_filename[PATH_MAX] = "";
-    snprintf(base_filename, sizeof(base_filename), "%s/file.%u",
-            ctx->tmpdir, ff->file_store_id);
-    snprintf(filename, sizeof(filename), "%s", base_filename);
-
     if (flags & OUTPUT_FILEDATA_FLAG_OPEN) {
+        snprintf(filename, sizeof(filename), "%s/file.%u", ctx->tmpdir, ff->file_store_id);
         file_fd = open(filename, O_CREAT | O_TRUNC | O_NOFOLLOW | O_WRONLY,
                 0644);
         if (file_fd == -1) {
             StatsIncr(tv, aft->fs_error_counter);
-            SCLogWarning(SC_ERR_OPENING_FILE,
-                    "Filestore (v2) failed to create %s: %s", filename,
-                    strerror(errno));
+            SCLogWarning("Filestore (v2) failed to create %s: %s", filename, strerror(errno));
             return -1;
         }
 
@@ -228,12 +216,12 @@ static int OutputFilestoreLogger(ThreadVars *tv, void *thread_data,
     /* we can get called with a NULL ffd when we need to close */
     } else if (data != NULL) {
         if (ff->fd == -1) {
+            snprintf(filename, sizeof(filename), "%s/file.%u", ctx->tmpdir, ff->file_store_id);
             file_fd = open(filename, O_APPEND | O_NOFOLLOW | O_WRONLY);
             if (file_fd == -1) {
                 StatsIncr(tv, aft->fs_error_counter);
-                WARN_ONCE(SC_ERR_OPENING_FILE,
-                        "Filestore (v2) failed to open file %s: %s",
-                        filename, strerror(errno));
+                WARN_ONCE(WOT_OPEN, "Filestore (v2) failed to open file %s: %s", filename,
+                        strerror(errno));
                 return -1;
             }
         } else {
@@ -244,10 +232,10 @@ static int OutputFilestoreLogger(ThreadVars *tv, void *thread_data,
     if (file_fd != -1) {
         ssize_t r = write(file_fd, (const void *)data, (size_t)data_len);
         if (r == -1) {
+            snprintf(filename, sizeof(filename), "%s/file.%u", ctx->tmpdir, ff->file_store_id);
             StatsIncr(tv, aft->fs_error_counter);
-            WARN_ONCE(SC_ERR_FWRITE,
-                    "Filestore (v2) failed to write to %s: %s",
-                    filename, strerror(errno));
+            WARN_ONCE(WOT_WRITE, "Filestore (v2) failed to write to %s: %s", filename,
+                    strerror(errno));
             if (ff->fd != -1) {
                 SC_ATOMIC_SUB(filestore_open_file_cnt, 1);
             }
@@ -264,7 +252,7 @@ static int OutputFilestoreLogger(ThreadVars *tv, void *thread_data,
             ff->fd = -1;
             SC_ATOMIC_SUB(filestore_open_file_cnt, 1);
         }
-        OutputFilestoreFinalizeFiles(tv, aft, ctx, p, ff, dir);
+        OutputFilestoreFinalizeFiles(tv, aft, ctx, p, ff, tx, tx_id, dir);
     }
 
     return 0;
@@ -273,10 +261,9 @@ static int OutputFilestoreLogger(ThreadVars *tv, void *thread_data,
 static TmEcode OutputFilestoreLogThreadInit(ThreadVars *t, const void *initdata,
         void **data)
 {
-    OutputFilestoreLogThread *aft = SCMalloc(sizeof(OutputFilestoreLogThread));
+    OutputFilestoreLogThread *aft = SCCalloc(1, sizeof(OutputFilestoreLogThread));
     if (unlikely(aft == NULL))
         return TM_ECODE_FAILED;
-    memset(aft, 0, sizeof(OutputFilestoreLogThread));
 
     if (initdata == NULL) {
         SCLogDebug("Error getting context for LogFileStore. \"initdata\" argument NULL");
@@ -292,7 +279,7 @@ static TmEcode OutputFilestoreLogThreadInit(ThreadVars *t, const void *initdata,
 
     /* File system type errors (open, write, rename) will only be
      * logged once. But this stat will be incremented for every
-     * occurence. */
+     * occurrence. */
     aft->fs_error_counter = StatsRegisterCounter("file_store.fs_errors", t);
 
     *data = (void *)aft;
@@ -323,9 +310,9 @@ static void OutputFilestoreLogDeInitCtx(OutputCtx *output_ctx)
     SCFree(output_ctx);
 }
 
-static void GetLogDirectory(const ConfNode *conf, char *out, size_t out_size)
+static void GetLogDirectory(const SCConfNode *conf, char *out, size_t out_size)
 {
-    const char *log_base_dir = ConfNodeLookupChildValue(conf, "dir");
+    const char *log_base_dir = SCConfNodeLookupChildValue(conf, "dir");
     if (log_base_dir == NULL) {
         SCLogConfig("Filestore (v2) default log directory %s", default_log_dir);
         log_base_dir = default_log_dir;
@@ -333,7 +320,7 @@ static void GetLogDirectory(const ConfNode *conf, char *out, size_t out_size)
     if (PathIsAbsolute(log_base_dir)) {
         strlcpy(out, log_base_dir, out_size);
     } else {
-        const char *default_log_prefix = ConfigGetLogDirectory();
+        const char *default_log_prefix = SCConfigGetLogDirectory();
         snprintf(out, out_size, "%s/%s", default_log_prefix, log_base_dir);
     }
 }
@@ -345,9 +332,7 @@ static bool InitFilestoreDirectory(const char *dir)
     if (!SCPathExists(dir)) {
         SCLogInfo("Filestore (v2) creating directory %s", dir);
         if (SCCreateDirectoryTree(dir, true) != 0) {
-            SCLogError(SC_ERR_CREATE_DIRECTORY,
-                    "Filestore (v2) failed to create directory %s: %s", dir,
-                    strerror(errno));
+            SCLogError("Filestore (v2) failed to create directory %s: %s", dir, strerror(errno));
             return false;
         }
     }
@@ -356,17 +341,15 @@ static bool InitFilestoreDirectory(const char *dir)
         char leaf[PATH_MAX];
         int n = snprintf(leaf, sizeof(leaf), "%s/%02x", dir, i);
         if (n < 0 || n >= PATH_MAX) {
-            SCLogError(SC_ERR_CREATE_DIRECTORY,
-                    "Filestore (v2) failed to create leaf directory: "
-                    "path too long");
+            SCLogError("Filestore (v2) failed to create leaf directory: "
+                       "path too long");
             return false;
         }
         if (!SCPathExists(leaf)) {
             SCLogInfo("Filestore (v2) creating directory %s", leaf);
             if (SCDefaultMkDir(leaf) != 0) {
-                SCLogError(SC_ERR_CREATE_DIRECTORY,
-                        "Filestore (v2) failed to create directory %s: %s",
-                        leaf, strerror(errno));
+                SCLogError(
+                        "Filestore (v2) failed to create directory %s: %s", leaf, strerror(errno));
                 return false;
             }
         }
@@ -376,16 +359,13 @@ static bool InitFilestoreDirectory(const char *dir)
     char tmpdir[PATH_MAX];
     int n = snprintf(tmpdir, sizeof(tmpdir), "%s/tmp", dir);
     if (n < 0 || n >= PATH_MAX) {
-        SCLogError(SC_ERR_CREATE_DIRECTORY,
-                "Filestore (v2) failed to create tmp directory: path too long");
+        SCLogError("Filestore (v2) failed to create tmp directory: path too long");
         return false;
     }
     if (!SCPathExists(tmpdir)) {
         SCLogInfo("Filestore (v2) creating directory %s", tmpdir);
         if (SCDefaultMkDir(tmpdir) != 0) {
-            SCLogError(SC_ERR_CREATE_DIRECTORY,
-                    "Filestore (v2) failed to create directory %s: %s", tmpdir,
-                    strerror(errno));
+            SCLogError("Filestore (v2) failed to create directory %s: %s", tmpdir, strerror(errno));
             return false;
         }
     }
@@ -397,20 +377,19 @@ static bool InitFilestoreDirectory(const char *dir)
  *  \param conf Pointer to ConfNode containing this loggers configuration.
  *  \return NULL if failure, OutputFilestoreCtx* to the file_ctx if succesful
  * */
-static OutputInitResult OutputFilestoreLogInitCtx(ConfNode *conf)
+static OutputInitResult OutputFilestoreLogInitCtx(SCConfNode *conf)
 {
     OutputInitResult result = { NULL, false };
 
     intmax_t version = 0;
-    if (!ConfGetChildValueInt(conf, "version", &version) || version < 2) {
-        result.ok = true;
+    if (!SCConfGetChildValueInt(conf, "version", &version) || version < 2) {
+        SCLogWarning("File-store v1 has been removed. Please update to file-store v2.");
         return result;
     }
 
     if (RunModeOutputFiledataEnabled()) {
-        SCLogWarning(SC_ERR_NOT_SUPPORTED,
-                "A file data logger is already enabled. Filestore (v2) "
-                "will not be enabled.");
+        SCLogWarning("A file data logger is already enabled. Filestore (v2) "
+                     "will not be enabled.");
         return result;
     }
 
@@ -429,7 +408,7 @@ static OutputInitResult OutputFilestoreLogInitCtx(ConfNode *conf)
     int written = snprintf(ctx->tmpdir, sizeof(ctx->tmpdir) - 1, "%s/tmp",
             log_directory);
     if (written == sizeof(ctx->tmpdir)) {
-        SCLogError(SC_ERR_SPRINTF, "File-store output directory overflow.");
+        SCLogError("File-store output directory overflow.");
         SCFree(ctx);
         return result;
     }
@@ -441,6 +420,7 @@ static OutputInitResult OutputFilestoreLogInitCtx(ConfNode *conf)
 
     OutputCtx *output_ctx = SCCalloc(1, sizeof(OutputCtx));
     if (unlikely(output_ctx == NULL)) {
+        SCFree(ctx->xff_cfg);
         SCFree(ctx);
         return result;
     }
@@ -448,27 +428,20 @@ static OutputInitResult OutputFilestoreLogInitCtx(ConfNode *conf)
     output_ctx->data = ctx;
     output_ctx->DeInit = OutputFilestoreLogDeInitCtx;
 
-    const char *write_fileinfo = ConfNodeLookupChildValue(conf,
-            "write-fileinfo");
-    if (write_fileinfo != NULL && ConfValIsTrue(write_fileinfo)) {
-#ifdef HAVE_LIBJANSSON
+    const char *write_fileinfo = SCConfNodeLookupChildValue(conf, "write-fileinfo");
+    if (write_fileinfo != NULL && SCConfValIsTrue(write_fileinfo)) {
         SCLogConfig("Filestore (v2) will output fileinfo records.");
         ctx->fileinfo = true;
-#else
-        SCLogWarning(SC_ERR_NO_JSON_SUPPORT,
-                "Filestore (v2) requires JSON support to log fileinfo records");
-#endif
     }
 
-    const char *force_filestore = ConfNodeLookupChildValue(conf,
-            "force-filestore");
-    if (force_filestore != NULL && ConfValIsTrue(force_filestore)) {
+    const char *force_filestore = SCConfNodeLookupChildValue(conf, "force-filestore");
+    if (force_filestore != NULL && SCConfValIsTrue(force_filestore)) {
         FileForceFilestoreEnable();
         SCLogInfo("forcing filestore of all files");
     }
 
-    const char *force_magic = ConfNodeLookupChildValue(conf, "force-magic");
-    if (force_magic != NULL && ConfValIsTrue(force_magic)) {
+    const char *force_magic = SCConfNodeLookupChildValue(conf, "force-magic");
+    if (force_magic != NULL && SCConfValIsTrue(force_magic)) {
         FileForceMagicEnable();
         SCLogConfig("Filestore (v2) forcing magic lookup for stored files");
     }
@@ -478,32 +451,40 @@ static OutputInitResult OutputFilestoreLogInitCtx(ConfNode *conf)
     /* The new filestore requires SHA256. */
     FileForceSha256Enable();
 
-    const char *stream_depth_str = ConfNodeLookupChildValue(conf,
-            "stream-depth");
+    ProvidesFeature(FEATURE_OUTPUT_FILESTORE);
+
+    const char *stream_depth_str = SCConfNodeLookupChildValue(conf, "stream-depth");
     if (stream_depth_str != NULL && strcmp(stream_depth_str, "no")) {
         uint32_t stream_depth = 0;
         if (ParseSizeStringU32(stream_depth_str,
                                &stream_depth) < 0) {
-            SCLogError(SC_ERR_SIZE_PARSE, "Error parsing "
+            SCLogError("Error parsing "
                        "file-store.stream-depth "
                        "from conf file - %s.  Killing engine",
-                       stream_depth_str);
+                    stream_depth_str);
             exit(EXIT_FAILURE);
-        } else {
-            FileReassemblyDepthEnable(stream_depth);
+        }
+        if (stream_depth) {
+            if (stream_depth <= stream_config.reassembly_depth) {
+                SCLogWarning("file-store.stream-depth value %" PRIu32 " has "
+                             "no effect since it's less than stream.reassembly.depth "
+                             "value.",
+                        stream_depth);
+            } else {
+                FileReassemblyDepthEnable(stream_depth);
+            }
         }
     }
 
-    const char *file_count_str = ConfNodeLookupChildValue(conf,
-            "max-open-files");
+    const char *file_count_str = SCConfNodeLookupChildValue(conf, "max-open-files");
     if (file_count_str != NULL) {
         uint32_t file_count = 0;
         if (ParseSizeStringU32(file_count_str,
                                &file_count) < 0) {
-            SCLogError(SC_ERR_SIZE_PARSE, "Error parsing "
+            SCLogError("Error parsing "
                        "file-store.max-open-files "
                        "from conf file - %s.  Killing engine",
-                       stream_depth_str);
+                    file_count_str);
             exit(EXIT_FAILURE);
         } else {
             if (file_count != 0) {
@@ -514,25 +495,22 @@ static OutputInitResult OutputFilestoreLogInitCtx(ConfNode *conf)
         }
     }
 
-    StatsRegisterGlobalCounter("file_store.open_files",
-            OutputFilestoreOpenFilesCounter);
-
     result.ctx = output_ctx;
     result.ok = true;
     SCReturnCT(result, "OutputInitResult");
 }
 
-#endif /* HAVE_NSS */
-
 void OutputFilestoreRegister(void)
 {
-#ifdef HAVE_NSS
     OutputRegisterFiledataModule(LOGGER_FILE_STORE, MODULE_NAME, "file-store",
-            OutputFilestoreLogInitCtx, OutputFilestoreLogger,
-            OutputFilestoreLogThreadInit, OutputFilestoreLogThreadDeinit,
-            NULL);
+            OutputFilestoreLogInitCtx, OutputFilestoreLogger, OutputFilestoreLogThreadInit,
+            OutputFilestoreLogThreadDeinit);
 
     SC_ATOMIC_INIT(filestore_open_file_cnt);
     SC_ATOMIC_SET(filestore_open_file_cnt, 0);
-#endif
+}
+
+void OutputFilestoreRegisterGlobalCounters(void)
+{
+    StatsRegisterGlobalCounter("file_store.open_files", OutputFilestoreOpenFilesCounter);
 }
