@@ -15,12 +15,11 @@
  * 02110-1301, USA.
  */
 
-#ifndef __OUTPUT_JSON_DNP3_OBJECTS_H__
-#define __OUTPUT_JSON_DNP3_OBJECTS_H__
+#ifndef SURICATA_OUTPUT_JSON_DNP3_OBJECTS_H
+#define SURICATA_OUTPUT_JSON_DNP3_OBJECTS_H
 
-#ifdef HAVE_LIBJANSSON
-void OutputJsonDNP3SetItem(json_t *js, DNP3Object *object,
-    DNP3Point *item);
-#endif /* HAVE_LIBJANSSON */
+#include "rust-bindings.h"
 
-#endif /* __OUTPUT_JSON_DNP3_OBJECTS_H__ */
+void OutputJsonDNP3SetItem(SCJsonBuilder *js, DNP3Object *object, DNP3Point *item);
+
+#endif /* SURICATA_OUTPUT_JSON_DNP3_OBJECTS_H */
