@@ -25,7 +25,7 @@
  */
 
 #include "suricata-common.h"
-#include "debug.h"
+#include "packet.h"
 #include "detect.h"
 #include "flow.h"
 #include "conf.h"
@@ -56,21 +56,21 @@
 #include "util-time.h"
 #include "util-buffer.h"
 
+#include "action-globals.h"
+
 #define MODULE_NAME "JsonDropLog"
 
 #define LOG_DROP_ALERTS  BIT_U8(1)
 #define LOG_DROP_VERDICT BIT_U8(2)
 
 typedef struct JsonDropOutputCtx_ {
-    LogFileCtx *file_ctx;
     uint8_t flags;
-    OutputJsonCommonSettings cfg;
+    OutputJsonCtx *eve_ctx;
 } JsonDropOutputCtx;
 
 typedef struct JsonDropLogThread_ {
-    LogFileCtx *file_ctx;
     JsonDropOutputCtx *drop_ctx;
-    MemBuffer *buffer;
+    OutputJsonThreadCtx *ctx;
 } JsonDropLogThread;
 
 /* default to true as this has been the default behavior for a long time */
@@ -83,78 +83,85 @@ static int g_droplog_flows_start = 1;
  * \param tv    Pointer the current thread variables
  * \param p     Pointer the packet which is being logged
  *
- * \return return TM_EODE_OK on success
+ * \return return TM_ECODE_OK on success
  */
-static int DropLogJSON (JsonDropLogThread *aft, const Packet *p)
+static int DropLogJSON(ThreadVars *tv, JsonDropLogThread *aft, const Packet *p)
 {
     JsonDropOutputCtx *drop_ctx = aft->drop_ctx;
 
     JsonAddrInfo addr = json_addr_info_zero;
     JsonAddrInfoInit(p, LOG_DIR_PACKET, &addr);
 
-    JsonBuilder *js = CreateEveHeader(p, LOG_DIR_PACKET, "drop", &addr);
+    SCJsonBuilder *js = CreateEveHeader(p, LOG_DIR_PACKET, "drop", &addr, drop_ctx->eve_ctx);
     if (unlikely(js == NULL))
         return TM_ECODE_OK;
 
-    EveAddCommonOptions(&drop_ctx->cfg, p, p->flow, js);
+    if (p->flow != NULL) {
+        if (p->flowflags & FLOW_PKT_TOSERVER) {
+            SCJbSetString(js, "direction", "to_server");
+        } else {
+            SCJbSetString(js, "direction", "to_client");
+        }
+    }
 
-    jb_open_object(js, "drop");
-
-    /* reset */
-    MemBufferReset(aft->buffer);
+    SCJbOpenObject(js, "drop");
 
     uint16_t proto = 0;
-    if (PKT_IS_IPV4(p)) {
-        jb_set_uint(js, "len", IPV4_GET_IPLEN(p));
-        jb_set_uint(js, "tos", IPV4_GET_IPTOS(p));
-        jb_set_uint(js, "ttl", IPV4_GET_IPTTL(p));
-        jb_set_uint(js, "ipid", IPV4_GET_IPID(p));
-        proto = IPV4_GET_IPPROTO(p);
-    } else if (PKT_IS_IPV6(p)) {
-        jb_set_uint(js, "len", IPV6_GET_PLEN(p));
-        jb_set_uint(js, "tc", IPV6_GET_CLASS(p));
-        jb_set_uint(js, "hoplimit", IPV6_GET_HLIM(p));
-        jb_set_uint(js, "flowlbl", IPV6_GET_FLOW(p));
+    if (PacketIsIPv4(p)) {
+        const IPV4Hdr *ip4h = PacketGetIPv4(p);
+        SCJbSetUint(js, "len", IPV4_GET_RAW_IPLEN(ip4h));
+        SCJbSetUint(js, "tos", IPV4_GET_RAW_IPTOS(ip4h));
+        SCJbSetUint(js, "ttl", IPV4_GET_RAW_IPTTL(ip4h));
+        SCJbSetUint(js, "ipid", IPV4_GET_RAW_IPID(ip4h));
+        proto = IPV4_GET_RAW_IPPROTO(ip4h);
+    } else if (PacketIsIPv6(p)) {
+        const IPV6Hdr *ip6h = PacketGetIPv6(p);
+        SCJbSetUint(js, "len", IPV6_GET_RAW_PLEN(ip6h));
+        SCJbSetUint(js, "tc", IPV6_GET_RAW_CLASS(ip6h));
+        SCJbSetUint(js, "hoplimit", IPV6_GET_RAW_HLIM(ip6h));
+        SCJbSetUint(js, "flowlbl", IPV6_GET_RAW_FLOW(ip6h));
         proto = IPV6_GET_L4PROTO(p);
     }
     switch (proto) {
         case IPPROTO_TCP:
-            if (PKT_IS_TCP(p)) {
-                jb_set_uint(js, "tcpseq", TCP_GET_SEQ(p));
-                jb_set_uint(js, "tcpack", TCP_GET_ACK(p));
-                jb_set_uint(js, "tcpwin", TCP_GET_WINDOW(p));
-                jb_set_bool(js, "syn", TCP_ISSET_FLAG_SYN(p) ? true : false);
-                jb_set_bool(js, "ack", TCP_ISSET_FLAG_ACK(p) ? true : false);
-                jb_set_bool(js, "psh", TCP_ISSET_FLAG_PUSH(p) ? true : false);
-                jb_set_bool(js, "rst", TCP_ISSET_FLAG_RST(p) ? true : false);
-                jb_set_bool(js, "urg", TCP_ISSET_FLAG_URG(p) ? true : false);
-                jb_set_bool(js, "fin", TCP_ISSET_FLAG_FIN(p) ? true : false);
-                jb_set_uint(js, "tcpres",  TCP_GET_RAW_X2(p->tcph));
-                jb_set_uint(js, "tcpurgp", TCP_GET_URG_POINTER(p));
+            if (PacketIsTCP(p)) {
+                const TCPHdr *tcph = PacketGetTCP(p);
+                SCJbSetUint(js, "tcpseq", TCP_GET_RAW_SEQ(tcph));
+                SCJbSetUint(js, "tcpack", TCP_GET_RAW_ACK(tcph));
+                SCJbSetUint(js, "tcpwin", TCP_GET_RAW_WINDOW(tcph));
+                SCJbSetBool(js, "syn", TCP_ISSET_FLAG_RAW_SYN(tcph) ? true : false);
+                SCJbSetBool(js, "ack", TCP_ISSET_FLAG_RAW_ACK(tcph) ? true : false);
+                SCJbSetBool(js, "psh", TCP_ISSET_FLAG_RAW_PUSH(tcph) ? true : false);
+                SCJbSetBool(js, "rst", TCP_ISSET_FLAG_RAW_RST(tcph) ? true : false);
+                SCJbSetBool(js, "urg", TCP_ISSET_FLAG_RAW_URG(tcph) ? true : false);
+                SCJbSetBool(js, "fin", TCP_ISSET_FLAG_RAW_FIN(tcph) ? true : false);
+                SCJbSetUint(js, "tcpres", TCP_GET_RAW_X2(tcph));
+                SCJbSetUint(js, "tcpurgp", TCP_GET_RAW_URG_POINTER(tcph));
             }
             break;
         case IPPROTO_UDP:
-            if (PKT_IS_UDP(p)) {
-                jb_set_uint(js, "udplen", UDP_GET_LEN(p));
+            if (PacketIsUDP(p)) {
+                const UDPHdr *udph = PacketGetUDP(p);
+                SCJbSetUint(js, "udplen", UDP_GET_RAW_LEN(udph));
             }
             break;
         case IPPROTO_ICMP:
-            if (PKT_IS_ICMPV4(p)) {
-                jb_set_uint(js, "icmp_id", ICMPV4_GET_ID(p));
-                jb_set_uint(js, "icmp_seq", ICMPV4_GET_SEQ(p));
-            } else if(PKT_IS_ICMPV6(p)) {
-                jb_set_uint(js, "icmp_id", ICMPV6_GET_ID(p));
-                jb_set_uint(js, "icmp_seq", ICMPV6_GET_SEQ(p));
+            if (PacketIsICMPv4(p)) {
+                SCJbSetUint(js, "icmp_id", ICMPV4_GET_ID(p));
+                SCJbSetUint(js, "icmp_seq", ICMPV4_GET_SEQ(p));
+            } else if (PacketIsICMPv6(p)) {
+                SCJbSetUint(js, "icmp_id", ICMPV6_GET_ID(p));
+                SCJbSetUint(js, "icmp_seq", ICMPV6_GET_SEQ(p));
             }
             break;
     }
     if (p->drop_reason != 0) {
         const char *str = PacketDropReasonToString(p->drop_reason);
-        jb_set_string(js, "reason", str);
+        SCJbSetString(js, "reason", str);
     }
 
     /* Close drop. */
-    jb_close(js);
+    SCJbClose(js);
 
     if (aft->drop_ctx->flags & LOG_DROP_VERDICT) {
         EveAddVerdict(js, p);
@@ -171,7 +178,7 @@ static int DropLogJSON (JsonDropLogThread *aft, const Packet *p)
             if ((pa->action & (ACTION_REJECT|ACTION_REJECT_DST|ACTION_REJECT_BOTH)) ||
                ((pa->action & ACTION_DROP) && EngineModeIsIPS()))
             {
-                AlertJsonHeader(NULL, p, pa, js, 0, &addr);
+                AlertJsonHeader(p, pa, js, 0, &addr, NULL);
                 logged = 1;
                 break;
             }
@@ -179,13 +186,13 @@ static int DropLogJSON (JsonDropLogThread *aft, const Packet *p)
         if (logged == 0) {
             if (p->alerts.drop.action != 0) {
                 const PacketAlert *pa = &p->alerts.drop;
-                AlertJsonHeader(NULL, p, pa, js, 0, &addr);
+                AlertJsonHeader(p, pa, js, 0, &addr, NULL);
             }
         }
     }
 
-    OutputJsonBuilderBuffer(js, aft->file_ctx, &aft->buffer);
-    jb_free(js);
+    OutputJsonBuilderBuffer(tv, p, p->flow, js, aft->ctx);
+    SCJbFree(js);
 
     return TM_ECODE_OK;
 }
@@ -202,15 +209,10 @@ static TmEcode JsonDropLogThreadInit(ThreadVars *t, const void *initdata, void *
         goto error_exit;
     }
 
-    aft->buffer = MemBufferCreateNew(JSON_OUTPUT_BUFFER_SIZE);
-    if (aft->buffer == NULL) {
-        goto error_exit;
-    }
-
-    /** Use the Ouptut Context (file pointer and mutex) */
+    /** Use the Output Context (file pointer and mutex) */
     aft->drop_ctx = ((OutputCtx *)initdata)->data;
-    aft->file_ctx = LogFileEnsureExists(aft->drop_ctx->file_ctx, t->id);
-    if (!aft->file_ctx) {
+    aft->ctx = CreateEveThreadCtx(t, aft->drop_ctx->eve_ctx);
+    if (!aft->ctx) {
         goto error_exit;
     }
 
@@ -218,9 +220,6 @@ static TmEcode JsonDropLogThreadInit(ThreadVars *t, const void *initdata, void *
     return TM_ECODE_OK;
 
 error_exit:
-    if (aft->buffer != NULL) {
-        MemBufferFree(aft->buffer);
-    }
     SCFree(aft);
     return TM_ECODE_FAILED;
 }
@@ -232,7 +231,7 @@ static TmEcode JsonDropLogThreadDeinit(ThreadVars *t, void *data)
         return TM_ECODE_OK;
     }
 
-    MemBufferFree(aft->buffer);
+    FreeEveThreadCtx(aft->ctx);
 
     /* clear memory */
     memset(aft, 0, sizeof(*aft));
@@ -244,8 +243,6 @@ static TmEcode JsonDropLogThreadDeinit(ThreadVars *t, void *data)
 static void JsonDropOutputCtxFree(JsonDropOutputCtx *drop_ctx)
 {
     if (drop_ctx != NULL) {
-        if (drop_ctx->file_ctx != NULL)
-            LogFileFreeCtx(drop_ctx->file_ctx);
         SCFree(drop_ctx);
     }
 }
@@ -260,12 +257,12 @@ static void JsonDropLogDeInitCtxSub(OutputCtx *output_ctx)
     SCFree(output_ctx);
 }
 
-static OutputInitResult JsonDropLogInitCtxSub(ConfNode *conf, OutputCtx *parent_ctx)
+static OutputInitResult JsonDropLogInitCtxSub(SCConfNode *conf, OutputCtx *parent_ctx)
 {
     OutputInitResult result = { NULL, false };
     if (OutputDropLoggerEnable() != 0) {
-        SCLogError(SC_ERR_CONF_YAML_ERROR, "only one 'drop' logger "
-            "can be enabled");
+        SCLogError("only one 'drop' logger "
+                   "can be enabled");
         return result;
     }
 
@@ -282,33 +279,32 @@ static OutputInitResult JsonDropLogInitCtxSub(ConfNode *conf, OutputCtx *parent_
     }
 
     if (conf) {
-        const char *extended = ConfNodeLookupChildValue(conf, "alerts");
+        const char *extended = SCConfNodeLookupChildValue(conf, "alerts");
         if (extended != NULL) {
-            if (ConfValIsTrue(extended)) {
+            if (SCConfValIsTrue(extended)) {
                 drop_ctx->flags |= LOG_DROP_ALERTS;
             }
         }
-        extended = ConfNodeLookupChildValue(conf, "flows");
+        extended = SCConfNodeLookupChildValue(conf, "flows");
         if (extended != NULL) {
             if (strcasecmp(extended, "start") == 0) {
                 g_droplog_flows_start = 1;
             } else if (strcasecmp(extended, "all") == 0) {
                 g_droplog_flows_start = 0;
             } else {
-                SCLogWarning(SC_ERR_CONF_YAML_ERROR, "valid options for "
-                        "'flow' are 'start' and 'all'");
+                SCLogWarning("valid options for "
+                             "'flow' are 'start' and 'all'");
             }
         }
-        extended = ConfNodeLookupChildValue(conf, "verdict");
+        extended = SCConfNodeLookupChildValue(conf, "verdict");
         if (extended != NULL) {
-            if (ConfValIsTrue(extended)) {
+            if (SCConfValIsTrue(extended)) {
                 drop_ctx->flags |= LOG_DROP_VERDICT;
             }
         }
     }
 
-    drop_ctx->file_ctx = ajt->file_ctx;
-    drop_ctx->cfg = ajt->cfg;
+    drop_ctx->eve_ctx = ajt;
 
     output_ctx->data = drop_ctx;
     output_ctx->DeInit = JsonDropLogDeInitCtxSub;
@@ -325,12 +321,12 @@ static OutputInitResult JsonDropLogInitCtxSub(ConfNode *conf, OutputCtx *parent_
  * \param data  Pointer to the droplog struct
  * \param p     Pointer the packet which is being logged
  *
- * \retval 0 on succes
+ * \retval 0 on success
  */
 static int JsonDropLogger(ThreadVars *tv, void *thread_data, const Packet *p)
 {
     JsonDropLogThread *td = thread_data;
-    int r = DropLogJSON(td, p);
+    int r = DropLogJSON(tv, td, p);
     if (r < 0)
         return -1;
 
@@ -348,55 +344,61 @@ static int JsonDropLogger(ThreadVars *tv, void *thread_data, const Packet *p)
     return 0;
 }
 
-
 /**
  * \brief Check if we need to drop-log this packet
  *
  * \param tv    Pointer the current thread variables
  * \param p     Pointer the packet which is tested
  *
- * \retval bool TRUE or FALSE
+ * \retval bool true or false
  */
-static int JsonDropLogCondition(ThreadVars *tv, const Packet *p)
+static bool JsonDropLogCondition(ThreadVars *tv, void *data, const Packet *p)
 {
     if (!EngineModeIsIPS()) {
         SCLogDebug("engine is not running in inline mode, so returning");
-        return FALSE;
+        return false;
     }
     if (PKT_IS_PSEUDOPKT(p)) {
         SCLogDebug("drop log doesn't log pseudo packets");
-        return FALSE;
+        return false;
     }
 
-    if (!PACKET_TEST_ACTION(p, ACTION_DROP)) {
-        return FALSE;
+    if (!(PacketCheckAction(p, ACTION_DROP))) {
+        return false;
     }
 
     if (g_droplog_flows_start && p->flow != NULL) {
-        int ret = FALSE;
+        bool ret = false;
 
         /* for a flow that will be dropped fully, log just once per direction */
         if (p->flow->flags & FLOW_ACTION_DROP) {
             if (PKT_IS_TOSERVER(p) && !(p->flow->flags & FLOW_TOSERVER_DROP_LOGGED))
-                ret = TRUE;
+                ret = true;
             else if (PKT_IS_TOCLIENT(p) && !(p->flow->flags & FLOW_TOCLIENT_DROP_LOGGED))
-                ret = TRUE;
+                ret = true;
         }
 
         /* if drop is caused by signature, log anyway */
         if (p->alerts.drop.action != 0)
-            ret = TRUE;
+            ret = true;
 
         return ret;
     }
 
-    return TRUE;
+    return true;
 }
 
 void JsonDropLogRegister (void)
 {
-    OutputRegisterPacketSubModule(LOGGER_JSON_DROP, "eve-log", MODULE_NAME,
-        "eve-log.drop", JsonDropLogInitCtxSub, JsonDropLogger,
-        JsonDropLogCondition, JsonDropLogThreadInit, JsonDropLogThreadDeinit,
-        NULL);
+    OutputPacketLoggerFunctions output_logger_functions = {
+        .LogFunc = JsonDropLogger,
+        .FlushFunc = OutputJsonLogFlush,
+        .ConditionFunc = JsonDropLogCondition,
+        .ThreadInitFunc = JsonDropLogThreadInit,
+        .ThreadDeinitFunc = JsonDropLogThreadDeinit,
+        .ThreadExitPrintStatsFunc = NULL,
+    };
+
+    OutputRegisterPacketSubModule(LOGGER_JSON_DROP, "eve-log", MODULE_NAME, "eve-log.drop",
+            JsonDropLogInitCtxSub, &output_logger_functions);
 }
