@@ -238,14 +238,12 @@ static void EveHttpLogJSONBasic(SCJsonBuilder *js, htp_tx_t *tx)
     if (htp_tx_response_headers(tx) != NULL) {
         const htp_header_t *h_content_type = htp_tx_response_header(tx, "content-type");
         if (h_content_type != NULL) {
-            const size_t size = htp_header_value_len(h_content_type) * 2 + 1;
-            char string[size];
-            BytesToStringBuffer(htp_header_value_ptr(h_content_type),
-                    htp_header_value_len(h_content_type), string, size);
-            char *p = strchr(string, ';');
+            uint32_t len = (uint32_t)htp_header_value_len(h_content_type);
+            const uint8_t *p = memchr(htp_header_value_ptr(h_content_type), ';', len);
             if (p != NULL)
-                *p = '\0';
-            SCJbSetString(js, "http_content_type", string);
+                len = (uint32_t)(p - htp_header_value_ptr(h_content_type));
+            SCJbSetStringFromBytes(
+                    js, "http_content_type", htp_header_value_ptr(h_content_type), len);
         }
         const htp_header_t *h_content_range = htp_tx_response_header(tx, "content-range");
         if (h_content_range != NULL) {
@@ -370,7 +368,6 @@ static void EveHttpLogJSONHeaders(
 static void BodyPrintableBuffer(SCJsonBuilder *js, HtpBody *body, const char *key)
 {
     if (body->sb != NULL && body->sb->region.buf != NULL) {
-        uint32_t offset = 0;
         const uint8_t *body_data;
         uint32_t body_data_len;
         uint64_t body_offset;
@@ -380,11 +377,7 @@ static void BodyPrintableBuffer(SCJsonBuilder *js, HtpBody *body, const char *ke
             return;
         }
 
-        uint8_t printable_buf[body_data_len + 1];
-        PrintStringsToBuffer(printable_buf, &offset, body_data_len + 1, body_data, body_data_len);
-        if (offset > 0) {
-            SCJbSetString(js, key, (char *)printable_buf);
-        }
+        SCJbSetPrintAsciiString(js, key, body_data, body_data_len);
     }
 }
 
