@@ -1,4 +1,4 @@
-/* Copyright (C) 2017 Open Information Security Foundation
+/* Copyright (C) 2017-2020 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -21,12 +21,12 @@
  * \author Victor Julien <victor@inliniac.net>
  */
 
-#ifndef __OUTPUT_JSON_NFS_H__
-#define __OUTPUT_JSON_NFS_H__
+#ifndef SURICATA_OUTPUT_JSON_NFS_H
+#define SURICATA_OUTPUT_JSON_NFS_H
 
 void JsonNFSLogRegister(void);
-#ifdef HAVE_RUST
-json_t *JsonNFSAddMetadataRPC(const Flow *f, uint64_t tx_id);
-json_t *JsonNFSAddMetadata(const Flow *f, uint64_t tx_id);
-#endif /* HAVE_RUST */
-#endif /* __OUTPUT_JSON_NFS_H__ */
+
+bool EveNFSAddMetadataRPC(const Flow *f, uint64_t tx_id, SCJsonBuilder *jb);
+bool EveNFSAddMetadata(const Flow *f, uint64_t tx_id, SCJsonBuilder *jb);
+
+#endif /* SURICATA_OUTPUT_JSON_NFS_H */
