@@ -23,14 +23,17 @@
  * Stats Logger Output registration functions
  */
 
-#ifndef __OUTPUT_STATS_H__
-#define __OUTPUT_STATS_H__
+#ifndef SURICATA_OUTPUT_STATS_H
+#define SURICATA_OUTPUT_STATS_H
+
+#include "tm-modules.h"
 
 typedef struct StatsRecord_ {
     const char *name;
+    const char *short_name;
     const char *tm_name;
-    uint64_t value;         /**< total value */
-    uint64_t pvalue;        /**< prev value (may be higher for memuse counters) */
+    int64_t value;  /**< total value */
+    int64_t pvalue; /**< prev value (may be higher for memuse counters) */
 } StatsRecord;
 
 typedef struct StatsTable_ {
@@ -46,9 +49,8 @@ TmEcode OutputStatsLog(ThreadVars *tv, void *thread_data, StatsTable *st);
 
 typedef int (*StatsLogger)(ThreadVars *, void *thread_data, const StatsTable *);
 
-int OutputRegisterStatsLogger(const char *name, StatsLogger LogFunc,
-    OutputCtx *, ThreadInitFunc ThreadInit, ThreadDeinitFunc ThreadDeinit,
-    ThreadExitPrintStatsFunc ThreadExitPrintStats);
+int OutputRegisterStatsLogger(const char *name, StatsLogger LogFunc, OutputCtx *,
+        ThreadInitFunc ThreadInit, ThreadDeinitFunc ThreadDeinit);
 
 void TmModuleStatsLoggerRegister (void);
 
@@ -56,4 +58,4 @@ int OutputStatsLoggersRegistered(void);
 
 void OutputStatsShutdown(void);
 
-#endif /* __OUTPUT_STATS_H__ */
+#endif /* SURICATA_OUTPUT_STATS_H */
