@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2010 Open Information Security Foundation
+/* Copyright (C) 2007-2019 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -30,6 +30,7 @@
 #include "suricata.h"
 #include "util-var.h"
 #include "pkt-var.h"
+#include "util-validate.h"
 
 #ifdef DEBUG
 void PacketQueueValidateDebug(PacketQueue *q);
@@ -136,7 +137,7 @@ void PacketQueueValidate(PacketQueue *q)
 }
 #endif /* DEBUG */
 
-void PacketEnqueue (PacketQueue *q, Packet *p)
+static inline void PacketEnqueueDo(PacketQueue *q, Packet *p)
 {
     //PacketQueueValidateDebug(q);
 
@@ -164,25 +165,29 @@ void PacketEnqueue (PacketQueue *q, Packet *p)
     //PacketQueueValidateDebug(q);
 }
 
-Packet *PacketDequeue (PacketQueue *q)
+void PacketEnqueueNoLock(PacketQueueNoLock *qnl, Packet *p)
 {
-    Packet *p = NULL;
+    DEBUG_VALIDATE_BUG_ON(p->pkt_src == 0);
+    PacketQueue *q = (PacketQueue *)qnl;
+    PacketEnqueueDo(q, p);
+}
 
+void PacketEnqueue (PacketQueue *q, Packet *p)
+{
+    PacketEnqueueDo(q, p);
+}
+
+static inline Packet *PacketDequeueDo (PacketQueue *q)
+{
     //PacketQueueValidateDebug(q);
     /* if the queue is empty there are no packets left. */
     if (q->len == 0) {
         return NULL;
     }
-
     q->len--;
 
     /* pull the bottom packet from the queue */
-    p = q->bot;
-
-    /* Weird issue: sometimes it looks that two thread arrive
-     * here at the same time so the bot ptr is NULL (only on OS X?)
-     */
-    BUG_ON (p == NULL);
+    Packet *p = q->bot;
 
     /* more packets in queue */
     if (q->bot->prev != NULL) {
@@ -200,3 +205,32 @@ Packet *PacketDequeue (PacketQueue *q)
     return p;
 }
 
+Packet *PacketDequeueNoLock (PacketQueueNoLock *qnl)
+{
+    PacketQueue *q = (PacketQueue *)qnl;
+    Packet *p = PacketDequeueDo(q);
+    DEBUG_VALIDATE_BUG_ON(p != NULL && p->pkt_src == 0);
+    return p;
+}
+
+Packet *PacketDequeue (PacketQueue *q)
+{
+    return PacketDequeueDo(q);
+}
+
+PacketQueue *PacketQueueAlloc(void)
+{
+    PacketQueue *pq = SCCalloc(1, sizeof(*pq));
+    if (pq == NULL)
+        return NULL;
+    SCMutexInit(&pq->mutex_q, NULL);
+    SCCondInit(&pq->cond_q, NULL);
+    return pq;
+}
+
+void PacketQueueFree(PacketQueue *pq)
+{
+    SCCondDestroy(&pq->cond_q);
+    SCMutexDestroy(&pq->mutex_q);
+    SCFree(pq);
+}
