@@ -20,14 +20,13 @@
  *  \author Eric Leblond <eric@regit.org>
  */
 
-#ifndef __RUNMODE_AF_PACKET_H__
-#define __RUNMODE_AF_PACKET_H__
+#ifndef SURICATA_RUNMODE_AF_PACKET_H
+#define SURICATA_RUNMODE_AF_PACKET_H
 
 int RunModeIdsAFPSingle(void);
 int RunModeIdsAFPAutoFp(void);
 int RunModeIdsAFPWorkers(void);
 void RunModeIdsAFPRegister(void);
 const char *RunModeAFPGetDefaultMode(void);
-int AFPRunModeIsIPS();
 
-#endif /* __RUNMODE_AF_PACKET_H__ */
+#endif /* SURICATA_RUNMODE_AF_PACKET_H */
