@@ -30,8 +30,6 @@
 #include "util-affinity.h"
 #include "util-runmodes.h"
 
-static const char *default_mode;
-
 static int DagConfigGetThreadCount(void *conf)
 {
     return 1;
@@ -44,31 +42,26 @@ static void *ParseDagConfig(const char *iface)
 
 const char *RunModeErfDagGetDefaultMode(void)
 {
-    return default_mode;
+    return "autofp";
 }
 
 void RunModeErfDagRegister(void)
 {
-    default_mode = "autofp";
-
     RunModeRegisterNewRunMode(RUNMODE_DAG, "autofp",
-        "Multi threaded DAG mode.  Packets from "
-        "each flow are assigned to a single detect "
-        "thread, unlike \"dag_auto\" where packets "
-        "from the same flow can be processed by any "
-        "detect thread",
-        RunModeIdsErfDagAutoFp);
+            "Multi threaded DAG mode.  Packets from "
+            "each flow are assigned to a single detect "
+            "thread, unlike \"dag_auto\" where packets "
+            "from the same flow can be processed by any "
+            "detect thread",
+            RunModeIdsErfDagAutoFp, NULL);
 
-    RunModeRegisterNewRunMode(RUNMODE_DAG, "single",
-        "Singled threaded DAG mode",
-        RunModeIdsErfDagSingle);
+    RunModeRegisterNewRunMode(
+            RUNMODE_DAG, "single", "Singled threaded DAG mode", RunModeIdsErfDagSingle, NULL);
 
     RunModeRegisterNewRunMode(RUNMODE_DAG, "workers",
-        "Workers DAG mode, each thread does all "
-        " tasks from acquisition to logging",
-        RunModeIdsErfDagWorkers);
-
-    return;
+            "Workers DAG mode, each thread does all "
+            " tasks from acquisition to logging",
+            RunModeIdsErfDagWorkers, NULL);
 }
 
 int RunModeIdsErfDagSingle(void)
@@ -77,8 +70,6 @@ int RunModeIdsErfDagSingle(void)
 
     SCEnter();
 
-    RunModeInitialize();
-
     TimeModeSetLive();
 
     ret = RunModeSetLiveCaptureSingle(ParseDagConfig,
@@ -88,8 +79,7 @@ int RunModeIdsErfDagSingle(void)
         thread_name_single,
         NULL);
     if (ret != 0) {
-        SCLogError(SC_ERR_RUNMODE, "DAG single runmode failed to start");
-        exit(EXIT_FAILURE);
+        FatalError("DAG single runmode failed to start");
     }
 
     SCLogInfo("RunModeIdsDagSingle initialised");
@@ -103,19 +93,12 @@ int RunModeIdsErfDagAutoFp(void)
 
     SCEnter();
 
-    RunModeInitialize();
-
     TimeModeSetLive();
 
-    ret = RunModeSetLiveCaptureAutoFp(ParseDagConfig,
-        DagConfigGetThreadCount,
-        "ReceiveErfDag",
-        "DecodeErfDag",
-        thread_name_autofp,
-        NULL);
+    ret = RunModeSetLiveCaptureAutoFp(ParseDagConfig, DagConfigGetThreadCount, "ReceiveErfDag",
+            "DecodeErfDag", thread_name_autofp, NULL);
     if (ret != 0) {
-        SCLogError(SC_ERR_RUNMODE, "DAG autofp runmode failed to start");
-        exit(EXIT_FAILURE);
+        FatalError("DAG autofp runmode failed to start");
     }
 
     SCLogInfo("RunModeIdsDagAutoFp initialised");
@@ -129,19 +112,12 @@ int RunModeIdsErfDagWorkers(void)
 
     SCEnter();
 
-    RunModeInitialize();
-
     TimeModeSetLive();
 
-    ret = RunModeSetLiveCaptureWorkers(ParseDagConfig,
-        DagConfigGetThreadCount,
-        "ReceiveErfDag",
-        "DecodeErfDag",
-        thread_name_workers,
-        NULL);
+    ret = RunModeSetLiveCaptureWorkers(ParseDagConfig, DagConfigGetThreadCount, "ReceiveErfDag",
+            "DecodeErfDag", thread_name_workers, NULL);
     if (ret != 0) {
-        SCLogError(SC_ERR_RUNMODE, "DAG workers runmode failed to start");
-        exit(EXIT_FAILURE);
+        FatalError("DAG workers runmode failed to start");
     }
 
     SCLogInfo("RunModeIdsErfDagWorkers initialised");
