@@ -20,13 +20,13 @@
  *  \author Victor Julien <victor@inliniac.net>
  */
 
-#ifndef __RUNMODE_ERF_DAG_H__
-#define __RUNMODE_ERF_DAG_H__
+#ifndef SURICATA_RUNMODE_ERF_DAG_H
+#define SURICATA_RUNMODE_ERF_DAG_H
 
-int RunModeIdsErfDagAutoFp(DetectEngineCtx *);
-int RunModeIdsErfDagSingle(DetectEngineCtx *);
-int RunModeIdsErfDagWorkers(DetectEngineCtx *);
+int RunModeIdsErfDagAutoFp(void);
+int RunModeIdsErfDagSingle(void);
+int RunModeIdsErfDagWorkers(void);
 void RunModeErfDagRegister(void);
 const char *RunModeErfDagGetDefaultMode(void);
 
-#endif /* __RUNMODE_ERF_DAG_H__ */
+#endif /* SURICATA_RUNMODE_ERF_DAG_H */
