@@ -1,4 +1,4 @@
-/* Copyright (C) 2014 Open Information Security Foundation
+/* Copyright (C) 2014-2021 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -20,14 +20,13 @@
 *  \author Aleksey Katargin <gureedo@gmail.com>
 */
 
-#ifndef __RUNMODE_NETMAP_H__
-#define __RUNMODE_NETMAP_H__
+#ifndef SURICATA_RUNMODE_NETMAP_H
+#define SURICATA_RUNMODE_NETMAP_H
 
 int RunModeIdsNetmapSingle(void);
 int RunModeIdsNetmapAutoFp(void);
 int RunModeIdsNetmapWorkers(void);
 void RunModeIdsNetmapRegister(void);
 const char *RunModeNetmapGetDefaultMode(void);
-int NetmapRunModeIsIPS(void);
 
-#endif /* __RUNMODE_NETMAP_H__ */
+#endif /* SURICATA_RUNMODE_NETMAP_H */
