@@ -1,4 +1,4 @@
-/* Copyright (C) 2014 Open Information Security Foundation
+/* Copyright (C) 2014-2021 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -19,12 +19,11 @@
 * \file
 *
 * \author Aleksey Katargin <gureedo@gmail.com>
+* \author Victor Julien <victor@inliniac.net>
 */
 
-#ifndef __SOURCE_NETMAP_H__
-#define __SOURCE_NETMAP_H__
-
-#include "queue.h"
+#ifndef SURICATA_SOURCE_NETMAP_H
+#define SURICATA_SOURCE_NETMAP_H
 
 /* copy modes */
 enum {
@@ -40,10 +39,14 @@ typedef struct NetmapIfaceSettings_
     /* real inner interface name */
     char iface[NETMAP_IFACE_NAME_LENGTH];
 
-    int threads;
     /* sw ring flag for out_iface */
-    int sw_ring;
-    int promisc;
+    bool sw_ring;
+    bool promisc;
+    bool real;      /**< real iface or not. Not in case of vale, pipe */
+    bool ips;       /**< set to true if checksum_mode != NETMAP_COPY_MODE_NONE */
+    bool threads_auto;
+
+    uint16_t threads;
     int copy_mode;
     ChecksumValidationMode checksum_mode;
     const char *bpf_filter;
@@ -57,9 +60,6 @@ typedef struct NetmapIfaceConfig_
     /* settings for out capture device*/
     NetmapIfaceSettings in;
 
-    /* semantic interface name */
-    char *out_iface_name;
-
     /* settings for outgoing iface for IPS/TAP */
     NetmapIfaceSettings out;
 
@@ -69,9 +69,6 @@ typedef struct NetmapIfaceConfig_
 
 typedef struct NetmapPacketVars_
 {
-    int ring_id;
-    int slot_id;
-    int dst_ring_id;
     /* NetmapThreadVars */
     void *ntv;
 } NetmapPacketVars;
@@ -81,4 +78,4 @@ int NetmapGetRSSCount(const char *ifname);
 void TmModuleReceiveNetmapRegister (void);
 void TmModuleDecodeNetmapRegister (void);
 
-#endif /* __SOURCE_NETMAP_H__ */
+#endif /* SURICATA_SOURCE_NETMAP_H */
