@@ -28,10 +28,10 @@
 #include "suricata-common.h"
 #include "suricata.h"
 #include "tm-threads.h"
-
+#include "packet.h"
 #include "util-byte.h"
 #include "util-debug.h"
-#include "util-device.h"
+#include "util-device-private.h"
 #include "util-error.h"
 #include "util-ioctl.h"
 #include "util-privs.h"
@@ -69,6 +69,7 @@ void TmModuleVerdictWinDivertRegister(void)
 {
     tmm_modules[TMM_VERDICTWINDIVERT].name = "VerdictWinDivert";
     tmm_modules[TMM_VERDICTWINDIVERT].ThreadInit = NoWinDivertSupportExit;
+    tmm_modules[TMM_VERDICTWINDIVERT].flags = TM_FLAG_VERDICT_TM;
 }
 
 void TmModuleDecodeWinDivertRegister(void)
@@ -81,21 +82,20 @@ void TmModuleDecodeWinDivertRegister(void)
 TmEcode NoWinDivertSupportExit(ThreadVars *tv, const void *initdata,
                                void **data)
 {
-    SCLogError(
-            SC_ERR_WINDIVERT_NOSUPPORT,
-            "Error creating thread %s: you do not have support for WinDivert "
-            "enabled; please recompile with --enable-windivert",
+    SCLogError("Error creating thread %s: you do not have support for WinDivert "
+               "enabled; please recompile with --enable-windivert",
             tv->name);
     exit(EXIT_FAILURE);
 }
 
 #else /* implied we do have WinDivert support */
+#include "action-globals.h"
+#include "win32-syscall.h"
 
 typedef struct WinDivertThreadVars_ {
     WinDivertHandle filter_handle;
 
     int thread_num;
-    CaptureStats stats;
     int64_t qpc_start_time;
     int64_t qpc_start_count;
     int64_t qpc_freq_usec;
@@ -130,8 +130,12 @@ void *WinDivertGetQueue(int n)
 }
 
 // not defined in MinGW winerror.h
+#ifndef ERROR_INVALID_IMAGE_HASH
 #define ERROR_INVALID_IMAGE_HASH 577L
+#endif
+#ifndef ERROR_DATA_NOT_ACCEPTED
 #define ERROR_DATA_NOT_ACCEPTED 592L
+#endif
 
 /**
  * \brief return an error description for Win32 error values commonly returned
@@ -196,15 +200,13 @@ static const char *WinDivertGetErrorString(DWORD error_code)
 /**
  * \brief logs a WinDivert error at Error level.
  */
-#define WinDivertLogError(err_code)                                            \
-    do {                                                                       \
-        const char *win_err_str = Win32GetErrorString((err_code), NULL);       \
-        SCLogError(SC_ERR_WINDIVERT_GENERIC,                                   \
-                   "WinDivertOpen failed, error %" PRId32 " (0x%08" PRIx32     \
-                   "): %s %s",                                                 \
-                   (uint32_t)(err_code), (uint32_t)(err_code), win_err_str,    \
-                   WinDivertGetErrorString(err_code));                         \
-        LocalFree((LPVOID)win_err_str);                                        \
+#define WinDivertLogError(err_code)                                                                \
+    do {                                                                                           \
+        const char *win_err_str = Win32GetErrorString((err_code), NULL);                           \
+        SCLogError("WinDivertOpen failed, error %" PRId32 " (0x%08" PRIx32 "): %s %s",             \
+                (uint32_t)(err_code), (uint32_t)(err_code), win_err_str,                           \
+                WinDivertGetErrorString(err_code));                                                \
+        LocalFree((LPVOID)win_err_str);                                                            \
     } while (0);
 
 /**
@@ -213,13 +215,10 @@ static const char *WinDivertGetErrorString(DWORD error_code)
  */
 static void WinDivertInitQPCValues(WinDivertThreadVars *wd_tv)
 {
-    struct timeval now;
-
-    TimeGet(&now);
+    SCTime_t now = TimeGet();
     (void)QueryPerformanceCounter((LARGE_INTEGER *)&wd_tv->qpc_start_count);
 
-    wd_tv->qpc_start_time =
-            (uint64_t)now.tv_sec * (1000 * 1000) + (uint64_t)now.tv_usec;
+    wd_tv->qpc_start_time = (uint64_t)SCTIME_SECS(now) * (1000 * 1000) + (uint64_t)SCTIME_SECS(now);
 
     (void)QueryPerformanceFrequency((LARGE_INTEGER *)&wd_tv->qpc_freq_usec);
     /* \bug: clock drift? */
@@ -227,21 +226,22 @@ static void WinDivertInitQPCValues(WinDivertThreadVars *wd_tv)
 }
 
 /**
- * \brief gets a timeval from a WinDivert timestamp
+ * \brief WinDivert timestamp to a SCTime_t
  */
-static struct timeval WinDivertTimestampToTimeval(WinDivertThreadVars *wd_tv,
-                                                  INT64 timestamp_count)
+static SCTime_t WinDivertTimestampToTimeStamp(WinDivertThreadVars *wd_tv, INT64 timestamp_count)
 {
-    struct timeval ts;
+    struct timeval tv;
 
     int64_t qpc_delta = (int64_t)timestamp_count - wd_tv->qpc_start_count;
-    int64_t unix_usec =
-            wd_tv->qpc_start_time + (qpc_delta / wd_tv->qpc_freq_usec);
+    int64_t unix_usec = wd_tv->qpc_start_time;
+    if (wd_tv->qpc_freq_usec) {
+        unix_usec += qpc_delta / wd_tv->qpc_freq_usec;
+    }
 
-    ts.tv_sec = (long)(unix_usec / (1000 * 1000));
-    ts.tv_usec = (long)(unix_usec - (int64_t)ts.tv_sec * (1000 * 1000));
+    tv.tv_sec = (long)(unix_usec / (1000 * 1000));
+    tv.tv_usec = (long)(unix_usec - (int64_t)tv.tv_sec * (1000 * 1000));
 
-    return ts;
+    return SCTIME_FROM_TIMEVAL(&tv);
 }
 
 /**
@@ -267,20 +267,17 @@ int WinDivertRegisterQueue(bool forward, char *filter_str)
     bool valid = WinDivertHelperCheckFilter(filter_str, layer, &error_str,
                                             &error_pos);
     if (!valid) {
-        SCLogWarning(
-                SC_ERR_WINDIVERT_INVALID_FILTER,
-                "Invalid filter \"%s\" supplied to WinDivert: %s at position "
-                "%" PRId32 "",
+        SCLogWarning("Invalid filter \"%s\" supplied to WinDivert: %s at position "
+                     "%" PRId32 "",
                 filter_str, error_str, error_pos);
-        SCReturnInt(SC_ERR_WINDIVERT_INVALID_FILTER);
+        SCReturnInt(-1);
     }
 
     /* initialize the queue */
     SCMutexLock(&g_wd_init_lock);
 
     if (g_wd_num >= WINDIVERT_MAX_QUEUE) {
-        SCLogError(SC_ERR_INVALID_ARGUMENT,
-                   "Too many WinDivert queues specified %" PRId32 "", g_wd_num);
+        SCLogError("Too many WinDivert queues specified %" PRId32 "", g_wd_num);
         ret = -1;
         goto unlock;
     }
@@ -305,9 +302,8 @@ int WinDivertRegisterQueue(bool forward, char *filter_str)
     size_t copy_len =
             strlcpy(wd_qv->filter_str, filter_str, sizeof(wd_qv->filter_str));
     if (filter_len > copy_len) {
-        SCLogWarning(SC_ERR_WINDIVERT_TOOLONG_FILTER,
-                     "Queue length exceeds storage by %" PRId32 " bytes",
-                     (int32_t)(filter_len - copy_len));
+        SCLogWarning("Queue length exceeds storage by %" PRId32 " bytes",
+                (int32_t)(filter_len - copy_len));
         ret = -1;
         goto unlock;
     }
@@ -347,14 +343,12 @@ TmEcode ReceiveWinDivertThreadDeinit(ThreadVars *, void *);
 void ReceiveWinDivertThreadExitStats(ThreadVars *, void *);
 
 /* Verdict functions */
-TmEcode VerdictWinDivert(ThreadVars *, Packet *, void *, PacketQueue *,
-                         PacketQueue *);
+TmEcode VerdictWinDivert(ThreadVars *, Packet *, void *);
 TmEcode VerdictWinDivertThreadInit(ThreadVars *, const void *, void **);
 TmEcode VerdictWinDivertThreadDeinit(ThreadVars *, void *);
 
 /* Decode functions */
-TmEcode DecodeWinDivert(ThreadVars *, Packet *, void *, PacketQueue *,
-                        PacketQueue *);
+TmEcode DecodeWinDivert(ThreadVars *, Packet *, void *);
 TmEcode DecodeWinDivertThreadInit(ThreadVars *, const void *, void **);
 TmEcode DecodeWinDivertThreadDeinit(ThreadVars *, void *);
 
@@ -389,6 +383,7 @@ void TmModuleVerdictWinDivertRegister(void)
     tm_ptr->ThreadInit = VerdictWinDivertThreadInit;
     tm_ptr->Func = VerdictWinDivert;
     tm_ptr->ThreadDeinit = VerdictWinDivertThreadDeinit;
+    tm_ptr->flags = TM_FLAG_VERDICT_TM;
 }
 
 void TmModuleDecodeWinDivertRegister(void)
@@ -412,6 +407,10 @@ TmEcode ReceiveWinDivertLoop(ThreadVars *tv, void *data, void *slot)
     WinDivertThreadVars *wd_tv = (WinDivertThreadVars *)data;
     wd_tv->slot = ((TmSlot *)slot)->slot_next;
 
+    // Indicate that the thread is actually running its application level code (i.e., it can poll
+    // packets)
+    TmThreadsSetFlag(tv, THV_RUNNING);
+
     while (true) {
         if (suricata_ctl_flags & SURICATA_STOP) {
             SCReturnInt(TM_ECODE_OK);
@@ -485,7 +484,7 @@ static TmEcode WinDivertRecvHelper(ThreadVars *tv, WinDivertThreadVars *wd_tv)
     }
     SCLogDebug("Packet received, length %" PRId32 "", GET_PKT_LEN(p));
 
-    p->ts = WinDivertTimestampToTimeval(wd_tv, p->windivert_v.addr.Timestamp);
+    p->ts = WinDivertTimestampToTimeStamp(wd_tv, p->windivert_v.addr.Timestamp);
     p->windivert_v.thread_num = wd_tv->thread_num;
 
 #ifdef COUNTERS
@@ -500,7 +499,6 @@ static TmEcode WinDivertRecvHelper(ThreadVars *tv, WinDivertThreadVars *wd_tv)
      * or push it to a packet pool. So processing time can vary.
      */
     if (TmThreadsSlotProcessPkt(tv, wd_tv->slot, p) != TM_ECODE_OK) {
-        TmqhOutputPacketpool(tv, p);
         SCReturnInt(TM_ECODE_FAILED);
     }
 
@@ -525,14 +523,14 @@ TmEcode ReceiveWinDivertThreadInit(ThreadVars *tv, const void *initdata,
     WinDivertThreadVars *wd_tv = (WinDivertThreadVars *)initdata;
 
     if (wd_tv == NULL) {
-        SCLogError(SC_ERR_INVALID_ARGUMENT, "initdata == NULL");
+        SCLogError("initdata == NULL");
         SCReturnInt(TM_ECODE_FAILED);
     }
 
     WinDivertQueueVars *wd_qv = WinDivertGetQueue(wd_tv->thread_num);
 
     if (wd_qv == NULL) {
-        SCLogError(SC_ERR_INVALID_ARGUMENT, "queue == NULL");
+        SCLogError("queue == NULL");
         SCReturnInt(TM_ECODE_FAILED);
     }
 
@@ -548,8 +546,7 @@ TmEcode ReceiveWinDivertThreadInit(ThreadVars *tv, const void *initdata,
     if (WinDivertCollectFilterDevices(wd_tv, wd_qv) == TM_ECODE_OK) {
         WinDivertDisableOffloading(wd_tv);
     } else {
-        SCLogWarning(SC_ERR_SYSCALL,
-                     "Failed to obtain network devices for WinDivert filter");
+        SCLogWarning("Failed to obtain network devices for WinDivert filter");
     }
 
     /* we open now so that we can immediately start handling packets,
@@ -642,8 +639,7 @@ static bool WinDivertIfaceMatchFilter(const char *filter_string, int if_index)
     if (!match) {
         int err = GetLastError();
         if (err != 0) {
-            SCLogWarning(SC_ERR_WINDIVERT_GENERIC,
-                         "Failed to evaluate filter: 0x%" PRIx32, err);
+            SCLogWarning("Failed to evaluate filter: 0x%" PRIx32, err);
         }
     }
 
@@ -715,7 +711,7 @@ void ReceiveWinDivertThreadExitStats(ThreadVars *tv, void *data)
     WinDivertThreadVars *wd_tv = (WinDivertThreadVars *)data;
     WinDivertQueueVars *wd_qv = WinDivertGetQueue(wd_tv->thread_num);
     if (wd_qv == NULL) {
-        SCLogError(SC_ERR_INVALID_ARGUMENT, "queue == NULL");
+        SCLogError("queue == NULL");
         SCReturn;
     }
 
@@ -734,8 +730,7 @@ void ReceiveWinDivertThreadExitStats(ThreadVars *tv, void *data)
 /**
  * \brief WinDivert verdict module packet entry function
  */
-TmEcode VerdictWinDivert(ThreadVars *tv, Packet *p, void *data, PacketQueue *pq,
-                         PacketQueue *postpq)
+TmEcode VerdictWinDivert(ThreadVars *tv, Packet *p, void *data)
 {
     SCEnter();
 
@@ -757,9 +752,6 @@ static TmEcode WinDivertVerdictHelper(ThreadVars *tv, Packet *p)
     SCEnter();
     WinDivertThreadVars *wd_tv = WinDivertGetThread(p->windivert_v.thread_num);
 
-    /* update counters */
-    CaptureStatsUpdate(tv, &wd_tv->stats, p);
-
 #ifdef COUNTERS
     WinDivertQueueVars *wd_qv = WinDivertGetQueue(wd_tv->thread_num);
 #endif /* COUNTERS */
@@ -779,7 +771,7 @@ static TmEcode WinDivertVerdictHelper(ThreadVars *tv, Packet *p)
 
     /* we can't verdict tunnel packets without ensuring all encapsulated
      * packets are verdicted */
-    if (IS_TUNNEL_PKT(p)) {
+    if (PacketIsTunnel(p)) {
         bool finalVerdict = VerdictTunnelPacket(p);
         if (!finalVerdict) {
             SCReturnInt(TM_ECODE_OK);
@@ -793,7 +785,7 @@ static TmEcode WinDivertVerdictHelper(ThreadVars *tv, Packet *p)
 
     /* DROP simply means we do nothing; the WinDivert driver does the rest.
      */
-    if (PACKET_TEST_ACTION(p, ACTION_DROP)) {
+    if (PacketCheckAction(p, ACTION_DROP)) {
 #ifdef COUNTERS
         SCMutexLock(&wd_qv->counters_mutex);
         wd_qv->dropped++;
@@ -830,9 +822,6 @@ TmEcode VerdictWinDivertThreadInit(ThreadVars *tv, const void *initdata,
     SCEnter();
 
     WinDivertThreadVars *wd_tv = (WinDivertThreadVars *)initdata;
-
-    CaptureStatsSetup(tv, &wd_tv->stats);
-
     *data = wd_tv;
 
     SCReturnInt(TM_ECODE_OK);
@@ -859,8 +848,7 @@ TmEcode VerdictWinDivertThreadDeinit(ThreadVars *tv, void *data)
  * to differentiate the two, so instead we must check the version and go
  * from there.
  */
-TmEcode DecodeWinDivert(ThreadVars *tv, Packet *p, void *data, PacketQueue *pq,
-                        PacketQueue *postpq)
+TmEcode DecodeWinDivert(ThreadVars *tv, Packet *p, void *data)
 {
     SCEnter();
 
@@ -868,21 +856,16 @@ TmEcode DecodeWinDivert(ThreadVars *tv, Packet *p, void *data, PacketQueue *pq,
     IPV6Hdr *ip6h = (IPV6Hdr *)GET_PKT_DATA(p);
     DecodeThreadVars *d_tv = (DecodeThreadVars *)data;
 
-    /* XXX HACK: flow timeout can call us for injected pseudo packets
-     *           see bug:
-     * https://redmine.openinfosecfoundation.org/issues/1107
-     */
-    if (PKT_IS_PSEUDOPKT(p))
-        SCReturnInt(TM_ECODE_OK);
+    BUG_ON(PKT_IS_PSEUDOPKT(p));
 
     DecodeUpdatePacketCounters(tv, d_tv, p);
 
     if (IPV4_GET_RAW_VER(ip4h) == 4) {
         SCLogDebug("IPv4 packet");
-        DecodeIPV4(tv, d_tv, p, GET_PKT_DATA(p), GET_PKT_LEN(p), pq);
+        DecodeIPV4(tv, d_tv, p, GET_PKT_DATA(p), GET_PKT_LEN(p));
     } else if (IPV6_GET_RAW_VER(ip6h) == 6) {
         SCLogDebug("IPv6 packet");
-        DecodeIPV6(tv, d_tv, p, GET_PKT_DATA(p), GET_PKT_LEN(p), pq);
+        DecodeIPV6(tv, d_tv, p, GET_PKT_DATA(p), GET_PKT_LEN(p));
     } else {
         SCLogDebug("packet unsupported by WinDivert, first byte: %02x",
                    *GET_PKT_DATA(p));
@@ -945,8 +928,7 @@ static TmEcode WinDivertCloseHelper(WinDivertThreadVars *wd_tv)
     }
 
     if (!WinDivertClose(wd_qv->filter_handle)) {
-        SCLogError(SC_ERR_FATAL, "WinDivertClose failed: error %" PRIu32 "",
-                   (uint32_t)(GetLastError()));
+        SCLogError("WinDivertClose failed: error %" PRIu32 "", (uint32_t)(GetLastError()));
         ret = TM_ECODE_FAILED;
         goto unlock;
     }
@@ -1010,7 +992,7 @@ static int SourceWinDivertTestIfaceMatchFilter(void)
 /**
  * \brief this function registers unit tests for the WinDivert Source
  */
-void SourceWinDivertRegisterTests()
+void SourceWinDivertRegisterTests(void)
 {
 #ifdef UNITTESTS
     UtRegisterTest("SourceWinDivertTestIfaceMatchFilter",
@@ -1018,4 +1000,4 @@ void SourceWinDivertRegisterTests()
 #endif
 }
 
-#endif /* WINDIVERT */
\ No newline at end of file
+#endif /* WINDIVERT */
