@@ -10,6 +10,7 @@
 #include "util-classification-config.h"
 #include "detect-engine.h"
 #include "detect-parse.h"
+#include "app-layer.h"
 
 int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size);
 
@@ -23,10 +24,13 @@ int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size)
         setenv("SC_LOG_FILE", "/dev/null", 0);
         //global init
         InitGlobal();
-        run_mode = RUNMODE_UNITTEST;
+        GlobalsInitPreConfig();
+        SCRunmodeSet(RUNMODE_UNITTEST);
         MpmTableSetup();
         SpmTableSetup();
         EngineModeSetIDS();
+        SigTableInit();
+        AppLayerSetup();
         SigTableSetup();
     }
     if (cnt++ == 1024) {
