@@ -25,17 +25,18 @@
  */
 
 #include "suricata-common.h"
+#include "thread-storage.h"
 #include "util-unittest.h"
-#include "debug.h"
 #include "util-debug.h"
 #include "threads.h"
 
+thread_local char t_thread_name[THREAD_NAME_LEN + 1];
 #ifdef UNITTESTS /* UNIT TESTS */
 
 /**
  * \brief Test Mutex macros
  */
-int ThreadMacrosTest01Mutex(void)
+static int ThreadMacrosTest01Mutex(void)
 {
     SCMutex mut;
     int r = 0;
@@ -59,11 +60,11 @@ int ThreadMacrosTest01Mutex(void)
  * ==31156==    by 0x532E8A: UtRunTests (util-unittest.c:182)
  * ==31156==    by 0x4065C3: main (suricata.c:789)
  *
- * To me this is a false positve, as the whole point of "trylock" is to see
+ * To me this is a false positive, as the whole point of "trylock" is to see
  * if a spinlock is actually locked.
  *
  */
-int ThreadMacrosTest02Spinlocks(void)
+static int ThreadMacrosTest02Spinlocks(void)
 {
     SCSpinlock mut;
     int r = 0;
@@ -83,7 +84,7 @@ int ThreadMacrosTest02Spinlocks(void)
 /**
  * \brief Test RWLock macros
  */
-int ThreadMacrosTest03RWLocks(void)
+static int ThreadMacrosTest03RWLocks(void)
 {
     SCRWLock rwl_write;
     int r = 0;
@@ -105,7 +106,7 @@ int ThreadMacrosTest03RWLocks(void)
 /**
  * \brief Test RWLock macros
  */
-int ThreadMacrosTest04RWLocks(void)
+static int ThreadMacrosTest04RWLocks(void)
 {
     SCRWLock rwl_read;
     int r = 0;
@@ -118,10 +119,11 @@ int ThreadMacrosTest04RWLocks(void)
     return (r == 0)? 1 : 0;
 }
 
+#if 0 // broken on OSX
 /**
  * \brief Test RWLock macros
  */
-int ThreadMacrosTest05RWLocks(void)
+static int ThreadMacrosTest05RWLocks(void)
 {
     SCRWLock rwl_read;
     int r = 0;
@@ -133,6 +135,7 @@ int ThreadMacrosTest05RWLocks(void)
 
     return (r == 0)? 1 : 0;
 }
+#endif
 
 #endif /* UNIT TESTS */
 
@@ -146,5 +149,7 @@ void ThreadMacrosRegisterTests(void)
     UtRegisterTest("ThreadMacrosTest02Spinlocks", ThreadMacrosTest02Spinlocks);
     UtRegisterTest("ThreadMacrosTest03RWLocks", ThreadMacrosTest03RWLocks);
     UtRegisterTest("ThreadMacrosTest04RWLocks", ThreadMacrosTest04RWLocks);
+//    UtRegisterTest("ThreadMacrosTest05RWLocks", ThreadMacrosTest05RWLocks);
+    RegisterThreadStorageTests();
 #endif /* UNIT TESTS */
 }
