@@ -21,12 +21,12 @@
  * \author Victor Julien <victor@inliniac.net>
  */
 
-#ifndef __TM_QUEUEHANDLERS_H__
-#define __TM_QUEUEHANDLERS_H__
+#ifndef SURICATA_TM_QUEUEHANDLERS_H
+#define SURICATA_TM_QUEUEHANDLERS_H
 
 enum {
+    TMQH_NOT_SET,
     TMQH_SIMPLE,
-    TMQH_NFQ,
     TMQH_PACKETPOOL,
     TMQH_FLOW,
 
@@ -43,11 +43,12 @@ typedef struct Tmqh_ {
     void (*RegisterTests)(void);
 } Tmqh;
 
-Tmqh tmqh_table[TMQH_SIZE];
+extern Tmqh tmqh_table[TMQH_SIZE];
 
 void TmqhSetup (void);
 void TmqhCleanup(void);
-Tmqh* TmqhGetQueueHandlerByName(const char *name);
-
-#endif /* __TM_QUEUEHANDLERS_H__ */
+int TmqhNameToID(const char *name);
+Tmqh *TmqhGetQueueHandlerByName(const char *name);
+Tmqh *TmqhGetQueueHandlerByID(const int id);
 
+#endif /* SURICATA_TM_QUEUEHANDLERS_H */
