@@ -22,8 +22,8 @@
  * \author Anoop Saldanha <anoopsaldanha@gmail.com>
  */
 
-#ifndef __TM_THREADS_COMMON_H__
-#define __TM_THREADS_COMMON_H__
+#ifndef SURICATA_TM_THREADS_COMMON_H
+#define SURICATA_TM_THREADS_COMMON_H
 
 /** \brief Thread Model Module id's.
  *
@@ -39,8 +39,8 @@ typedef enum {
     TMM_RECEIVEPCAPFILE,
     TMM_DECODEPCAP,
     TMM_DECODEPCAPFILE,
-    TMM_RECEIVEPFRING,
-    TMM_DECODEPFRING,
+    TMM_RECEIVEPLUGIN,
+    TMM_DECODEPLUGIN,
     TMM_RESPONDREJECT,
     TMM_DECODEIPFW,
     TMM_VERDICTIPFW,
@@ -50,20 +50,21 @@ typedef enum {
     TMM_RECEIVEERFDAG,
     TMM_DECODEERFDAG,
     TMM_RECEIVEAFP,
+    TMM_RECEIVEAFXDP,
     TMM_DECODEAFP,
+    TMM_DECODEAFXDP,
+    TMM_RECEIVEDPDK,
+    TMM_DECODEDPDK,
     TMM_RECEIVENETMAP,
     TMM_DECODENETMAP,
     TMM_ALERTPCAPINFO,
-    TMM_RECEIVEMPIPE,
-    TMM_DECODEMPIPE,
-    TMM_RECEIVENAPATECH,
-    TMM_DECODENAPATECH,
     TMM_STATSLOGGER,
     TMM_RECEIVENFLOG,
     TMM_DECODENFLOG,
     TMM_RECEIVEWINDIVERT,
     TMM_VERDICTWINDIVERT,
     TMM_DECODEWINDIVERT,
+    TMM_DECODELIB,
 
     TMM_FLOWMANAGER,
     TMM_FLOWRECYCLER,
@@ -90,5 +91,4 @@ enum {
     TVT_MAX,
 };
 
-#endif /* __TM_THREADS_COMMON_H__ */
-
+#endif /* SURICATA_TM_THREADS_COMMON_H */
