@@ -21,13 +21,11 @@
  * \author Victor Julien <victor@inliniac.net>
  */
 
-#ifndef __TMQH_FLOW_H__
-#define __TMQH_FLOW_H__
+#ifndef SURICATA_TMQH_FLOW_H
+#define SURICATA_TMQH_FLOW_H
 
 typedef struct TmqhFlowMode_ {
     PacketQueue *q;
-    SC_ATOMIC_DECLARE(uint64_t, total_packets);
-    SC_ATOMIC_DECLARE(uint64_t, total_flows);
 } TmqhFlowMode;
 
 /** \brief Ctx for the flow queue handler
@@ -38,8 +36,6 @@ typedef struct TmqhFlowCtx_ {
     uint16_t last;
 
     TmqhFlowMode *queues;
-
-    SC_ATOMIC_DECLARE(uint16_t, round_robin_idx);
 } TmqhFlowCtx;
 
 void TmqhFlowRegister (void);
@@ -47,4 +43,4 @@ void TmqhFlowRegisterTests(void);
 
 void TmqhFlowPrintAutofpHandler(void);
 
-#endif /* __TMQH_FLOW_H__ */
+#endif /* SURICATA_TMQH_FLOW_H */
