@@ -21,16 +21,18 @@
  * \author Eric Leblond <eric@regit.org>
  */
 
-#ifndef __UTIL_CHECKSUM_H__
-#define __UTIL_CHECKSUM_H__
+#ifndef SURICATA_UTIL_CHECKSUM_H
+#define SURICATA_UTIL_CHECKSUM_H
+
+#include "decode.h"
 
 int ReCalculateChecksum(Packet *p);
-int ChecksumAutoModeCheck(uint32_t thread_count,
-        unsigned int iface_count, unsigned int iface_fail);
+int ChecksumAutoModeCheck(uint64_t thread_count,
+        uint64_t iface_count, uint64_t iface_fail);
 
 /* constant linked with detection of interface with
  * invalid checksums */
-#define CHECKSUM_SAMPLE_COUNT 1000
+#define CHECKSUM_SAMPLE_COUNT 1000ULL
 #define CHECKSUM_INVALID_RATIO 10
 
 #endif
