@@ -21,8 +21,8 @@
  * \author Victor Julien <victor@inliniac.net>
  */
 
-#ifndef __UTIL_CLOCK_H__
-#define __UTIL_CLOCK_H__
+#ifndef SURICATA_UTIL_CLOCK_H
+#define SURICATA_UTIL_CLOCK_H
 
 #include <time.h>
 
@@ -33,8 +33,7 @@
 
 #define CLOCK_END           clo2 = clock()
 
-#define CLOCK_PRINT_SEC     printf("Seconds spent: %.4fs\n", ((clo2 - clo1)/(double)CLOCKS_PER_SEC))
+#define CLOCK_PRINT_SEC                                                                            \
+    printf("Seconds spent: %.4fs\n", ((double)(clo2 - clo1) / (double)CLOCKS_PER_SEC))
 
-#define GET_CLOCK_END_SECS  ((clo1 - clo2)/(double)CLOCKS_PER_SEC)
-
-#endif /*__UTIL_CLOCK_H__ */
+#endif /*SURICATA_UTIL_CLOCK_H */
