@@ -23,21 +23,23 @@
  */
 
 #include "suricata-common.h"
-#include "config.h"
+#include "suricata.h"
 #include "conf.h"
 #include "runmodes.h"
 #include "util-conf.h"
+#include "util-debug.h"
+#include "util-path.h"
 
-TmEcode ConfigSetLogDirectory(char *name)
+TmEcode ConfigSetLogDirectory(const char *name)
 {
-    return ConfSetFinal("default-log-dir", name) ? TM_ECODE_OK : TM_ECODE_FAILED;
+    return SCConfSetFinal("default-log-dir", name) ? TM_ECODE_OK : TM_ECODE_FAILED;
 }
 
-const char *ConfigGetLogDirectory()
+const char *SCConfigGetLogDirectory(void)
 {
     const char *log_dir = NULL;
 
-    if (ConfGet("default-log-dir", &log_dir) != 1) {
+    if (SCConfGet("default-log-dir", &log_dir) != 1) {
 #ifdef OS_WIN32
         log_dir = _getcwd(NULL, 0);
         if (log_dir == NULL) {
@@ -51,17 +53,55 @@ const char *ConfigGetLogDirectory()
     return log_dir;
 }
 
-TmEcode ConfigCheckLogDirectory(const char *log_dir)
+TmEcode ConfigCheckLogDirectoryExists(const char *log_dir)
 {
     SCEnter();
+    SCStat buf;
+    if (SCStatFn(log_dir, &buf) != 0) {
+        SCReturnInt(TM_ECODE_FAILED);
+    }
+    SCReturnInt(TM_ECODE_OK);
+}
+
+TmEcode ConfigSetDataDirectory(char *name)
+{
+    if (strlen(name) == 0)
+        return TM_ECODE_OK;
+
+    size_t size = strlen(name) + 1;
+    char tmp[size];
+    strlcpy(tmp, name, size);
+    if (size > 2 && tmp[size - 2] == '/') // > 2 to allow just /
+        tmp[size - 2] = '\0';
+
+    return SCConfSetFinal("default-data-dir", tmp) ? TM_ECODE_OK : TM_ECODE_FAILED;
+}
+
+const char *ConfigGetDataDirectory(void)
+{
+    const char *data_dir = NULL;
+
+    if (SCConfGet("default-data-dir", &data_dir) != 1) {
 #ifdef OS_WIN32
-    struct _stat buf;
-    if (_stat(log_dir, &buf) != 0) {
+        data_dir = _getcwd(NULL, 0);
+        if (data_dir == NULL) {
+            data_dir = DEFAULT_DATA_DIR;
+        }
 #else
-    struct stat buf;
-    if (stat(log_dir, &buf) != 0) {
+        data_dir = DEFAULT_DATA_DIR;
 #endif /* OS_WIN32 */
-            SCReturnInt(TM_ECODE_FAILED);
+    }
+
+    SCLogDebug("returning '%s'", data_dir);
+    return data_dir;
+}
+
+TmEcode ConfigCheckDataDirectory(const char *data_dir)
+{
+    SCEnter();
+    SCStat buf;
+    if (SCStatFn(data_dir, &buf) != 0) {
+        SCReturnInt(TM_ECODE_FAILED);
     }
     SCReturnInt(TM_ECODE_OK);
 }
@@ -78,9 +118,9 @@ TmEcode ConfigCheckLogDirectory(const char *log_dir)
  *
  * \param iface The name of the interface to find the config for.
  */
-ConfNode *ConfFindDeviceConfig(ConfNode *node, const char *iface)
+SCConfNode *ConfFindDeviceConfig(SCConfNode *node, const char *iface)
 {
-    ConfNode *if_node, *item;
+    SCConfNode *if_node, *item;
     TAILQ_FOREACH(if_node, &node->head, next) {
         TAILQ_FOREACH(item, &if_node->head, next) {
             if (strcmp(item->name, "interface") == 0 &&
@@ -97,31 +137,27 @@ int ConfUnixSocketIsEnable(void)
 {
     const char *value;
 
-    if (ConfGet("unix-command.enabled", &value) != 1) {
+    if (SCConfGet("unix-command.enabled", &value) != 1) {
         return 0;
     }
 
     if (value == NULL) {
-        SCLogError(SC_ERR_INVALID_YAML_CONF_ENTRY, "malformed value for unix-command.enabled: NULL");
+        SCLogError("malformed value for unix-command.enabled: NULL");
         return 0;
     }
 
     if (!strcmp(value, "auto")) {
-#ifdef HAVE_LIBJANSSON
 #ifdef OS_WIN32
         return 0;
 #else
-        if (!IsRunModeOffline(RunmodeGetCurrent())) {
+        if (!IsRunModeOffline(SCRunmodeGet())) {
             SCLogInfo("Running in live mode, activating unix socket");
             return 1;
         } else {
             return 0;
         }
-#endif
-#else
-        return 0;
 #endif
     }
 
-    return ConfValIsTrue(value);
+    return SCConfValIsTrue(value);
 }
