@@ -21,19 +21,16 @@
  * \author Pablo Rincon Crespo <pablo.rincon.crespo@gmail.com>
  */
 
-#ifndef __UTIL_CPU_H__
-#define __UTIL_CPU_H__
+#ifndef SURICATA_UTIL_CPU_H
+#define SURICATA_UTIL_CPU_H
 
 /* Processors configured: */
-uint16_t UtilCpuGetNumProcessorsConfigured();
+uint16_t UtilCpuGetNumProcessorsConfigured(void);
 /* Processors online: */
-uint16_t UtilCpuGetNumProcessorsOnline();
+uint16_t UtilCpuGetNumProcessorsOnline(void);
 
-/* Only on Solaris */
-uint16_t UtilCpuGetNumProcessorsMax();
-
-void UtilCpuPrintSummary();
+void UtilCpuPrintSummary(void);
 
 uint64_t UtilCpuGetTicks(void);
 
-#endif /* __UTIL_CPU_H__ */
+#endif /* SURICATA_UTIL_CPU_H */
