@@ -1,4 +1,4 @@
-/* Copyright (C) 2011-2016 Open Information Security Foundation
+/* Copyright (C) 2011-2021 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -17,13 +17,19 @@
 
 #include "suricata-common.h"
 #include "conf.h"
-#include "util-device.h"
+#include "util-device-private.h"
 #include "util-ioctl.h"
+#include "util-misc.h"
+#include "util-dpdk.h"
 
 #include "device-storage.h"
+#include "util-debug.h"
+#include "util-affinity.h"
 
 #define MAX_DEVNAME 10
 
+static LiveDevStorageId g_bypass_storage_id = { .id = -1 };
+
 /**
  * \file
  *
@@ -36,6 +42,11 @@
 static TAILQ_HEAD(, LiveDevice_) live_devices =
     TAILQ_HEAD_INITIALIZER(live_devices);
 
+typedef struct LiveDeviceName_ {
+    char *dev; /**< the device (e.g. "eth0") */
+    TAILQ_ENTRY(LiveDeviceName_) next;
+} LiveDeviceName;
+
 /** List of the name of devices
  *
  * As we don't know the size of the Storage on devices
@@ -45,9 +56,19 @@ static TAILQ_HEAD(, LiveDevice_) live_devices =
 static TAILQ_HEAD(, LiveDeviceName_) pre_live_devices =
     TAILQ_HEAD_INITIALIZER(pre_live_devices);
 
+typedef struct BypassInfo_ {
+    SC_ATOMIC_DECLARE(uint64_t, ipv4_hash_count);
+    SC_ATOMIC_DECLARE(uint64_t, ipv4_fail);
+    SC_ATOMIC_DECLARE(uint64_t, ipv4_success);
+    SC_ATOMIC_DECLARE(uint64_t, ipv6_hash_count);
+    SC_ATOMIC_DECLARE(uint64_t, ipv6_fail);
+    SC_ATOMIC_DECLARE(uint64_t, ipv6_success);
+} BypassInfo;
+
 /** if set to 0 when we don't have real devices */
 static int live_devices_stats = 1;
 
+
 static int LiveSafeDeviceName(const char *devname,
                               char *newdevname, size_t destlen);
 
@@ -117,23 +138,24 @@ int LiveRegisterDevice(const char *dev)
         return -1;
     }
 
+    int id = LiveGetDeviceCount();
+    if (id > UINT16_MAX) {
+        SCFree(pd);
+        return -1;
+    }
+
     pd->dev = SCStrdup(dev);
     if (unlikely(pd->dev == NULL)) {
         SCFree(pd);
         return -1;
     }
     /* create a short version to be used in thread names */
-    if (strlen(pd->dev) > MAX_DEVNAME) {
-        LiveSafeDeviceName(pd->dev, pd->dev_short, sizeof(pd->dev_short));
-    } else {
-        (void)strlcpy(pd->dev_short, pd->dev, sizeof(pd->dev_short));
-    }
+    LiveSafeDeviceName(pd->dev, pd->dev_short, sizeof(pd->dev_short));
 
     SC_ATOMIC_INIT(pd->pkts);
     SC_ATOMIC_INIT(pd->drop);
     SC_ATOMIC_INIT(pd->invalid_checksums);
-    pd->ignore_checksum = 0;
-    pd->id = LiveGetDeviceCount();
+    pd->id = (uint16_t)id;
     TAILQ_INSERT_TAIL(&live_devices, pd, next);
 
     SCLogDebug("Device \"%s\" registered and created.", dev);
@@ -157,6 +179,20 @@ int LiveGetDeviceCount(void)
     return i;
 }
 
+int LiveGetDeviceCountWithoutAssignedThreading(void)
+{
+    int i = 0;
+    LiveDevice *pd;
+
+    TAILQ_FOREACH (pd, &live_devices, next) {
+        if (GetAffinityTypeForNameAndIface("worker-cpu-set", pd->dev) == NULL) {
+            i++;
+        }
+    }
+
+    return i;
+}
+
 /**
  *  \brief Get a pointer to the device name at idx
  *
@@ -190,10 +226,15 @@ const char *LiveGetDeviceName(int number)
  */
 static int LiveSafeDeviceName(const char *devname, char *newdevname, size_t destlen)
 {
-    size_t devnamelen = strlen(devname);
+    const size_t devnamelen = strlen(devname);
 
     /* If we have to shorten the interface name */
     if (devnamelen > MAX_DEVNAME) {
+        /* special mode for DPDK pci addresses */
+        if (devnamelen >= 5 && strncmp(devname, "0000:", 5) == 0) {
+            strlcpy(newdevname, devname + 5, destlen);
+            return 0;
+        }
 
         /* IF the dest length is over 10 chars long it will not do any
          * good for the shortening. The shortening is done due to the
@@ -207,30 +248,9 @@ static int LiveSafeDeviceName(const char *devname, char *newdevname, size_t dest
             return 1;
         }
 
-        size_t length;
-        size_t half;
-        size_t spaces;
+        ShortenString(devname, newdevname, destlen, '.');
 
-        half = (destlen-1) / 2;
-
-        /* If the destlen is an even number */
-        if (half * 2 == (destlen-1)) {
-            half = half - 1;
-        }
-
-        spaces = (destlen-1) - (half*2);
-        length = half;
-
-        /* Add the first half to the new dev name */
-        snprintf(newdevname, half+1, "%s", devname);
-
-        /* Add the amount of spaces wanted */
-        for (size_t i = half; i < half+spaces; i++) {
-            length = strlcat(newdevname, ".", destlen);
-        }
-
-        snprintf(newdevname+length, half+1, "%s", devname+(devnamelen-half));
-        SCLogInfo("Shortening device name to: %s", newdevname);
+        SCLogInfo("%s: shortening device name to %s", devname, newdevname);
     } else {
         strlcpy(newdevname, devname, destlen);
     }
@@ -247,11 +267,10 @@ static int LiveSafeDeviceName(const char *devname, char *newdevname, size_t dest
  */
 LiveDevice *LiveGetDevice(const char *name)
 {
-    int i = 0;
     LiveDevice *pd;
 
     if (name == NULL) {
-        SCLogWarning(SC_ERR_INVALID_VALUE, "Name of device should not be null");
+        SCLogWarning("Name of device should not be null");
         return NULL;
     }
 
@@ -259,8 +278,6 @@ LiveDevice *LiveGetDevice(const char *name)
         if (!strcmp(name, pd->dev)) {
             return pd;
         }
-
-        i++;
     }
 
     return NULL;
@@ -281,15 +298,15 @@ int LiveBuildDeviceList(const char *runmode)
 
 int LiveBuildDeviceListCustom(const char *runmode, const char *itemname)
 {
-    ConfNode *base = ConfGetNode(runmode);
-    ConfNode *child;
+    SCConfNode *base = SCConfGetNode(runmode);
+    SCConfNode *child;
     int i = 0;
 
     if (base == NULL)
         return 0;
 
     TAILQ_FOREACH(child, &base->head, next) {
-        ConfNode *subchild;
+        SCConfNode *subchild;
         TAILQ_FOREACH(subchild, &child->head, next) {
             if ((!strcmp(subchild->name, itemname))) {
                 if (!strcmp(subchild->val, "default"))
@@ -309,33 +326,36 @@ int LiveBuildDeviceListCustom(const char *runmode, const char *itemname)
  *
  * This can be useful in the case, this is not a real interface.
  */
-void LiveDeviceHasNoStats()
+void LiveDeviceHasNoStats(void)
 {
     live_devices_stats = 0;
 }
 
-int LiveDeviceListClean()
+int LiveDeviceListClean(void)
 {
     SCEnter();
     LiveDevice *pd, *tpd;
 
+    /* dpdk: need to close all devices before freeing them. */
+    TAILQ_FOREACH (pd, &live_devices, next) {
+        DPDKCloseDevice(pd);
+    }
     TAILQ_FOREACH_SAFE(pd, &live_devices, next, tpd) {
         if (live_devices_stats) {
-            SCLogNotice("Stats for '%s':  pkts: %" PRIu64", drop: %" PRIu64 " (%.2f%%), invalid chksum: %" PRIu64,
-                    pd->dev,
-                    SC_ATOMIC_GET(pd->pkts),
-                    SC_ATOMIC_GET(pd->drop),
-                    100 * (SC_ATOMIC_GET(pd->drop) * 1.0) / SC_ATOMIC_GET(pd->pkts),
+            SCLogNotice("%s: packets: %" PRIu64 ", drops: %" PRIu64
+                        " (%.2f%%), invalid chksum: %" PRIu64,
+                    pd->dev, SC_ATOMIC_GET(pd->pkts), SC_ATOMIC_GET(pd->drop),
+                    SC_ATOMIC_GET(pd->pkts) > 0 ? 100 * ((double)SC_ATOMIC_GET(pd->drop)) /
+                                                          (double)SC_ATOMIC_GET(pd->pkts)
+                                                : 0,
                     SC_ATOMIC_GET(pd->invalid_checksums));
         }
 
         RestoreIfaceOffloading(pd);
+        DPDKFreeDevice(pd);
 
         if (pd->dev)
             SCFree(pd->dev);
-        SC_ATOMIC_DESTROY(pd->pkts);
-        SC_ATOMIC_DESTROY(pd->drop);
-        SC_ATOMIC_DESTROY(pd->invalid_checksums);
         LiveDevFreeStorage(pd);
         SCFree(pd);
     }
@@ -452,3 +472,189 @@ void LiveDeviceFinalize(void)
         SCFree(ld);
     }
 }
+
+static void LiveDevExtensionFree(void *x)
+{
+    if (x)
+        SCFree(x);
+}
+
+/**
+ * Register bypass stats storage
+ */
+void LiveDevRegisterExtension(void)
+{
+    g_bypass_storage_id = LiveDevStorageRegister("bypass_stats", sizeof(void *),
+                                                 NULL, LiveDevExtensionFree);
+}
+
+/**
+ * Prepare a LiveDevice so we can set bypass stats
+ */
+int LiveDevUseBypass(LiveDevice *dev)
+{
+    BypassInfo *bpinfo = SCCalloc(1, sizeof(*bpinfo));
+    if (bpinfo == NULL) {
+        SCLogError("Can't allocate bypass info structure");
+        return -1;
+    }
+
+    SC_ATOMIC_INIT(bpinfo->ipv4_hash_count);
+    SC_ATOMIC_INIT(bpinfo->ipv4_hash_count);
+
+    LiveDevSetStorageById(dev, g_bypass_storage_id, bpinfo);
+    return 0;
+}
+
+/**
+ * Increase number of currently bypassed flows for a protocol family
+ *
+ * \param dev pointer to LiveDevice to set stats for
+ * \param cnt number of flows to add
+ * \param family AF_INET to set IPv4 count or AF_INET6 to set IPv6 count
+ */
+void LiveDevAddBypassStats(LiveDevice *dev, uint64_t cnt, int family)
+{
+    BypassInfo *bpfdata = LiveDevGetStorageById(dev, g_bypass_storage_id);
+    if (bpfdata) {
+        if (family == AF_INET) {
+            SC_ATOMIC_ADD(bpfdata->ipv4_hash_count, cnt);
+        } else if (family == AF_INET6) {
+            SC_ATOMIC_ADD(bpfdata->ipv6_hash_count, cnt);
+        }
+    }
+}
+
+/**
+ * Decrease number of currently bypassed flows for a protocol family
+ *
+ * \param dev pointer to LiveDevice to set stats for
+ * \param cnt number of flows to remove
+ * \param family AF_INET to set IPv4 count or AF_INET6 to set IPv6 count
+ */
+void LiveDevSubBypassStats(LiveDevice *dev, uint64_t cnt, int family)
+{
+    BypassInfo *bpfdata = LiveDevGetStorageById(dev, g_bypass_storage_id);
+    if (bpfdata) {
+        if (family == AF_INET) {
+            SC_ATOMIC_SUB(bpfdata->ipv4_hash_count, cnt);
+        } else if (family == AF_INET6) {
+            SC_ATOMIC_SUB(bpfdata->ipv6_hash_count, cnt);
+        }
+    }
+}
+
+/**
+ * Increase number of failed captured flows for a protocol family
+ *
+ * \param dev pointer to LiveDevice to set stats for
+ * \param cnt number of flows to add
+ * \param family AF_INET to set IPv4 count or AF_INET6 to set IPv6 count
+ */
+void LiveDevAddBypassFail(LiveDevice *dev, uint64_t cnt, int family)
+{
+    BypassInfo *bpfdata = LiveDevGetStorageById(dev, g_bypass_storage_id);
+    if (bpfdata) {
+        if (family == AF_INET) {
+            SC_ATOMIC_ADD(bpfdata->ipv4_fail, cnt);
+        } else if (family == AF_INET6) {
+            SC_ATOMIC_ADD(bpfdata->ipv6_fail, cnt);
+        }
+    }
+}
+
+/**
+ * Increase number of currently successfully bypassed flows for a protocol family
+ *
+ * \param dev pointer to LiveDevice to set stats for
+ * \param cnt number of flows to add
+ * \param family AF_INET to set IPv4 count or AF_INET6 to set IPv6 count
+ */
+void LiveDevAddBypassSuccess(LiveDevice *dev, uint64_t cnt, int family)
+{
+    BypassInfo *bpfdata = LiveDevGetStorageById(dev, g_bypass_storage_id);
+    if (bpfdata) {
+        if (family == AF_INET) {
+            SC_ATOMIC_ADD(bpfdata->ipv4_success, cnt);
+        } else if (family == AF_INET6) {
+            SC_ATOMIC_ADD(bpfdata->ipv6_success, cnt);
+        }
+    }
+}
+
+#ifdef BUILD_UNIX_SOCKET
+TmEcode LiveDeviceGetBypassedStats(json_t *cmd, json_t *answer, void *data)
+{
+    if (g_bypass_storage_id.id < 0) {
+        json_object_set_new(answer, "message", json_string("Bypass not enabled"));
+        SCReturnInt(TM_ECODE_FAILED);
+    }
+    LiveDevice *ldev = NULL, *ndev = NULL;
+    json_t *ifaces = NULL;
+    while(LiveDeviceForEach(&ldev, &ndev)) {
+        BypassInfo *bpinfo = LiveDevGetStorageById(ldev, g_bypass_storage_id);
+        if (bpinfo) {
+            uint64_t ipv4_hash_count = SC_ATOMIC_GET(bpinfo->ipv4_hash_count);
+            uint64_t ipv6_hash_count = SC_ATOMIC_GET(bpinfo->ipv6_hash_count);
+            uint64_t ipv4_success = SC_ATOMIC_GET(bpinfo->ipv4_success);
+            uint64_t ipv4_fail = SC_ATOMIC_GET(bpinfo->ipv4_fail);
+            uint64_t ipv6_success = SC_ATOMIC_GET(bpinfo->ipv6_success);
+            uint64_t ipv6_fail = SC_ATOMIC_GET(bpinfo->ipv6_fail);
+            json_t *iface = json_object();
+            if (ifaces == NULL) {
+                ifaces = json_object();
+                if (ifaces == NULL) {
+                    json_object_set_new(answer, "message",
+                            json_string("internal error at json object creation"));
+                    return TM_ECODE_FAILED;
+                }
+            }
+            json_object_set_new(iface, "ipv4_maps_count", json_integer(ipv4_hash_count));
+            json_object_set_new(iface, "ipv4_success", json_integer(ipv4_success));
+            json_object_set_new(iface, "ipv4_fail", json_integer(ipv4_fail));
+            json_object_set_new(iface, "ipv6_maps_count", json_integer(ipv6_hash_count));
+            json_object_set_new(iface, "ipv6_success", json_integer(ipv6_success));
+            json_object_set_new(iface, "ipv6_fail", json_integer(ipv6_fail));
+            json_object_set_new(ifaces, ldev->dev, iface);
+        }
+    }
+    if (ifaces) {
+        json_object_set_new(answer, "message", ifaces);
+        SCReturnInt(TM_ECODE_OK);
+    }
+
+    json_object_set_new(answer, "message",
+                        json_string("No interface using bypass"));
+    SCReturnInt(TM_ECODE_FAILED);
+}
+#endif
+
+uint64_t LiveDevicePktsGet(LiveDevice *dev)
+{
+    return SC_ATOMIC_GET(dev->pkts);
+}
+
+void LiveDevicePktsIncr(LiveDevice *dev)
+{
+    (void)SC_ATOMIC_ADD(dev->pkts, 1);
+}
+
+void LiveDevicePktsAdd(LiveDevice *dev, uint64_t n)
+{
+    (void)SC_ATOMIC_ADD(dev->pkts, n);
+}
+
+void LiveDeviceDropAdd(LiveDevice *dev, uint64_t n)
+{
+    (void)SC_ATOMIC_ADD(dev->drop, n);
+}
+
+void LiveDeviceBypassedAdd(LiveDevice *dev, uint64_t n)
+{
+    (void)SC_ATOMIC_ADD(dev->bypassed, n);
+}
+
+uint64_t LiveDeviceInvalidChecksumsGet(LiveDevice *dev)
+{
+    return SC_ATOMIC_GET(dev->invalid_checksums);
+}
