@@ -22,11 +22,16 @@
  * \author Duarte Silva <duarte.silva@serializing.me>
  */
 
-#ifndef __UTIL_IP_H__
-#define __UTIL_IP_H__
+#ifndef SURICATA_UTIL_IP_H
+#define SURICATA_UTIL_IP_H
 
+#define SC_IPV4_LEN 4
+#define SC_IPV6_LEN 16
+
+bool IPv4AddressStringIsValid(const char *str);
+bool IPv6AddressStringIsValid(const char *str);
 struct in_addr *ValidateIPV4Address(const char *);
 struct in6_addr *ValidateIPV6Address(const char *);
 void MaskIPNetblock(uint8_t *, int, int);
 
-#endif /* __UTIL_IP_H__ */
+#endif /* SURICATA_UTIL_IP_H */
