@@ -21,15 +21,15 @@
  * \author Victor Julien <victor@inliniac.net>
  */
 
-#ifndef __UTIL_MAGIC_H__
-#define __UTIL_MAGIC_H__
+#ifndef SURICATA_UTIL_MAGIC_H
+#define SURICATA_UTIL_MAGIC_H
 
 #ifdef HAVE_MAGIC
-int MagicInit(void);
-void MagicDeinit(void);
-char *MagicGlobalLookup(const uint8_t *, uint32_t);
+magic_t MagicInitContext(void);
+void MagicDeinitContext(magic_t ctx);
+
 char *MagicThreadLookup(magic_t *, const uint8_t *, uint32_t);
 #endif
 void MagicRegisterTests(void);
 
-#endif /* __UTIL_MAGIC_H__ */
+#endif /* _ */
