@@ -24,8 +24,10 @@
  *
  */
 
-#ifndef __UTIL_MEMCPY_H__
-#define __UTIL_MEMCPY_H__
+#ifndef SURICATA_UTIL_MEMCPY_H
+#define SURICATA_UTIL_MEMCPY_H
+
+#include "suricata-common.h"
 
 /**
  * \internal
@@ -35,7 +37,7 @@
  * \param s   Pointer to the src string for memcpy.
  * \param len len of the string sent in s.
  */
-static inline void memcpy_tolower(uint8_t *d, uint8_t *s, uint16_t len)
+static inline void memcpy_tolower(uint8_t *d, const uint8_t *s, uint16_t len)
 {
     uint16_t i;
     for (i = 0; i < len; i++)
@@ -44,4 +46,4 @@ static inline void memcpy_tolower(uint8_t *d, uint8_t *s, uint16_t len)
     return;
 }
 
-#endif /* __UTIL_MEMCPY_H__ */
+#endif /* SURICATA_UTIL_MEMCPY_H */
