@@ -15,8 +15,8 @@
  * 02110-1301, USA.
  */
 
-#ifndef __UTIL_OPTIMIZE_H__
-#define __UTIL_OPTIMIZE_H__
+#ifndef SURICATA_UTIL_OPTIMIZE_H
+#define SURICATA_UTIL_OPTIMIZE_H
 
 /**
  * \file
@@ -24,12 +24,17 @@
  * \author Victor Julien <victor@inliniac.net>
  */
 
+#if CPPCHECK==1
+#define likely
+#define unlikely
+#else
 #ifndef likely
 #define likely(expr) __builtin_expect(!!(expr), 1)
 #endif
 #ifndef unlikely
 #define unlikely(expr) __builtin_expect(!!(expr), 0)
 #endif
+#endif
 
 /** from http://en.wikipedia.org/wiki/Memory_ordering
  *
@@ -43,5 +48,4 @@
  */
 #define hw_barrier() __sync_synchronize()
 
-#endif /* __UTIL_OPTIMIZE_H__ */
-
+#endif /* SURICATA_UTIL_OPTIMIZE_H */
