@@ -22,8 +22,8 @@
  *
  */
 
-#ifndef __UTIL_PAGES_H__
-#define __UTIL_PAGES_H__
+#ifndef SURICATA_UTIL_PAGES_H
+#define SURICATA_UTIL_PAGES_H
 
 #include "suricata-common.h"
 
@@ -31,12 +31,14 @@
     /* OpenBSD won't allow for this test:
      * "suricata(...): mprotect W^X violation" */
     #define PageSupportsRWX() 0
+    #define HAVE_PAGESUPPORTSRWX_AS_MACRO 1
 #else
     #ifndef HAVE_SYS_MMAN_H
         #define PageSupportsRWX() 1
+        #define HAVE_PAGESUPPORTSRWX_AS_MACRO 1
     #else
         int PageSupportsRWX(void);
     #endif /* HAVE_SYS_MMAN_H */
 #endif
 
-#endif /* __UTIL_PAGES_H__ */
+#endif /* SURICATA_UTIL_PAGES_H */
