@@ -21,25 +21,15 @@
  * \author Victor Julien <victor@inliniac.net>
  */
 
-#ifndef __UTIL_PROFILE_LOCKS_H__
-#define __UTIL_PROFILE_LOCKS_H__
+#ifndef SURICATA_UTIL_PROFILE_LOCKS_H
+#define SURICATA_UTIL_PROFILE_LOCKS_H
 
 #ifdef PROFILING
 
-#define PROFILING_MAX_LOCKS 64
-
-enum {
-    LOCK_MUTEX,
-    LOCK_SPIN,
-    LOCK_RWW,   /**< rwlock, writer */
-    LOCK_RWR,   /**< rwlock, reader */
-};
-
 void SCProfilingAddPacketLocks(void *);
 
-int LockRecordInitHash();
-void LockRecordFreeHash();
+int LockRecordInitHash(void);
+void LockRecordFreeHash(void);
 
 #endif /* PROFILING */
-#endif /* __UTIL_PROFILE_LOCKS_H__ */
-
+#endif /* SURICATA_UTIL_PROFILE_LOCKS_H */
