@@ -25,8 +25,11 @@
  */
 
 #include "suricata-common.h"
+#include "suricata.h"
 #include "util-random.h"
+#include "util-debug.h"
 
+#if !(defined(HAVE_WINCRYPT_H) &&  defined(OS_WIN32))
 #if defined(HAVE_CLOCK_GETTIME)
 
 static long int RandomGetClock(void)
@@ -35,12 +38,12 @@ static long int RandomGetClock(void)
     clock_gettime(CLOCK_REALTIME, &ts);
 
     // coverity[dont_call : FALSE]
-    srandom(ts.tv_nsec ^ ts.tv_sec);
+    srandom((unsigned int)(ts.tv_nsec ^ ts.tv_sec));
     long int value = random();
     return value;
 }
 
-#elif !(defined(HAVE_WINCRYPT_H) &&  defined(OS_WIN32))
+#else
 
 static long int RandomGetPosix(void)
 {
@@ -55,6 +58,7 @@ static long int RandomGetPosix(void)
 }
 
 #endif
+#endif /* !(defined(HAVE_WINCRYPT_H) &&  defined(OS_WIN32)) */
 
 #if defined(HAVE_WINCRYPT_H) && defined(OS_WIN32)
 #include <wincrypt.h>
@@ -65,9 +69,21 @@ long int RandomGet(void)
         return 0;
 
     HCRYPTPROV p;
-    if (!(CryptAcquireContext(&p, NULL, NULL,
-                PROV_RSA_FULL, 0))) {
-        return -1;
+    if (!CryptAcquireContext(&p, NULL, NULL, PROV_RSA_FULL, 0)) {
+        DWORD err = GetLastError();
+        SCLogDebug("CryptAcquireContext error: %" PRIu32, (uint32_t)err);
+        if (err == (DWORD)NTE_BAD_KEYSET) {
+            /* The key doesn't exist yet, create it */
+            if (!CryptAcquireContext(&p, NULL, NULL, PROV_RSA_FULL,
+                                     CRYPT_NEWKEYSET)) {
+
+                SCLogDebug("CryptAcquireContext error: %" PRIu32,
+                           (uint32_t)err);
+                return -1;
+            }
+        } else {
+            return -1;
+        }
     }
 
     long int value = 0;
@@ -87,7 +103,7 @@ long int RandomGet(void)
         return 0;
 
     long int value = 0;
-    int ret = getrandom(&value, sizeof(value), 0);
+    ssize_t ret = getrandom(&value, sizeof(value), 0);
     /* ret should be sizeof(value), but if it is > 0 and < sizeof(value)
      * it's still better than nothing so we return what we have */
     if (ret <= 0) {
