@@ -20,7 +20,7 @@
  *
  * \author Pablo Rincon Crespo <pablo.rincon.crespo@gmail.com>
  *
- * Bs2Bm use a simple context array to determine the charactes
+ * Bs2Bm use a simple context array to determine the characters
  * that are not present on the pattern. This way on partial matches
  * broken by a char not present, we can skip to the next character
  * making less checks
@@ -34,7 +34,7 @@
 /**
  * \brief Array setup function for Bs2Bm of bad characters index (not found at the needle)
  *
- * \param neddle pointer to the pattern we ar searching for
+ * \param needle pointer to the pattern we ar searching for
  * \param needle_len length limit of the needle
  * \param badchars pointer to an empty array of bachars. The array prepared contains
  *                 characters that can't be inside the needle_len. So the skips can be
@@ -53,30 +53,6 @@ void Bs2BmBadchars(const uint8_t *needle, uint16_t needle_len, uint8_t *badchars
         badchars[needle[i]] = 0;
 }
 
-/**
- * \brief Array setup function for Bs2BmNocase of bad characters index (not found at the needle)
- *
- * \param neddle pointer to the pattern we ar searching for
- * \param needle_len length limit of the needle
- * \param badchars pointer to an empty array of bachars. The array prepared contains
- *                 characters that can't be inside the needle_len. So the skips can be
- *                 faster
- */
-void Bs2BmBadcharsNocase(const uint8_t *needle, uint16_t needle_len, uint8_t *badchars)
-{
-    uint32_t i;
-    for (i = 0; i < ALPHABET_SIZE; i++)
-        badchars[i] = 1;
-
-    /* set to 0 the values where index as ascii is present
-     * because they are not badchars
-     */
-    for (i = 0; i < needle_len; i++) {
-        badchars[u8_tolower(needle[i])] = 0;
-    }
-}
-
-
 /**
  * \brief Basic search with a bad characters array. The array badchars contains
  *        flags at character's ascii index that can't be inside the needle. So the skips can be
@@ -84,13 +60,14 @@ void Bs2BmBadcharsNocase(const uint8_t *needle, uint16_t needle_len, uint8_t *ba
  *
  * \param haystack pointer to the buffer to search in
  * \param haystack_len length limit of the buffer
- * \param neddle pointer to the pattern we ar searching for
+ * \param needle pointer to the pattern we ar searching for
  * \param needle_len length limit of the needle
  * \param badchars pointer to an array of bachars prepared by Bs2BmBadchars()
  *
  * \retval ptr to start of the match; NULL if no match
  */
-uint8_t * Bs2Bm(const uint8_t *haystack, uint32_t haystack_len, const uint8_t *needle, uint16_t needle_len, uint8_t badchars[])
+uint8_t *Bs2Bm(const uint8_t *haystack, uint32_t haystack_len, const uint8_t *needle,
+        uint16_t needle_len, const uint8_t badchars[])
 {
     const uint8_t *h, *n;
     const uint8_t *hmax = haystack + haystack_len;
@@ -133,13 +110,14 @@ uint8_t * Bs2Bm(const uint8_t *haystack, uint32_t haystack_len, const uint8_t *n
  *
  * \param haystack pointer to the buffer to search in
  * \param haystack_len length limit of the buffer
- * \param neddle pointer to the pattern we ar searching for
+ * \param needle pointer to the pattern we ar searching for
  * \param needle_len length limit of the needle
  * \param badchars pointer to an array of bachars prepared by Bs2BmBadchars()
  *
  * \retval ptr to start of the match; NULL if no match
  */
-uint8_t *Bs2BmNocase(const uint8_t *haystack, uint32_t haystack_len, const uint8_t *needle, uint16_t needle_len, uint8_t badchars[])
+uint8_t *Bs2BmNocase(const uint8_t *haystack, uint32_t haystack_len, const uint8_t *needle,
+        uint16_t needle_len, const uint8_t badchars[])
 {
     const uint8_t *h, *n;
     const uint8_t *hmax = haystack + haystack_len;
