@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2016 Open Information Security Foundation
+/* Copyright (C) 2007-2020 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -50,18 +50,28 @@
  * would be considered timed out.
  */
 
+#ifdef OS_WIN32
+/* for MinGW we need to set _POSIX_C_SOURCE before including
+ * sys/time.h. */
+#ifndef _POSIX_C_SOURCE
+#define _POSIX_C_SOURCE 200809L
+#endif
+#endif
+
 #include "suricata-common.h"
+#include "suricata.h"
 #include "detect.h"
 #include "threads.h"
 #include "tm-threads.h"
 #include "util-debug.h"
+#include "util-time.h"
 
 #ifdef UNITTESTS
 static struct timeval current_time = { 0, 0 };
 #endif
 //static SCMutex current_time_mutex = SCMUTEX_INITIALIZER;
 static SCSpinlock current_time_spinlock;
-static char live = TRUE;
+static bool live_time_tracking = true;
 
 struct tm *SCLocalTime(time_t timep, struct tm *result);
 struct tm *SCUtcTime(time_t timep, struct tm *result);
@@ -79,43 +89,46 @@ void TimeDeinit(void)
     SCSpinDestroy(&current_time_spinlock);
 }
 
+bool TimeModeIsReady(void)
+{
+    if (live_time_tracking)
+        return true;
+    return TmThreadsTimeSubsysIsReady();
+}
+
 void TimeModeSetLive(void)
 {
-    live = TRUE;
+    live_time_tracking = true;
     SCLogDebug("live time mode enabled");
 }
 
 void TimeModeSetOffline (void)
 {
-    live = FALSE;
+    live_time_tracking = false;
     SCLogDebug("offline time mode enabled");
 }
 
-int TimeModeIsLive(void)
+bool TimeModeIsLive(void)
 {
-    return live;
+    return live_time_tracking;
 }
 
-void TimeSetByThread(const int thread_id, const struct timeval *tv)
+void TimeSetByThread(const int thread_id, SCTime_t tv)
 {
-    if (live == TRUE)
+    if (live_time_tracking)
         return;
 
     TmThreadsSetThreadTimestamp(thread_id, tv);
 }
 
 #ifdef UNITTESTS
-void TimeSet(struct timeval *tv)
+void TimeSet(SCTime_t ts)
 {
-    if (live == TRUE)
-        return;
-
-    if (tv == NULL)
+    if (live_time_tracking)
         return;
 
     SCSpinLock(&current_time_spinlock);
-    current_time.tv_sec = tv->tv_sec;
-    current_time.tv_usec = tv->tv_usec;
+    SCTIME_TO_TIMEVAL(&current_time, ts);
 
     SCLogDebug("time set to %" PRIuMAX " sec, %" PRIuMAX " usec",
                (uintmax_t)current_time.tv_sec, (uintmax_t)current_time.tv_usec);
@@ -131,34 +144,34 @@ void TimeSetToCurrentTime(void)
 
     gettimeofday(&tv, NULL);
 
-    TimeSet(&tv);
+    SCTime_t ts = SCTIME_FROM_TIMEVAL(&tv);
+    TimeSet(ts);
 }
 #endif
 
-void TimeGet(struct timeval *tv)
+SCTime_t TimeGet(void)
 {
-    if (tv == NULL)
-        return;
-
-    if (live == TRUE) {
-        gettimeofday(tv, NULL);
+    struct timeval tv = { 0 };
+    if (live_time_tracking) {
+        gettimeofday(&tv, NULL);
     } else {
 #ifdef UNITTESTS
         if (unlikely(RunmodeIsUnittests())) {
             SCSpinLock(&current_time_spinlock);
-            tv->tv_sec = current_time.tv_sec;
-            tv->tv_usec = current_time.tv_usec;
+            tv.tv_sec = current_time.tv_sec;
+            tv.tv_usec = current_time.tv_usec;
             SCSpinUnlock(&current_time_spinlock);
         } else {
 #endif
-            TmreadsGetMinimalTimestamp(tv);
+            TmThreadsGetMinimalTimestamp(&tv);
 #ifdef UNITTESTS
         }
 #endif
     }
 
-    SCLogDebug("time we got is %" PRIuMAX " sec, %" PRIuMAX " usec",
-               (uintmax_t)tv->tv_sec, (uintmax_t)tv->tv_usec);
+    SCLogDebug("time we got is %" PRIuMAX " sec, %" PRIuMAX " usec", (uintmax_t)tv.tv_sec,
+            (uintmax_t)tv.tv_usec);
+    return SCTIME_FROM_TIMEVAL(&tv);
 }
 
 #ifdef UNITTESTS
@@ -166,43 +179,65 @@ void TimeGet(struct timeval *tv)
  *  \param tv_sec seconds to increment the time with */
 void TimeSetIncrementTime(uint32_t tv_sec)
 {
-    struct timeval tv;
-    memset(&tv, 0x00, sizeof(tv));
-    TimeGet(&tv);
+    SCTime_t ts = TimeGet();
 
-    tv.tv_sec += tv_sec;
+    ts = SCTIME_ADD_SECS(ts, tv_sec);
 
-    TimeSet(&tv);
+    TimeSet(ts);
 }
 #endif
 
-void CreateIsoTimeString (const struct timeval *ts, char *str, size_t size)
+#ifdef OS_WIN32
+/** \internal
+ *  \brief wrapper around strftime on Windows to provide output
+ *         compatible with posix %z
+ */
+static inline void WinStrftime(const SCTime_t ts, const struct tm *t, char *str, size_t size)
 {
-    time_t time = ts->tv_sec;
+    char time_fmt[64] = { 0 };
+    char tz[6] = { 0 };
+    const long int tzdiff = -_timezone;
+    const int h = abs(_timezone) / 3600 + _daylight;
+    const int m = (abs(_timezone) % 3600) / 60;
+    snprintf(tz, sizeof(tz), "%c%02d%02d", tzdiff < 0 ? '-' : '+', h, m);
+    strftime(time_fmt, sizeof(time_fmt), "%Y-%m-%dT%H:%M:%S.%%06u", t);
+    snprintf(str, size, time_fmt, SCTIME_USECS(ts));
+    strlcat(str, tz, size); // append our timezone
+}
+#endif
+
+void CreateIsoTimeString(const SCTime_t ts, char *str, size_t size)
+{
+    time_t time = SCTIME_SECS(ts);
     struct tm local_tm;
     memset(&local_tm, 0, sizeof(local_tm));
     struct tm *t = (struct tm*)SCLocalTime(time, &local_tm);
-    char time_fmt[64] = { 0 };
 
     if (likely(t != NULL)) {
-        strftime(time_fmt, sizeof(time_fmt), "%Y-%m-%dT%H:%M:%S.%%06u%z", t);
-        snprintf(str, size, time_fmt, ts->tv_usec);
+#ifdef OS_WIN32
+        WinStrftime(ts, t, str, size);
+#else
+        char time_fmt[64] = { 0 };
+        int64_t usec = SCTIME_USECS(ts);
+        strftime(time_fmt, sizeof(time_fmt), "%Y-%m-%dT%H:%M:%S.%%06" PRIi64 "%z", t);
+        snprintf(str, size, time_fmt, usec);
+#endif
     } else {
         snprintf(str, size, "ts-error");
     }
 }
 
-void CreateUtcIsoTimeString (const struct timeval *ts, char *str, size_t size)
+void CreateUtcIsoTimeString(const SCTime_t ts, char *str, size_t size)
 {
-    time_t time = ts->tv_sec;
+    time_t time = SCTIME_SECS(ts);
     struct tm local_tm;
     memset(&local_tm, 0, sizeof(local_tm));
     struct tm *t = (struct tm*)SCUtcTime(time, &local_tm);
-    char time_fmt[64] = { 0 };
 
     if (likely(t != NULL)) {
+        char time_fmt[64] = { 0 };
         strftime(time_fmt, sizeof(time_fmt), "%Y-%m-%dT%H:%M:%S", t);
-        snprintf(str, size, time_fmt, ts->tv_usec);
+        snprintf(str, size, time_fmt, SCTIME_USECS(ts));
     } else {
         snprintf(str, size, "ts-error");
     }
@@ -210,7 +245,7 @@ void CreateUtcIsoTimeString (const struct timeval *ts, char *str, size_t size)
 
 void CreateFormattedTimeString (const struct tm *t, const char *fmt, char *str, size_t size)
 {
-    if (likely(t != NULL && fmt != NULL && str != NULL)) {
+    if (likely(t != NULL)) {
         strftime(str, size, fmt, t);
     } else {
         snprintf(str, size, "ts-error");
@@ -227,23 +262,22 @@ struct tm *SCUtcTime(time_t timep, struct tm *result)
  */
 
 #ifndef TLS
-/* OpenBSD does not support __thread, so don't use time caching on BSD
+/* OpenBSD does not support thread_local, so don't use time caching on BSD
  */
 struct tm *SCLocalTime(time_t timep, struct tm *result)
 {
     return localtime_r(&timep, result);
 }
 
-void CreateTimeString (const struct timeval *ts, char *str, size_t size)
+void CreateTimeString(const SCTime_t ts, char *str, size_t size)
 {
-    time_t time = ts->tv_sec;
+    time_t time = SCTIME_SECS(ts);
     struct tm local_tm;
     struct tm *t = (struct tm*)SCLocalTime(time, &local_tm);
 
     if (likely(t != NULL)) {
-        snprintf(str, size, "%02d/%02d/%02d-%02d:%02d:%02d.%06u",
-                t->tm_mon + 1, t->tm_mday, t->tm_year + 1900, t->tm_hour,
-                t->tm_min, t->tm_sec, (uint32_t) ts->tv_usec);
+        snprintf(str, size, "%02d/%02d/%02d-%02d:%02d:%02d.%06u", t->tm_mon + 1, t->tm_mday,
+                t->tm_year + 1900, t->tm_hour, t->tm_min, t->tm_sec, (uint32_t)SCTIME_USECS(ts));
     } else {
         snprintf(str, size, "ts-error");
     }
@@ -251,7 +285,7 @@ void CreateTimeString (const struct timeval *ts, char *str, size_t size)
 
 #else
 
-/* On systems supporting __thread, use Per-thread values for caching
+/* On systems supporting thread_local, use Per-thread values for caching
  * in CreateTimeString */
 
 /* The maximum possible length of the time string.
@@ -259,16 +293,16 @@ void CreateTimeString (const struct timeval *ts, char *str, size_t size)
  * Or "01/01/2013-15:42:21.123456", which is 26, so round up to 32. */
 #define MAX_LOCAL_TIME_STRING 32
 
-static __thread int mru_time_slot; /* Most recently used cached value */
-static __thread time_t last_local_time[2];
-static __thread short int cached_local_time_len[2];
-static __thread char cached_local_time[2][MAX_LOCAL_TIME_STRING];
+static thread_local int mru_time_slot; /* Most recently used cached value */
+static thread_local time_t last_local_time[2];
+static thread_local short int cached_local_time_len[2];
+static thread_local char cached_local_time[2][MAX_LOCAL_TIME_STRING];
 
 /* Per-thread values for caching SCLocalTime() These cached values are
  * independent from the CreateTimeString cached values. */
-static __thread int mru_tm_slot; /* Most recently used local tm */
-static __thread time_t cached_minute_start[2];
-static __thread struct tm cached_local_tm[2];
+static thread_local int mru_tm_slot; /* Most recently used local tm */
+static thread_local time_t cached_minute_start[2];
+static thread_local struct tm cached_local_tm[2];
 
 /** \brief Convert time_t into Year, month, day, hour and minutes.
  * \param timep Time in seconds since defined date.
@@ -276,7 +310,7 @@ static __thread struct tm cached_local_tm[2];
  *
  * To convert a time in seconds into year, month, day, hours, minutes
  * and seconds, call localtime_r(), which uses the current time zone
- * to compute these values. Note, glibc's localtime_r() aquires a lock
+ * to compute these values. Note, glibc's localtime_r() acquires a lock
  * each time it is called, which limits parallelism. To call
  * localtime_r() less often, the values returned are cached for the
  * current and previous minute and then seconds are adjusted to
@@ -352,12 +386,12 @@ static int UpdateCachedTime(int n, time_t time)
 /** \brief Return a formatted string for the provided time.
  *
  * Cache the Month/Day/Year - Hours:Min part of the time string for
- * the current minute. Copy that result into the the return string and
+ * the current minute. Copy that result into the return string and
  * then only print the seconds for each call.
  */
-void CreateTimeString (const struct timeval *ts, char *str, size_t size)
+void CreateTimeString(const SCTime_t ts, char *str, size_t size)
 {
-    time_t time = ts->tv_sec;
+    time_t time = SCTIME_SECS(ts);
     int seconds;
 
     /* Only get a new local time when the time crosses into a new
@@ -390,9 +424,7 @@ void CreateTimeString (const struct timeval *ts, char *str, size_t size)
     if (cached_len >= (int)size)
       cached_len = size;
     memcpy(str, cached_str, cached_len);
-    snprintf(str + cached_len, size - cached_len,
-             "%02d.%06u",
-             seconds, (uint32_t) ts->tv_usec);
+    snprintf(str + cached_len, size - cached_len, "%02d.%06u", seconds, (uint32_t)SCTIME_USECS(ts));
 }
 
 #endif /* defined(__OpenBSD__) */
@@ -517,7 +549,7 @@ int SCTimeToStringPattern (time_t epoch, const char *pattern, char *str, size_t
         return 1;
     }
 
-    int r = strftime(buffer, sizeof(buffer), pattern, tp);
+    size_t r = strftime(buffer, sizeof(buffer), pattern, tp);
     if (r == 0) {
         return 1;
     }
@@ -608,3 +640,8 @@ uint64_t SCTimespecAsEpochMillis(const struct timespec* ts)
 {
     return ts->tv_sec * 1000L + ts->tv_nsec / 1000000L;
 }
+
+uint64_t TimeDifferenceMicros(struct timeval t0, struct timeval t1)
+{
+    return (uint64_t)(t1.tv_sec - t0.tv_sec) * 1000000L + (t1.tv_usec - t1.tv_usec);
+}
