@@ -21,10 +21,25 @@
  * \author Pablo Rincon Crespo <pablo.rincon.crespo@gmail.com>
  */
 
-#ifndef __UTIL_UNITTEST_HELPER__
-#define __UTIL_UNITTEST_HELPER__
+#ifndef SURICATA_UTIL_UNITTEST_HELPER
+#define SURICATA_UTIL_UNITTEST_HELPER
 
+#if defined(UNITTESTS)
+#include "flow.h"
+#include "detect.h"
+#elif defined(FUZZ)
+#include "flow.h"
+#endif
+
+#if defined(UNITTESTS) || defined(FUZZ)
+Flow *TestHelperBuildFlow(int family, const char *src, const char *dst, Port sp, Port dp);
+int TestHelperBufferToFile(const char *name, const uint8_t *data, size_t size);
+#endif
 #ifdef UNITTESTS
+void UTHSetIPV4Hdr(Packet *p, IPV4Hdr *ip4h);
+void UTHSetIPV6Hdr(Packet *p, IPV6Hdr *ip6h);
+void UTHSetTCPHdr(Packet *p, TCPHdr *tcph);
+
 uint32_t UTHSetIPv4Address(const char *);
 
 Packet *UTHBuildPacketReal(uint8_t *, uint16_t, uint8_t ipproto, const char *, const char *, uint16_t, uint16_t);
@@ -35,7 +50,6 @@ Packet *UTHBuildPacketSrcDstPorts(uint8_t *, uint16_t, uint8_t ipproto, uint16_t
 Packet *UTHBuildPacketIPV6SrcDst(uint8_t *, uint16_t, uint8_t ipproto, const char *, const char *);
 
 int UTHPacketMatchSigMpm(Packet *, char *, uint16_t);
-Packet **UTHBuildPacketArrayFromEth(uint8_t **, int *, int);
 Packet *UTHBuildPacketFromEth(uint8_t *, uint16_t);
 
 void UTHFreePacket(Packet *);
@@ -60,8 +74,11 @@ int UTHGenericTest(Packet **, int, const char **, uint32_t *, uint32_t *, int);
 uint32_t UTHBuildPacketOfFlows(uint32_t, uint32_t, uint8_t);
 Packet *UTHBuildPacketIPV6Real(uint8_t *, uint16_t , uint8_t ipproto, const char *, const char *,
                            uint16_t , uint16_t );
+
+void * UTHmemsearch(const void *big, size_t big_len, const void *little, size_t little_len);
+int UTHParseSignature(const char *str, bool expect);
 #endif
 
 void UTHRegisterTests(void);
 
-#endif /* __UTIL_UNITTEST_HELPER__ */
+#endif /* SURICATA_UTIL_UNITTEST_HELPER */
