@@ -53,7 +53,7 @@
 
 #include "suricata-common.h"
 #include "util-debug.h"
-#include "util-device.h"
+#include "util-device-private.h"
 #include "util-mem.h"
 #include "util-unittest.h"
 
